// Copyright (C) 1999 Fredrik hrstrm
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#include"string.hh"
#include"map"

class TextModel
{
        public:

        virtual void addBlock (string text) = 0;
        virtual string getBlock (int n) = 0;
        virtual void setBlock (int n, string text) = 0;
        virtual void insertBlock (int n, string text) = 0;
        virtual void clear () = 0;
        virtual void fromString (string s) = 0;
        virtual string asString (string rowsep) = 0;
        virtual int nofBlocks () = 0;
};

typedef TextModel *TextModel_ptr;

class TextModel_i : public virtual TextModel
{
        public:

        TextModel_i ()
        {
                nof_blocks = 0;
        };
        
        string getBlock (int n)
        {
                if (nof_blocks<n) return Latin1("");
                return blocks[n];
        }
        
        void setBlock (int n, string text)
        {
                if (nof_blocks<n) return;
                blocks[n]=text;
        }

        void addBlock (string text)
        {
                blocks[nof_blocks++]=text;
        }

        int nofBlocks ()
        {
                return nof_blocks;
        }
        
        
        void insertBlock (int n, string text)
        {
                map<int,string> newblocks;
                map<int,string>::iterator i;
                for (i=blocks.begin (); i!=blocks.end(); ++i)
                {
                        if ((*i).first>=n)
                                newblocks[(*i).first+1] = (*i).second;
                        else
                                newblocks[(*i).first] = (*i).second;
                }
                newblocks[n] = text;
                nof_blocks++;
                
                blocks = newblocks;
        }

        
        void clear ()
        {
                blocks.clear ();
                nof_blocks = 0;
        }

        string asString (string rowsep)
        {
                string res;
                map<int,string>::iterator i;
                for (i=blocks.begin (); i!=blocks.end(); ++i)
                {
                        res = res+(*i).second + rowsep;
                }
                return res;
        }

        void fromString (string s)
        {
                int start=0, stop;

                clear ();
                while (start<s.size())
                {
                        stop = s.findSubstring (start, Latin1("\n"));
                        if (stop<start) stop = s.size();
                        addBlock (s.substring (start, stop));
                        start = stop+1;
                }
        }
        
        private:
        
        map<int,string> blocks;
        int nof_blocks;
};

class BasicModelFactory
{
        public:
        
        virtual TextModel_ptr newTextModel () = 0;
};

class BasicModelFactory_i
{
        public:
        
        static TextModel_ptr newTextModel ()
        {
                return new TextModel_i;
        }
};





