// Copyright (C) 1999 Fredrik hrstrm
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#include "Component_Implementations.hh"
extern "C" {        
#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <pthread.h>
#include <semaphore.h>
}

using namespace Component_Implementations;

Component_Interfaces::Resources *Component_Implementations::resources = NULL;
char *Component_Implementations::resource_filename = NULL;

void Component_Implementations::useResourceFile (char *name)
{
        resource_filename = name;
        resources = new Component_Implementations::Resources_i (0, Latin1("Resource"),
                                                                Latin1(resource_filename));
}

Component_Implementations::ClassType::ClassType (string t)
                : type_ (t)
{
}

Component_Implementations::Component_i::Component_i (Component *composite,
                                                     string name)
                :ClassType (Latin1("Component")), composite_ (composite), name_ (name)
{
        if (composite_)
        {
                path_ = composite->path()+Latin1("/")+name_+Latin1("(")+type_+Latin1(")");
                composite_->addChild (this);
        }
        else
        {
                path_ = Latin1("/")+name_+Latin1("(")+type_+Latin1(")");
        }
        nameandtype_ = name+Latin1("(")+type_+Latin1(")");
}

// Necessary even though it is defined as abstract virtual!
// G++ feature, go figure.
Component_Interfaces::Component::~Component ()
{
}

Component_Implementations::Component_i::~Component_i ()
{
}

Component_ptr Component_Implementations::Component_i::cleanup ()
{
        list<Component_ptr>::iterator i;
        for (i=components_.begin(); i!=components_.end();)
        {
                Component_ptr c = *i;
                components_.erase (i);
                c->cleanup()->_dispose ();
        }

        if (composite_!=NULL)
                composite_->delChild (this);

        if (factory_!=NULL)
                factory_->disposeComponent (this);
        
        return this;
}

void Component_Implementations::Component_i::_dispose ()
{
        delete this;
}

void Component_Implementations::Component_i::addChild (Component_ptr c)
{
        //        fprintf (stderr, "Adding child %s to %s\n", c->path(), path());
        components_.insert (components_.end (), c);
}

void Component_Implementations::Component_i::delChild (Component_ptr c)
{
        //        fprintf (stderr, "Deleting child %s from %s\n", c->path (), path());
        list<Component_ptr>::iterator i;
        for (i=components_.begin(); i!=components_.end(); ++i)
        {
                if (*i==c)
                {                        
                        components_.erase (i);
                        return;
                }
        }        
}

Component_ptr Component_Implementations::Component_i::composite ()
{
        return composite_;                        
}

list<Component_ptr> Component_Implementations::Component_i::components()
{
        return components_;                        
}
                
string Component_Implementations::Component_i::path ()
{
        return path_;

        /*
        char *path = new char[256];
        UniToUtf (path_, path);

        return path;
        */
        //        return CORBA::string_dup (path);
}

string Component_Implementations::Component_i::nameAndType ()
{
        return nameandtype_;

        /*
        char *path = new char[256];
        UniToUtf (nameandtype_, path);

        return path;
        */
        //        return CORBA::string_dup (path);
}

string Component_Implementations::Component_i::name ()
{
        return name_;
}

void
Component_Implementations::Component_i::updateResources ()
{}

Factory_ptr
Component_Implementations::Component_i::factory ()
{
        return factory_;
}

void Component_Implementations::Component_i::printTree ()
{
        path_.print (stderr, "PrintTree: ","\n");
        list<Component_ptr>::iterator i;
        for (i = components_.begin(); i!=components_.end(); ++i)
        {
                if (*i==NULL)
                        fprintf (stderr, "URK NULL CHILD!\n");
                else
                        (*i)->printTree ();
        }
}

// SelectController ///////////////////////////////////////////////////////////

Component_Implementations::SelectController *gthing = 0;

Component_Implementations::SelectController::SelectController ()
                : Component_i (0,Latin1("SelectController")), ClassType (Latin1("SelectController"))
{
        for (int i=0; i<64; ++i)
        {
                FD_CLR (i,&read_fds_);
                FD_CLR (i,&write_fds_);
                FD_CLR (i,&exception_fds_);
        }
        
        int r = pipe (fildes);
        if (r) 
        {
                cerr << "Could not create pipe for thread." << endl;
                exit (-1);
        }
        FD_SET (fildes[1],&read_fds_);
        read_fd_ [fildes[1]] = 0;
        gthing = this;

        thread_.start (call (this, &SelectController::run));        
}

Component_Implementations::SelectController::~SelectController ()
{
}

void 
Component_Implementations::SelectController::addConnection (DataReceiver *dr,
                                       int n)
{
        if (dr)
        {
                FD_SET (n,&read_fds_);
                read_fd_ [n] = dr;
        }
        else
        {
                FD_CLR (n,&read_fds_);
                read_fd_ [n] = 0;
        }
        shake ();
}

void
Component_Implementations::SelectController::shake ()
{
        char ch = 'A';
        cerr << "SHAKING" << endl;        
        write (fildes[0],&ch,1);
}

void 
Component_Implementations::SelectController::run (Component_Implementations::Callback_i *cb)
{
        cerr << "Started selectcontroller running!" << endl;
        
        buffer b;
        int seconds = 1, useconds = 0;
        int n;
        char ch;
        timeval timeout;
        fd_set rfds, wfds, efds;
        
        timeout.tv_sec = seconds;
        timeout.tv_usec = useconds;

        for (;;)
        {
                bcopy ((char*)&read_fds_, (char*)&rfds, sizeof(read_fds_));
                bcopy ((char*)&write_fds_, (char*)&wfds, sizeof(write_fds_));
                bcopy ((char*)&exception_fds_, (char*)&efds, sizeof(exception_fds_));
                
                n = select (64, &rfds, &wfds, &efds, NULL);
                if (!thread_.running ()) break;
                // n = select (64, &rfds, &wfds, &efds, &timeout);

                if (n)
                {
                        for (int i = 0; i<64; i++)
                        {
                                if (FD_ISSET (i,&rfds))
                                {
                                        if (read_fd_ [i]) read_fd_ [i]->receive (b);
                                        else
                                        {
                                                // I was just woken up so I could
                                                // reread the table.
                                                read (fildes[1],&ch, 1);
                                                cerr << "Awake, reload... CH=" <<
                                                ch << " AT " << i << endl;
                                        }
                                }
                                if (FD_ISSET (i,&wfds))
                                {
                                        // write_fdcb_[i]->invoke ();
                                }
                                if (FD_ISSET (i,&efds))
                                {
                                        // exception_fdcb_[i]->invoke ();
                                }
                        }
                }
        }        
}

// Mutex //////////////////////////////////////////////////////////////////////

Component_Implementations::Mutex::Mutex (string name)
{
    pthread_mutex_init (&mutex_, NULL);
    name_ = name;
}

void
Component_Implementations::Mutex::lock (char *p)
{
    if (p) cerr << "LOCK " << name_ << " AT " << p << endl;
    pthread_mutex_lock (&mutex_);    
}

int
Component_Implementations::Mutex::trylock (char *p)
{
    if (p) cerr << "TRYLOCK " << name_ << " AT " << p << endl;
    return pthread_mutex_trylock (&mutex_);    
}

void
Component_Implementations::Mutex::unlock (char *p)
{
    if (p) cerr << "UNLOCK " << name_ << " AT " << p << endl;
    pthread_mutex_unlock (&mutex_);
}
 
// RecursiveMutex /////////////////////////////////////////////////////////////

Component_Implementations::RecursiveMutex::RecursiveMutex (string name) 
    : structure_lock_ (name),
      lock_ (name)
{    
    owner_ = 0;
    nof_locks_ = 0;
    name_ = name;
}

void
Component_Implementations::RecursiveMutex::lock (char *p)
{        
    if (p) cerr << "RECLOCK " << name_ << " AT " << p << endl;
    structure_lock_.lock ();

    if (owner_ == pthread_self()) 
    { 
        nof_locks_++;
	structure_lock_.unlock ();
#ifdef DEBUG_THREADS	
	cerr << "Thread " << Thread::getThread(pthread_self ())->name() 
	     << " got lock " << name_ << " again " 
	     << nof_locks_ << endl;
#endif
    }
    else 
    {
	if (lock_.trylock()!=0)
	{
	    // Did not get lock.
	    structure_lock_.unlock ();
#ifdef DEBUG_THREADS
	    cerr << "Thread " << Thread::getThread(pthread_self ())->name() 
		 << " did not get lock " << name_ 
		 << " waiting." << endl;
#endif
	    lock_.lock ();
	    structure_lock_.lock ();
	}
#ifdef DEBUG_THREADS
	cerr << "Thread " << Thread::getThread(pthread_self ())->name() 
	     << " got lock " << name_ << " first time." 
	     << endl;
#endif
	owner_ = pthread_self ();
	nof_locks_ = 1;
	structure_lock_.unlock ();	    
    }
}

void
Component_Implementations::RecursiveMutex::unlock (char *p)
{
    if (p) cerr << "RECUNLOCK " << name_ << " AT " << p << endl;
    structure_lock_.lock ();
    if (owner_ == pthread_self() && --nof_locks_ == 0) 
    {
#ifdef DEBUG_THREADS
	cerr << "Thread " << Thread::getThread(pthread_self ())->name() 
	     << " unlocks " << name_ << "." << endl;
#endif
	lock_.unlock ();
	owner_ = 0;
    }
    else
    {
#ifdef DEBUG_THREADS
	cerr << "Thread " << Thread::getThread(pthread_self ())->name() 
	     << " rewinds lock " << name_ << "." << endl;
#endif
    }

    structure_lock_.unlock ();
}


// Semaphore //////////////////////////////////////////////////////////////////

Component_Implementations::Semaphore::Semaphore (string name)
{
    sem_init (&semaphore_, 0, 0);
    name_ = name;
}

Component_Implementations::Semaphore::~Semaphore ()
{
    sem_destroy (&semaphore_);
}

void
Component_Implementations::Semaphore::post (char *p)
{
    if (p) cerr << "POST " << name_ << " AT " << p << endl;
    sem_post (&semaphore_);
}

void
Component_Implementations::Semaphore::wait (char *p)
{
    if (p) cerr << "WAIT " << name_ << " AT " << p << endl;
    sem_wait (&semaphore_);
}

int Component_Implementations::Semaphore::value ()
{
        int v;        
        sem_getvalue (&semaphore_, &v);
        return v;
}

// Thread /////////////////////////////////////////////////////////////////////

Component_Implementations::Thread *Thread::all_threads_[65536];
int Component_Implementations::Thread::nof_threads_=0;
Component_Implementations::Mutex Component_Implementations::Thread::protect_thread_info_(Latin1("ThreadInfo"));

Component_Implementations::Thread::Thread (Component_i *p, string name)
                : Component_i (p, name), ClassType (Latin1("Thread")),
                  pid_ (0),
                  running_ (false)
{
}

Component_Implementations::Thread::~Thread () 
{
    stop ();
}

void 
Component_Implementations::Thread::start (Callback *cb)
{
        if (running_) return;
        running_ = true;
        called_ = cb;		
        pthread_create (&thread_, NULL, Thread::start_thread, this); 
}

void 
Component_Implementations::Thread::restart ()
{
        if (running_ || pid_) return;
        if (!called_) return;
        running_ = true;
        pthread_create (&thread_, NULL, Thread::start_thread, this);
}

void
Component_Implementations::Thread::stop ()
{
        running_ = false;		 // Thread must check with running ().
        // Joing only if stop is called by a different thread.
        if (getpid()!=pid_) pthread_join (thread_,NULL); 
}

void *
Component_Implementations::Thread::start_thread (Thread *thread)
{
        thread->pid_ = getpid ();              // Store pid in thread object.
        protect_thread_info_.lock ();
        all_threads_[thread->thread_] = thread;
        nof_threads_++;
        protect_thread_info_.unlock ();

        /*	printf ("Starting thread %s with pid %d\n",
                thread->path(), thread->thread_); */
        
        thread->called_->invoke ();	           // Invoke the given callback.
        
        thread->pid_ = 0;                      // Finished running, take away pid.
        protect_thread_info_.lock ();
        all_threads_[thread->thread_] = 0;
        nof_threads_--;
        protect_thread_info_.unlock ();
        
        thread->running_ = false;
        return 0;
}

Component_Implementations::Thread*
Component_Implementations::Thread::getThread (int n)
{
	cerr << "Returning thread " << n << endl;
        return all_threads_ [n];
}    

int
Component_Implementations::Thread::nofThreads ()
{
        return nof_threads_;
}

void
Component_Implementations::Thread::setMainThread ()
{
        all_threads_[pthread_self()] = new Thread (0,Latin1("main"));
        nof_threads_++;
}


// Resource_i ////////////////////////////////////////////////////////


string removePrepath (string m)
{
        if (m.size()<2) return Latin1("");
        int sp = m.findSubstring (0, Latin1("/"));        
        int pp = m.findSubstring (1, Latin1("("));
        if (sp<0 && pp<0) return Latin1("");
        if (sp>0 && (sp<pp || pp<0))
                return m.substring (sp+1, m.size());
        return m.substring (pp, m.size());
}

multimap<int, ResourceRow*>
Component_Implementations::Resources_i::findResources (string match)
{
        multimap<int,ResourceRow*> results;
        multimap<string,ResourceRow*>::iterator i = table.find (match);
        string m = match;
        
        while (i==table.end())
        {
                m = removePrepath (m);
                if (m.size()==0)
                {
                        match.print (stderr, "Warning resource ", " requested does not exist.\n");
                        return results;
                }
                i = table.find (m);
        }
        int mpos = match.size()-m.size();
        
        for (; i!=table.end() && (*i).first==m; ++i)
        {
                //                (*i).first.print (stderr, "Looking at >","<");
                string mm = (*i).second->matchmiddle;
                //                mm.print (stderr, "    mm >", "< ");
                if (mm.size ()==0)
                {
                        //                        fprintf (stderr, "dadded!\n");                        
                        results.insert (pair<int,ResourceRow*>(-1, (*i).second));
                        continue;
                }
                int mmpos = match.findSubstring (0, mm);
                if (mmpos>=0 && mmpos+mm.size() <=mpos)
                {
                        //                        fprintf (stderr, "added!\n");
                        results.insert (pair<int,ResourceRow*>(mmpos, (*i).second));
                        continue;
                }
                //                fprintf (stderr, "not added!\n");
        }
        
        return results;
}

ResourceRow *
Component_Implementations::Resources_i::findResource (string match)
{
        multimap<int,ResourceRow*> r = findResources (match);
        if (r.size()>0) return (*r.rbegin()).second;
        return NULL;
} 

string Component_Implementations::Resources_i::whichResource (string match)
{
        if (match.size()==0) return Latin1("");        
        ResourceRow *rr = findResource (match);
        if (rr==NULL) return Latin1("");
        return rr->match;
}

string Component_Implementations::Resources_i::getResource (string match)
{
        if (match.size()==0) return Latin1("");
        ResourceRow *rr = findResource (match);
        if (rr==NULL) return Latin1("");
        return rr->data;
}

string Component_Implementations::Resources_i::getExactResource (string match)
{
        map<string,ResourceRow*>::iterator i = exact_table.find (match);
        if (i==exact_table.end()) return Latin1 ("");
        return (*i).second->data;
}

list<string> Component_Implementations::Resources_i::getResources (string match)
{
        list<string> results;
        if (match.size()==0) return results;        
        multimap<int,ResourceRow*> r = findResources (match);
        multimap<int,ResourceRow*>::reverse_iterator i;
        for (i=r.rbegin(); i!=r.rend(); ++i)
        {
                results.push_back ((*i).second->match);
        }
        return results;
}

void Component_Implementations::Resources_i::setResource (string m,
                                                          string d)
{
        m = m.trimends ();
        d = d.trimends ();
        
        // Ignore empty data.
        if (m.size()==0) return;

        /*        m.print (stderr, "Setting ", " to ");
                  d.print (stderr, "", "\n"); */

        string matchend=m, matchmiddle=Latin1("");
        int ppp = m.findSubstring (0, Latin1("..."));        
        if (ppp>0)
        {
                matchend = m.substring (ppp+3);
                matchmiddle = m.substring (0,ppp);
        }

        ResourceRow *rr = new ResourceRow (m, matchend, matchmiddle, d);
        pair<string,ResourceRow*> *p = new pair<string,ResourceRow*>(matchend,rr);
        table.insert (*p);
        exact_table[m] = rr;
}

void Component_Implementations::Resources_i::loadFile (const char *name)
{
        FILE *f = fopen (name, "r");
        char def[256];
        char path[256];
        char row[256];
        char data[65536];

        while (!feof (f))
        {
                strcpy (data, "");
                int c = fscanf (f, "%s %s", def, path);
                if (c!=2) break;
                for (;;)
                {
                        char *r = fgets (row, 256, f);
                        if (r==0) break;
                        if (!strncmp (row, "###", 3)) break;
                        strcat (data, row);
                        /*
                        if (data[strlen(data)-1]=='\n')
                        {
                                data[strlen(data)-1]=0;
                                }*/
                }
                if (strlen(data)>0) setResource (Latin1(path), Latin1(data));
        }
        fclose (f);
}

void Component_Implementations::Resources_i::saveToFile (char *name)
{
        FILE *f = fopen (name, "w");
        char path[256];
        char data[65536];

        map<string,ResourceRow*>::iterator i;
        for (i=exact_table.begin (); i!=exact_table.end (); ++i)
        {
                string d = (*i).second->data.trimends ();
                (*i).first.storeAsLatin1 (path);
                fprintf (f, "DEFINE %s\n", path);
                (*i).second->data.storeAsLatin1 (data);
                fprintf (f, "%s\n###\n\n\n", data);
        }
        fclose (f);
}

// Integer //////////////////////////////////////////////////////////

Component_Implementations::Integer::Integer (Component *c,
                        string n,
                        int v)
                : Component_i (c,n),
                  ClassType (Latin1("Integer"))
{
        value_ = v;        
}

int
Component_Implementations::Integer::value ()
{
        string vv = Component_Implementations::resources->getResource (path());
        
        if (vv!=Latin1(""))
        {
                return vv.asInt();
        }
        return value_;        
}


Component_Implementations::String::String (Component *c,
                                           string n,
                                           string v)
                : Component_i (c,n),
                  ClassType (Latin1("String"))
{
        value_ = v;        
}

string
Component_Implementations::String::value ()
{        
        string vv = Component_Implementations::resources->getResource (path());
        
        if (vv!=Latin1(""))
        {
                return vv;
        }
        return value_;        
}

Component_Implementations::StringMap::StringMap (Component *c,
                                                 string n)
                : Component_i (c,n),
                  ClassType (Latin1("StringMap"))
{
}

map<string,string>&
Component_Implementations::StringMap::value ()
{        
        string row = Component_Implementations::resources->getResource (path());
        
        if (row!=Latin1(""))
        {
                int start=0, middle, end, len=row.size();
                for (;;)
                {
                        for (middle=start; row[middle]!=' ' && middle<len; ++middle);
                        if (middle>=len) break;                        
                        for (end=middle; row[end]!='\n' && end<len; ++end);
                        if (end>=len) break;
                        string key = row.substring (start, middle);
                        string value = row.substring (middle+1, end);
                        value_[key] = value;
                        start = end+1;
                }                
        }
        return value_;        
}

Component_Implementations::Factory_i::Factory_i (Component_ptr composite,
                                                  string name)
                : Component_i (composite, name), ClassType (Latin1("Factory_i"))
{
}

        
void
Component_Implementations::Factory_i::disposeComponent (Component_ptr component)
{
}


/*
Component_ptr Component_Implementations::Factory_i::newComponent (Component_ptr container)
{
        return NULL;
}

Component_ptr Component_Implementations::Factory_i::newComponentArg (Component_ptr container,
                                                                     Component_ptr arg)
{
        return NULL;
}
*/







