// Copyright (C) 1999 Fredrik hrstrm
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _Component_Interfaces_hh
#define _Component_Interfaces_hh

#include<list>
#include<vector>

namespace Component_Interfaces
{
        class Component;
        typedef Component *Component_ptr;
        class Flowback;
        typedef Flowback *Flowback_ptr;        
        class Callback;
        typedef Callback *Callback_ptr;        
        class Resources;
        typedef Resources *Resources_ptr;
        class Factory;
        typedef Factory *Factory_ptr;

        typedef list<Component_ptr> Children;
        
        class Component
        {
                public:

                virtual Component_ptr cleanup () = 0;
                virtual void _dispose () = 0;                
                virtual Component_ptr composite() = 0;
                virtual Children components() = 0;
                virtual string path() = 0;
                virtual string nameAndType() = 0;
                virtual string name () = 0;                
                virtual void addChild (Component_ptr c) = 0;
                virtual void delChild (Component_ptr c) = 0;
                virtual void printTree () = 0;
                virtual void updateResources () = 0;
                virtual Factory_ptr factory () = 0;
                virtual ~Component () = 0;
        };
        
        class Flowback :  public virtual Component
        {
                public:

                virtual void setInvoker (Component_ptr c) = 0;
                virtual Component_ptr getInvoker () = 0;                
                virtual void invokeflow (Component_ptr  c) = 0;
        };
        
        class Callback :  public virtual Flowback
        {
                public:
                
                virtual void invoke() = 0;
        };
        
        class Resources :  public virtual Component
        {
                public:

                virtual list<string> getResources (string match) = 0;
                virtual string getResource (string match) = 0;
                virtual string getExactResource (string match) = 0;
                virtual void setResource (string match, string data) = 0;
                virtual string whichResource (string match) = 0;
                virtual void saveToFile (char *) = 0;
        };

        class Session : public virtual Component
        {
                
        };

        class Factory : public virtual Component
        {
                public:
                 
                virtual void disposeComponent (Component_ptr component) = 0;

                /*
                virtual void useResources (string name) = 0;
                virtual void useResources (Resources_ptr r) = 0;
                virtual Component_ptr newRootComponent () = 0;
                virtual Component_ptr newComponent (Component_ptr container) = 0;
                virtual Component_ptr newComponentArg (Component_ptr container,
                Component_ptr arg) = 0; */
        };
};

#endif









