// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#include"Interaction_Http_Implementations.hh"

using namespace Window_Http_Interfaces;

Interaction_Http_Implementations::InteractionHttpFactory_i *
Interaction_Http_Implementations::interaction_http_factory =
         new Interaction_Http_Implementations::InteractionHttpFactory_i;

// Part ///////////////

Interaction_Http_Implementations::HttpPart_i::HttpPart_i (Component_ptr c, string n, Group_ptr g)
                : Component_i (c, n), ClassType (Latin1("HttpPart")), group_ (g), editor_ (NULL)
{
        //        if (group_!=NULL) group_->addPart (this);
}

Interaction_Interfaces::Group_ptr
Interaction_Http_Implementations::HttpPart_i::parent ()
{
        return group_;
}

void
Interaction_Http_Implementations::HttpPart_i::moveToGroup (Group_ptr g)
{
        if (group_!=NULL) group_->dropPart (this);
        group_ = g;
        if (group_!=NULL) group_->addPart (this);
}

HttpWindow_ptr
Interaction_Http_Implementations::HttpPart_i::getWindow ()


{
        return window_;
}

void
Interaction_Http_Implementations::HttpPart_i::activate ()
{
        window_->mapWindow ();
        active_ = true;
}

void
Interaction_Http_Implementations::HttpPart_i::deactivate ()
{
        window_->unmapWindow ();
        active_ = false;        
}

void
Interaction_Http_Implementations::HttpPart_i::editPart (Part_ptr p, string postfix)
{
        if (editor_==NULL && group_!=NULL)
                group_->editPart (p, postfix);
        else
        if (editor_!=NULL)        
        {
                HttpPart_ptr hp = dynamic_cast<HttpPart_ptr>(p);
                editor_->editPart (hp, postfix);
        }        
}

void
Interaction_Http_Implementations::HttpPart_i::useEditor (Part_ptr p)
{
        editor_ = dynamic_cast<HttpPart_ptr>(p);
}

bool
Interaction_Http_Implementations::HttpPart_i::hasEditor ()
{
        if (editor_!=NULL) return true;
        HttpAtomicGroup_ptr g = dynamic_cast<HttpAtomicGroup_ptr>(group_);
        if (g==NULL) return false;
        
        return g->hasEditor ();
}

void
Interaction_Http_Implementations::HttpPart_i::reloadAppearance ()
{
}

void
Interaction_Http_Implementations::HttpPart_i::handleEditAppearance (HttpCallback_i *cb)
{
        editPart (this, Latin1("/appearance(String)"));
}

void
Interaction_Http_Implementations::HttpPart_i::handleEditContent (HttpCallback_i *cb)
{
        editPart (this, Latin1("/content(String)"));
}

// Message ///////////////

Interaction_Http_Implementations::HttpAtomicMessage_i::HttpAtomicMessage_i (Component_ptr c,
                                                                            string n,
                                                                            Group_ptr g)
                : HttpPart_i (c,n,g), ClassType (Latin1("HttpAtomicMessage")),
                  content_ (this, Latin1("content"), Latin1("")),
                  appearance_ (this, Latin1("appearance"), Latin1("%|edithook|%(ATOMICMESSAGE %|content|%)"))
{
        HttpGroup_ptr gi = dynamic_cast<HttpGroup_ptr>(g);
        Window_ptr parent = NULL;
        if (gi!=NULL) parent = gi->getWindow ();

        window_ = window_http_factory->newHttpWindow (this, Latin1("w"), parent);
        window_->setMappedValue (appearance_.value());
        window_->setValue (Latin1("content"), content_.value());
        window_->setNameTag (Latin1("w")+nameAndType());
        if (gi->hasEditor())
                window_->addEditHook (Latin1("w")+nameAndType(),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditAppearance),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditContent));
        window_->mapWindow ();
}

void
Interaction_Http_Implementations::HttpAtomicMessage_i::reloadAppearance ()
{
        window_->setMappedValue (appearance_.value());
        window_->setValue (Latin1("content"), content_.value());        
}

Interaction_Http_Implementations::HttpElementMessage_i::HttpElementMessage_i (Component_ptr c,
                                                                              string n,
                                                                              Group_ptr g)
                : HttpPart_i (c,n,g), ClassType (Latin1("HttpElementMessage")),
                  appearance_ (this, Latin1("appearance"), Latin1("%|edithook|%(ELEMENTMESSAGE)"))
{
        HttpGroup_ptr gi = dynamic_cast<HttpGroup_ptr>(g);
        Window_ptr parent = NULL;
        if (gi!=NULL) parent = gi->getWindow ();

        window_ = window_http_factory->newHttpWindow (this, Latin1("w"), parent);
        window_->setMappedValue (appearance_.value());
        window_->setValue (Latin1("content"), content_);
        window_->setNameTag (Latin1("w")+nameAndType());
        if (gi->hasEditor ())
                window_->addEditHook (Latin1("w")+nameAndType(),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditAppearance),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditContent));
        
        window_->mapWindow ();
}

void
Interaction_Http_Implementations::HttpElementMessage_i::addElement (string key, Message_ptr m)
{
}

void
Interaction_Http_Implementations::HttpElementMessage_i::showElement (string key)
{
}

void
Interaction_Http_Implementations::HttpElementMessage_i::reloadAppearance ()
{
        window_->setMappedValue (appearance_.value());
}

Interaction_Http_Implementations::HttpTextMessage_i::HttpTextMessage_i (Component_ptr c,
                                                                        string n,
                                                                        Group_ptr g)
                : HttpPart_i (c,n,g), ClassType (Latin1("HttpTextMessage")),
                  appearance_ (this, Latin1("appearance"), Latin1(""))
{
        HttpGroup_ptr gi = dynamic_cast<HttpGroup_ptr>(g);
        Window_ptr parent = NULL;
        if (gi!=NULL) parent = gi->getWindow ();

        window_ = window_http_factory->newHttpWindow (this, Latin1("w"), parent);
        window_->setMappedValue (appearance_.value());
        window_->setValue (Latin1("content"), content_);
        window_->setNameTag (Latin1("w")+nameAndType());
        if (gi->hasEditor ())
                window_->addEditHook (Latin1("w")+nameAndType(),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditAppearance),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditContent));
        window_->mapWindow ();
}

void
Interaction_Http_Implementations::HttpTextMessage_i::setText (string text)
{
        content_ = text;
        window_->setValue (Latin1("content"), content_);
}

string
Interaction_Http_Implementations::HttpTextMessage_i::getText ()
{
        return content_;
}

void
Interaction_Http_Implementations::HttpTextMessage_i::reloadAppearance ()
{
        window_->setMappedValue (appearance_.value());
}

// Choice //////////////////

Interaction_Http_Implementations::HttpElementChoice_i::HttpElementChoice_i (Component_ptr c,
                                                                            string n,
                                                                            Group_ptr g)
                : HttpPart_i (c,n,g), ClassType (Latin1("HttpElementChoice"))
{
        element_choice_group = interaction_http_factory->newAtomicGroup (this, Latin1("element_choice_group"),
                                                                         g);
        element_choice_group->activate ();
}

void Interaction_Http_Implementations::HttpElementChoice_i::addElement (string key, Message_ptr m)
{
        m->moveToGroup (element_choice_group);
}

void Interaction_Http_Implementations::HttpElementChoice_i::setSelectedElement (string key)
{
        selected_ = key;
}

string Interaction_Http_Implementations::HttpElementChoice_i::getSelectedElement ()
{
        return selected_;
}

Interaction_Http_Implementations::HttpTextChoice_i::HttpTextChoice_i (Component_ptr c,
                                                                      string n,
                                                                      Group_ptr g)
                : HttpPart_i (c,n,g), ClassType (Latin1("HttpTextChoice")),
                  initial_content_ (this, Latin1("initialcontent"), Latin1("")),
                  appearance_ (this, Latin1("appearance"), Latin1("%|edithook|%(TEXTCHOICE %|content|%)"))
{
        HttpGroup_ptr gi = dynamic_cast<HttpGroup_ptr>(g);
        Window_ptr parent = NULL;
        if (gi!=NULL) parent = gi->getWindow ();

        window_ = window_http_factory->newHttpWindow (this, Latin1("w"), parent);
        window_->setMappedValue (appearance_.value());
        content_ = initial_content_.value();
        window_->setValue (Latin1("content"), content_);
        window_->setNameTag (Latin1("w")+nameAndType());
        window_->onMessage (Latin1(""), call (this, &HttpTextChoice_i::handleMessage));
        if (gi->hasEditor ())
                window_->addEditHook (Latin1("w")+nameAndType(),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditAppearance),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditContent));
        
        window_->mapWindow ();
}

void Interaction_Http_Implementations::HttpTextChoice_i::setText (string text)
{
        content_ = text;
        window_->setValue (Latin1("content"), content_);
}

string Interaction_Http_Implementations::HttpTextChoice_i::getText ()
{
        return content_;
}

void Interaction_Http_Implementations::HttpTextChoice_i::handleMessage (HttpCallback_i *cb)
{
        content_ = cb->getMessage ();
        window_->setValue (Latin1("content"), content_);        
}

void
Interaction_Http_Implementations::HttpTextChoice_i::reloadAppearance ()
{
        window_->setMappedValue (appearance_.value());
        
}

// FlowControl /////////////

Interaction_Http_Implementations::HttpAtomicFlowControl_i::HttpAtomicFlowControl_i (Component_ptr c,
                                                                                    string n,
                                                                                    Group_ptr g)
                : HttpPart_i (c,n,g), ClassType (Latin1("HttpAtomicFlowControl")),
                  content_ (this, Latin1("content"), Latin1("")),
                  appearance_ (this, Latin1("appearance"), Latin1("%|edithook|%(FLOWCONTROL)")),
                  action_ (NULL)
{
        HttpGroup_ptr gi = dynamic_cast<HttpGroup_ptr>(g);
        Window_ptr parent = NULL;
        if (gi!=NULL) parent = gi->getWindow ();

        window_ = window_http_factory->newHttpWindow (this, Latin1("zw"), parent);
        window_->setMappedValue (appearance_.value());
        window_->setValue (Latin1("content"), content_.value());
        window_->setNameTag (Latin1("w")+nameAndType());
        if (gi->hasEditor ())
                window_->addEditHook (Latin1("w")+nameAndType(),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditAppearance),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditContent));
        window_->mapWindow ();
}

void Interaction_Http_Implementations::HttpAtomicFlowControl_i::setText (string t)
{
        window_->setValue (Latin1("content"), t);        
}

void Interaction_Http_Implementations::HttpAtomicFlowControl_i::onAction (Callback_ptr cb)
{
        action_ = cb;
        action_->setInvoker (this);
        window_->onMessage (Latin1(""), call (this, &HttpAtomicFlowControl_i::handleAction));
}

void Interaction_Http_Implementations::HttpAtomicFlowControl_i::handleAction (HttpCallback_i *cb)
{
        if (action_!=NULL) action_->invoke ();
}

void
Interaction_Http_Implementations::HttpAtomicFlowControl_i::reloadAppearance ()
{
        window_->setMappedValue (appearance_.value());
        window_->setValue (Latin1("content"), content_.value());
        
}

void
Interaction_Http_Implementations::HttpAtomicFlowControl_i::useMessage (Message_ptr m)
{
        HttpMessage_ptr hm = dynamic_cast<HttpMessage_ptr>(m);
        message_ = hm;
}

// TextEdit //////////////////

Interaction_Http_Implementations::HttpTextEdit_i::HttpTextEdit_i (Component_ptr c,
                                                                  string n,
                                                                  Group_ptr g,
                                                                  TextModel_ptr m)
                : HttpPart_i (c,n,g), ClassType (Latin1("HttpTextEdit")),
                  appearance_ (this, Latin1("appearance"), Latin1("%|edithook|%(TEXTEDIT %|content|%)")),
                  text_model_ (m)
{
        HttpGroup_ptr gi = dynamic_cast<HttpGroup_ptr>(g);
        Window_ptr parent = NULL;
        if (gi!=NULL) parent = gi->getWindow ();

        window_ = window_http_factory->newHttpWindow (this, Latin1("w"), parent);
        window_->setMappedValue (appearance_.value());
        window_->setValue (Latin1("content"),
                           TranslateHtmlSensitiveChars (text_model_->asString (Latin1("\n"))));
        window_->setNameTag (Latin1("w")+nameAndType());
        if (gi->hasEditor ())                
                window_->addEditHook (Latin1("w")+nameAndType(),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditAppearance),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditContent));
        window_->onMessage (Latin1(""), call (this, &HttpTextEdit_i::handleMessage_));        
        window_->mapWindow ();
}

void 
Interaction_Http_Implementations::HttpTextEdit_i::handleMessage_ (HttpCallback_i *cb)
{
        string c = cb->getMessage ();
        text_model_->fromString (c);
        window_->setValue (Latin1("content"),
                           TranslateHtmlSensitiveChars (text_model_->asString (Latin1("\n"))));
}

void 
Interaction_Http_Implementations::HttpTextEdit_i::useModel (TextModel_ptr model)
{}

void 
Interaction_Http_Implementations::HttpTextEdit_i::updateModel ()
{
        window_->setValue (Latin1("content"),
                           TranslateHtmlSensitiveChars (text_model_->asString (Latin1("\n"))));
}

void
Interaction_Http_Implementations::HttpTextEdit_i::reloadAppearance ()
{
        window_->setMappedValue (appearance_.value());
}

// Group ///////////////


Interaction_Http_Implementations::HttpAtomicGroup_i::HttpAtomicGroup_i (Component_ptr c, string n, HttpWindow_ptr wp)
                : HttpPart_i (c,n,NULL),
                  ClassType (Latin1("HttpAtomicGroup")),
                  appearance_ (this, Latin1("appearance"), Latin1("%|edithook|%(GROUP %[CHILDREN]%)"))
{
        window_ = window_http_factory->newHttpWindow (this, Latin1("w"), wp);
        window_->setMappedValue (appearance_.value());
        window_->setNameTag (Latin1("w")+nameAndType());
        /*
        window_->addEditHook (Latin1("w")+nameAndType(),
        call ((HttpPart_i*)this, &HttpPart_i::handleEditAppearance)); */
        active_ = false;
}

Interaction_Http_Implementations::HttpAtomicGroup_i::HttpAtomicGroup_i (Component_ptr c, string n, Group_ptr g)
                : HttpPart_i (c,n,g),
                  ClassType (Latin1("HttpAtomicGroup")),
                  appearance_ (this, Latin1("appearance"), Latin1("%|edithook|%(GROUP %[CHILDREN]%)"))
{
        HttpAtomicGroup_i *gi = dynamic_cast<HttpAtomicGroup_i*>(g);
        Window_ptr wp = NULL;
        if (gi!=NULL) wp = gi->window_;
        window_ = window_http_factory->newHttpWindow (this, Latin1("w"), wp);
        window_->setMappedValue (appearance_.value());
        window_->setNameTag (Latin1("w")+nameAndType());
        if (gi->hasEditor ())                
                window_->addEditHook (Latin1("w")+nameAndType(),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditAppearance),
                                      call ((HttpPart_i*)this, &HttpPart_i::handleEditContent));
        active_ = false;        
}

void
Interaction_Http_Implementations::HttpAtomicGroup_i::addPart (Part_ptr c)
{
        HttpPart_ptr hp = dynamic_cast<HttpPart_ptr> (c);
        if (hp!=NULL)
        {
                children_.push_back (hp);
                window_->addWindowChild (hp->getWindow ());
        }
}

void
Interaction_Http_Implementations::HttpAtomicGroup_i::dropPart (Part_ptr c)
{        
        HttpPart_ptr hc = dynamic_cast<HttpPart_ptr>(c);
        window_->dropWindowChild (hc->getWindow());
        
        list<HttpPart_ptr>::iterator i;
        for (i=children_.begin(); i!=children_.end(); ++i)
        {
                if (*i==hc)
                {
                        children_.erase (i);
                        return;
                }
        }
}

void
Interaction_Http_Implementations::HttpAtomicGroup_i::focus (Part_ptr part)
{
        HttpPart_ptr p = dynamic_cast<HttpPart_ptr>(part);
        p->getWindow()->focusWindow ();
}

void
Interaction_Http_Implementations::HttpAtomicGroup_i::unfocus (Part_ptr part)
{
        HttpPart_ptr p = dynamic_cast<HttpPart_ptr>(part);        
        p->getWindow()->unfocusWindow ();
}

void
Interaction_Http_Implementations::HttpAtomicGroup_i::reloadAppearance ()
{
        window_->setMappedValue (appearance_.value());
        list<HttpPart_ptr>::iterator i;
        for (i=children_.begin(); i!=children_.end(); ++i)
        {
                (*i)->reloadAppearance ();
        }        
}

// ListView //////////////////

Interaction_Http_Implementations::HttpListView_i::HttpListView_i (Component_ptr c,
                                                                  string n,
                                                                  Group_ptr g,
                                                                  ListModel_ptr m)
                : HttpPart_i (c,n,g), ClassType (Latin1("HttpListView")),                  
                  rview_width_ (this, Latin1 ("view_width"), 3),
                  rview_height_ (this, Latin1 ("view_height"), 5),
                  list_model_ (m)
{
        matrix_group_ = interaction_http_factory->newAtomicGroup (this, Latin1("matrix_group"), g);

        view_width_ = rview_width_.value ();
        view_height_ = rview_height_.value ();
        cells_.setSize (view_width_, view_height_);

        string cell;
        for (int y=0; y<view_height_; ++y)
                for (int x=0; x<view_width_; ++x)
                {
                        
                        if (x==0) cell = Latin1("cell_left");
                        else
                        if (x==view_width_-1) cell = Latin1("cell_right");
                        else
                                cell = Latin1("cell_middle");
                        TextMessage_ptr msg = interaction_http_factory->newTextMessage (this, cell, matrix_group_);
                        string xx, yy;
                        xx.asString (x);
                        yy.asString (y);
                        msg->setText (xx+Latin1("+")+yy);
                        cells_.set (x,y,msg);
                }                        
        matrix_group_->activate ();
}

void 
Interaction_Http_Implementations::HttpListView_i::useModel (ListModel_ptr model)
{
        list_model_ = model;
}

void 
Interaction_Http_Implementations::HttpListView_i::show (int xp, int yp)
{
        pos_x_ = xp;
        pos_y_ = yp;
        for (int y=0; y<view_height_; ++y)
                for (int x=0; x<view_width_; ++x)
                {                        
                        TextMessage_ptr msg = cells_.get (x,y);
                        list_model_->fillCell (xp+x, yp+y, msg);
                }                        
}

void 
Interaction_Http_Implementations::HttpListView_i::setSize (int w, int h)
{
        width_ = w;
        height_ = h;
}

// Factory //////////////

Interaction_Http_Implementations::InteractionHttpFactory_i::InteractionHttpFactory_i () 
                : Factory_i (NULL, Latin1("InteractionHttpFactory")), ClassType (Latin1("InteractionHttpFactory"))
{
}

Interaction_Http_Interfaces::HttpAtomicGroup_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newHttpRootGroup (Component_ptr container,
                                                                              string name,
                                                                              HttpWindow_ptr root)
{
        return new HttpAtomicGroup_i (container, name, root);
}


Interaction_Interfaces::AtomicMessage_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newAtomicMessage (Component_ptr container,
                                                                              string name,
                                                                              Group_ptr g)
{
        return new HttpAtomicMessage_i (container, name, g);
}

Interaction_Interfaces::ElementMessage_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newElementMessage (Component_ptr container,
                                                                               string name,
                                                                               Group_ptr g)
{
        return new HttpElementMessage_i (container, name, g);
}

Interaction_Interfaces::TextMessage_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newTextMessage (Component_ptr container,
                                                                              string name,
                                                                              Group_ptr g)
{
        return new HttpTextMessage_i (container, name, g);
}

Interaction_Interfaces::ElementChoice_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newElementChoice (Component_ptr container,
                                                                              string name,
                                                                              Group_ptr g)
{
        return new HttpElementChoice_i (container, name, g);
}

Interaction_Interfaces::TextChoice_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newTextChoice (Component_ptr container,
                                                                           string name,
                                                                           Group_ptr g)
{
        return new HttpTextChoice_i (container, name, g);
}

Interaction_Interfaces::AtomicFlowControl_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newAtomicFlowControl (Component_ptr container,
                                                                                  string name,
                                                                                  Group_ptr g)
{
        return new HttpAtomicFlowControl_i (container, name, g);
}

Interaction_Interfaces::AtomicGroup_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newAtomicGroup (Component_ptr container,
                                                                            string name,
                                                                            Group_ptr root)
{
        return new HttpAtomicGroup_i (container, name, root);
}

Interaction_Interfaces::TextEdit_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newTextEdit (Component_ptr container,
                                                                         string name,
                                                                         Group_ptr root,
                                                                         TextModel_ptr model)
{
        return new HttpTextEdit_i (container, name, root, model);
}

Interaction_Interfaces::TextView_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newTextView (Component_ptr container,
                                                                         string name,
                                                                         Group_ptr root,
                                                                         TextModel_ptr model)
{
        return NULL;
        //       return new HttpTextView_i (container, name, root);
}

Interaction_Interfaces::ListView_ptr
Interaction_Http_Implementations::InteractionHttpFactory_i::newListView (Component_ptr container,
                                                                         string name,
                                                                         Group_ptr root,
                                                                         ListModel_ptr model)
{
        return new HttpListView_i (container, name, root, model);
}









