// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _Interaction_Http_Implementations_hh
#define _Interaction_Http_Implementations_hh

#include"Interaction_Http_Interfaces.hh"
#include"Component_Implementations.hh"
#include"Window_Http_Implementations.hh"
#include"matrix.hh"

namespace Interaction_Http_Implementations
{
        using namespace Interaction_Http_Interfaces;
        using namespace Component_Implementations;
        using namespace Window_Http_Implementations;

        class HttpPart_i : public virtual HttpPart,
                           public Component_i
        {
                private:
                
                Group_ptr group_;
                HttpPart_ptr editor_;
                
                protected:
                
                HttpWindow_ptr window_;
                bool active_;
                
                public:

                HttpPart_i (Component_ptr c, string n, Group_ptr g);
                Group_ptr parent ();                
                void moveToGroup (Group_ptr g);
                HttpWindow_ptr getWindow ();
                void activate ();
                void deactivate ();
                void editPart (Part_ptr p, string postfix);
                void useEditor (Part_ptr p);
                bool hasEditor ();
                
                virtual void reloadAppearance ();
                virtual void handleEditAppearance (HttpCallback_i *cb);
                virtual void handleEditContent (HttpCallback_i *cb);                
        };

        class HttpAtomicMessage_i : public virtual HttpAtomicMessage,
                                    public HttpPart_i
        {
                String content_;
                String appearance_;

                public:
                
                HttpAtomicMessage_i (Component_ptr c,
                                     string name,
                                     Group_ptr g);

                void reloadAppearance ();
        };

        class HttpElementMessage_i : public virtual HttpElementMessage,
                                     public HttpPart_i
        {
                string content_;
                String appearance_;

                public:
                
                HttpElementMessage_i (Component_ptr c,
                                      string name,
                                      Group_ptr g);
                void addElement (string key, Message_ptr message);
                void showElement (string key);
                void reloadAppearance ();                
        };
        
        class HttpTextMessage_i : public virtual HttpTextMessage,
                                          public HttpPart_i
        {
                string content_;
                String appearance_;

                public:
                
                HttpTextMessage_i (Component_ptr c,
                                   string name,
                                   Group_ptr g);
                void setText (string text);
                string getText ();
                void reloadAppearance ();                
        };
        
        class HttpAtomicGroup_i : public virtual HttpAtomicGroup,
                                  public HttpPart_i
        {
                String appearance_;                
                list<HttpPart_ptr> children_;
                
                public:

                HttpAtomicGroup_i (Component_ptr c, string n, HttpWindow_ptr w);
                HttpAtomicGroup_i (Component_ptr c, string n, Group_ptr g);
                void addPart (Part_ptr part);
                void dropPart (Part_ptr part);                
                void focus (Part_ptr part);
                void unfocus (Part_ptr part);
                void reloadAppearance ();                
        };
        
        class HttpAtomicFlowControl_i : public virtual HttpAtomicFlowControl,
                                        public HttpPart_i
        {
                String content_;                
                String appearance_;
                Callback_ptr action_;
                HttpMessage_ptr message_;
                
                public:
                
                HttpAtomicFlowControl_i (Component_ptr c,
                                         string name,
                                         Group_ptr g);

                
                void setText (string t);
                void onAction (Callback_ptr cb);
                void handleAction (HttpCallback_i *cb);
                void reloadAppearance ();
                void useMessage (Message_ptr m);
        };

        class HttpElementChoice_i : public virtual HttpElementChoice,
                                    public HttpPart_i
        {
                string selected_;
                AtomicGroup_ptr element_choice_group;

                public:
                
                HttpElementChoice_i (Component_ptr c,
                                     string name,
                                     Group_ptr g);

                void addElement (string key, Message_ptr m);
                void setSelectedElement (string key);
                string getSelectedElement ();
        };

        
        class HttpTextChoice_i : public virtual HttpTextChoice,
                                 public HttpPart_i
        {
                string content_;
                String initial_content_;
                String appearance_;
                void handleMessage (HttpCallback_i *cb);

                public:
                
                HttpTextChoice_i (Component_ptr c,
                                  string name,
                                  Group_ptr g);

                void setText (string text);
                string getText ();
                void reloadAppearance ();
        };

        class HttpTextEdit_i : public virtual HttpTextEdit,
                               public HttpPart_i
        {
                String appearance_;
                TextModel_ptr text_model_;

                public:
                
                HttpTextEdit_i (Component_ptr c,
                                string name,
                                Group_ptr g,
                                TextModel_ptr model);

                void handleMessage_ (HttpCallback_i *cb);
                void useModel (TextModel_ptr model);
                void updateModel ();
                void reloadAppearance ();                
        };

        
        class HttpListView_i : public virtual HttpListView,
                               public HttpPart_i
        {
                public:

                AtomicGroup_ptr matrix_group_;
                Integer rview_width_, rview_height_;
                int view_width_, view_height_, width_, height_, pos_x_, pos_y_;
                ListModel_ptr list_model_;
                matrix<TextMessage_ptr> cells_;
                
                HttpListView_i (Component_ptr c,
                                string name,
                                Group_ptr g,
                                ListModel_ptr model);

                void useModel (ListModel_ptr model);
                void show (int x, int y);
                void setSize (int w, int h);
        };

        class InteractionHttpFactory_i : public virtual InteractionHttpFactory,
                                         public Factory_i
        {
                public:
                
                InteractionHttpFactory_i ();                
                
                HttpAtomicGroup_ptr newHttpRootGroup (Component_ptr container,
                                                      string name,
                                                      HttpWindow_ptr root);
                
                AtomicMessage_ptr newAtomicMessage (Component_ptr container,
                                                     string name,
                                                     Group_ptr g);

                ElementMessage_ptr newElementMessage (Component_ptr container,
                                                       string name,
                                                       Group_ptr g);
                
                TextMessage_ptr newTextMessage (Component_ptr container,
                                                 string name,
                                                 Group_ptr g);
                
                ElementChoice_ptr  newElementChoice (Component_ptr container,
                                                     string name,
                                                     Group_ptr g);                
                
                TextChoice_ptr  newTextChoice (Component_ptr container,
                                                             string name,
                                                             Group_ptr g);

                AtomicFlowControl_ptr newAtomicFlowControl (Component_ptr container,
                                                             string name,
                                                             Group_ptr g);
                
                AtomicGroup_ptr newAtomicGroup (Component_ptr container,
                                          string name,
                                          Group_ptr g);

                TextEdit_ptr newTextEdit (Component_ptr composite,
                                          string name,
                                          Group_ptr g,
                                          TextModel_ptr model);

                TextView_ptr newTextView (Component_ptr composite,
                                          string name,
                                          Group_ptr g,
                                          TextModel_ptr);                                

                ListView_ptr newListView (Component_ptr composite,
                                          string name,
                                          Group_ptr g,
                                          ListModel_ptr);                                
                
        };

        extern InteractionHttpFactory_i *interaction_http_factory; 
};

#endif


        /*        
        class HttpAction_i : public virtual Action,
                             public Callback_i
        {
                public:

                HttpAction_i (Component_i *c,
                              SysCaller *sc)
                                : Callback_i (c, sc), ClassType ("HttpAction_i")
                {}
        };

        class HttpTableCallback_i : public virtual HttpTableCallback,
                                    public Callback_i
        {
                map<int,HttpPart_ptr> cols_;
                int row_;
                TableRowState state_;
                
                public:

                HttpTableCallback_i (Component_i *c,
                                     SysCaller *sc)
                                : Callback_i (c, sc), ClassType ("HttpTableCallback_i")
                {}

                int getRow ();
                void setRow (int r);
                void setRowState (TableRowState ts);
                TableRowState getRowState ();                                                
                Part_ptr getColElement (int c);
                HttpPart_ptr getHttpColElement (int c);                
                void setColElement (int c, Part_ptr p);
        };        


        
        class HttpTable_i : public virtual HttpTable,
                            public HttpPart_i
        {
                typedef map<int,HttpPart_ptr> ColParts;
                typedef map<int,ColParts*> RowParts;
                // Here is the visible (and a little more) table data stored.
                RowParts data;

                // Row and cols are defined in the resource database.
                String row_appearance_, col_appearance_;
                // How many visible rows and cols in resouce.
                Integer vrows_, vcols_;
                
                // Total number of rows and cols in table.
                int nof_rows_, nof_cols_;
                // The visible rows/cols on screen.
                int visible_rows_, visible_cols_;
                // The first visible row.
                int row_offset_;
                // Call this to render a row.
                HttpTableCallback_ptr row_render_cb_;
                // This is the group which the table lies within.
                HttpGroup_ptr table_group_;

                // Remember windows, when this is deleted so are the
                // windows.
                Component_ptr remember_windows_;
                
                void renderTable (HttpCallback_i *cb);
                
                public:

                HttpTable_i (Component_ptr c, string n, Group_ptr g);
                void setNofRows (int r);
                void setNofCols (int r);
                void setRowOffset (int o);
                int  getRowOffset ();
                int  getVisibleRows ();
                int  getVisibleCols ();
                
                void onRenderRow (TableCallback_ptr cb);

                void updateResources ();
        };

                class HttpButton_i : public virtual HttpButton,
                             public HttpPart_i        
        {
                Action *action_;
                void doAction (HttpCallback_i *cb);
                String appearance_;

                public:
                
                HttpButton_i (Component_ptr c,
                              string name,
                              Group_ptr g);

                void onAction (Action *cb);
        };

        class HttpInput_i : public virtual HttpInput,
                            public HttpPart_i        
        {
                string value_;
                String initial_value_;
                String appearance_;
                void doAction (HttpCallback_i *cb);

                public:
                
                HttpInput_i (Component_ptr c,
                             string name,
                             Group_ptr g);

                string getValue ();
                void setValue (string v);                
        };

        class HttpOutput_i : public virtual HttpOutput,
                             public HttpPart_i        
        {
                String initial_value_;
                String appearance_;

                public:
                
                HttpOutput_i (Component_ptr c,
                              string name,
                              Group_ptr g);

                void setValue (string v);
        };
        
        class HttpChoice_i : public virtual HttpChoice,
                             public HttpPart_i
        {
                HttpGroup_ptr choice_group_;
                string selected_;
                String row_appearance_;
                map<string,HttpWindow_ptr> choices_;

                void getMessage (HttpCallback_i *cb);
                
                public:
                
                HttpChoice_i (Component_ptr c,
                              string name,
                              Group_ptr g);

                Group_ptr group ();
                void addChoice (string key, Part_ptr msg);
                void removeChoice (string key);
                void removeAllChoices ();
                void setChoice (string key);
                string getChoice ();
        };

        */
