// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _Interaction_Http_Interfaces_hh
#define _Interaction_Http_Interfaces_hh

#include "Interaction_Interfaces.hh"
#include "Window_Http_Interfaces.hh"

namespace Interaction_Http_Interfaces
{
        using namespace Interaction_Interfaces;
        using namespace Window_Http_Interfaces;        
        
        class InteractionHttpFactory;
        typedef InteractionHttpFactory *InteractionHttpFactory_ptr;
        
        class HttpPart;
        typedef HttpPart *HttpPart_ptr;
        class HttpMessage;
        typedef HttpMessage *HttpMessage_ptr;
        class HttpAtomicMessage;
        typedef HttpAtomicMessage *HttpAtomicMessage_ptr;
        class HttpElementMessage;
        typedef HttpElementMessage *HttpElementMessage_ptr;
        class HttpTextMessage;
        typedef HttpTextMessage *HttpTextMessage_ptr;

        class HttpChoice;
        typedef HttpChoice *HttpChoice_ptr;
        class HttpElementChoice;
        typedef HttpElementChoice *HttpElementChoice_ptr;
        class HttpTextChoice;
        typedef HttpTextChoice *HttpTextChoice_ptr;
        
        class HttpFlowControl;
        typedef HttpFlowControl *HttpFlowControl_ptr;
        class HttpAtomicFlowControl;
        typedef HttpAtomicFlowControl *HttpAtomicFlowControl_ptr;

        class HttpGroup;
        typedef HttpGroup *HttpGroup_ptr;
        class HttpAtomicGroup;
        typedef HttpAtomicGroup *HttpAtomicGroup_ptr;

        class HttpTextView;
        typedef HttpTextView *HttpTextView_ptr;        

        class HttpListView;
        typedef HttpListView *HttpListView_ptr;        
        
        class HttpPart : public virtual Part
        {
                public:

                virtual HttpWindow_ptr getWindow () = 0;
                virtual void reloadAppearance () = 0;
                virtual bool hasEditor () = 0;
        };
        
        class HttpMessage : public virtual Message,
                            public virtual HttpPart
        {
        };

        class HttpAtomicMessage : public virtual AtomicMessage,
                                  public virtual HttpMessage
        {
        };

        class HttpElementMessage : public virtual ElementMessage,
                                   public virtual HttpMessage
        {
        };
        
        class HttpTextMessage : public virtual TextMessage,
                                public virtual HttpMessage
        {
        };        
        
        class HttpChoice : public virtual Choice,
                           public virtual HttpPart
        {
        };

        class HttpElementChoice : public virtual ElementChoice,
                               public virtual HttpChoice
        {
        };

        
        class HttpTextChoice : public virtual TextChoice,
                               public virtual HttpChoice
        {
        };

        class HttpFlowControl : public virtual FlowControl,
                                public virtual HttpPart
        {
        };
        
        class HttpAtomicFlowControl : public virtual AtomicFlowControl,
                                      public virtual HttpFlowControl
        {
        };
        
        class HttpGroup : public virtual Group,
                          public virtual HttpPart
        {
        };

        class HttpAtomicGroup : public virtual AtomicGroup,
                                public virtual HttpGroup
        {
        };
        
        class HttpTextEdit : public virtual TextEdit
        {
        };
        
        class HttpTextView : public virtual TextView
        {
        };

        class HttpListView : public virtual ListView
        {
        };
        
        class InteractionHttpFactory : public virtual InteractionFactory
        {
                public:

                virtual HttpAtomicGroup_ptr newHttpRootGroup (Component_ptr container,
                                                              string name,
                                                              HttpWindow_ptr root) = 0;
        };
};

#endif






/*

        class HttpTableCallback : public virtual TableCallback
        {
                public:

                virtual void setRow (int r) = 0;
                virtual void setRowState (TableRowState ts) = 0;
                virtual HttpPart_ptr getHttpColElement (int c) = 0;
        };
        typedef HttpTableCallback *HttpTableCallback_ptr;





*/
