// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _Interaction_Interfaces_hh
#define _Interaction_Interfaces_hh

#include"string.hh"
#include"Component_Interfaces.hh"
#include"Basic_Models.hh"

namespace Interaction_Interfaces
{
        using namespace Component_Interfaces;

        class InteractionFactory;
        typedef InteractionFactory *InteractionFactory_ptr;
        
        class Part;
        typedef Part *Part_ptr;

        // Messages ////////////////////
        
        class Message;
        typedef Message *Message_ptr;

        class AtomicMessage;
        typedef AtomicMessage *AtomicMessage_ptr;

        class SubsetMessage;
        typedef SubsetMessage *SubsetMessage_ptr;
        
        class ElementMessage;
        typedef ElementMessage *ElementMessage_ptr;

        class NumericMessage;
        typedef NumericMessage *NumericMessage_ptr;

        class TextMessage;
        typedef TextMessage *TextMessage_ptr;
        
        // Choices ///////////////////////

        class Choice;
        typedef Choice *Choice_ptr;

        class SubsetChoice;
        typedef SubsetChoice *SubsetChoice_ptr;

        class ElementChoice;
        typedef ElementChoice *ElementChoice_ptr;

        class NumericChoice;
        typedef NumericChoice *NumericChoice_ptr;

        class TextChoice;
        typedef TextChoice *TextChoice_ptr;

        // FlowControl ////////////////////
        
        class FlowControl;
        typedef FlowControl *FlowControl_ptr;

        class AtomicFlowControl;
        typedef AtomicFlowControl *AtomicFlowControl_ptr;

        class LockingFlowControl;
        typedef LockingFlowControl *LockingFlowControl_ptr;

        // Grouping //////////////////////
        
        class Group;
        typedef Group *Group_ptr;

        class AtomicGroup;
        typedef AtomicGroup *AtomicGroup_ptr;

        class ArrayGroup;
        typedef ArrayGroup *ArrayGroup_ptr;

        class HierarchicalGroup;
        typedef HierarchicalGroup *HierarchicalGroup_ptr;

        // Editor //////////////////////////

        class TextEdit;
        typedef TextEdit *TextEdit_ptr;

        // Views ///////////////////////////

        class TextView;
        typedef TextView *TextView_ptr;

        class ListView;
        typedef ListView *ListView_ptr;

        class ListModel;
        typedef ListModel *ListModel_ptr;
        
        // Adapters ////////////////////////
        
        class Adapter;
        typedef Adapter *Adapter_ptr;

        class BasicActionAdapter;
        typedef BasicActionAdapter *BasicActionAdapter_ptr;

        class TextEditAdapter;
        typedef TextEditAdapter *TextEditAdapter_ptr;

        // Controller //////////////////////

        class EditTextController;
        typedef EditTextController *EditTextController_ptr;
        
        // Actual classes ///////////////////

        class Part : public virtual Component
        {
                public:

                virtual Group_ptr parent () = 0;
                virtual void moveToGroup (Group_ptr g) = 0;
                virtual void activate () = 0;
                virtual void deactivate () = 0;
                virtual void editPart (Part_ptr p, string postfix) = 0;
                virtual void useEditor (Part_ptr p) = 0;
        };

        class Message : public virtual Part
        {
                public:
        };

        class AtomicMessage : public virtual Message
        {
                public:
        };

        class ElementMessage : public virtual Message
        {
                public:
                virtual void addElement (string key, Message_ptr message) = 0;
                virtual void showElement (string key) = 0;
        };
        
        class TextMessage : public virtual Message
        {
                public:
                virtual void setText (string text) = 0;
                virtual string getText () = 0;
        };
        
        class Choice : public virtual Part
        {
                public:
        };

        class ElementChoice : public virtual Choice
        {
                public:
                
                virtual void addElement (string key, Message_ptr p) = 0;
                virtual void setSelectedElement (string key) = 0;
                virtual string getSelectedElement () = 0;
        };

        class TextChoice : public virtual Choice
        {
                public:
                
                virtual void setText (string text) = 0;
                virtual string getText () = 0;
        };

        class FlowControl : public virtual Part
        {
                public:

                virtual void useMessage (Message_ptr m) = 0;
        };

        class AtomicFlowControl : public virtual FlowControl
        {
                public:

                virtual void onAction (Callback_ptr cb) = 0;
                virtual void setText (string t) = 0;
        };

        class Group : public virtual Part
        {
                public:

                virtual void addPart (Part_ptr part) = 0;
                virtual void dropPart (Part_ptr part) = 0;                
                virtual void focus (Part_ptr part) = 0;
                virtual void unfocus (Part_ptr part) = 0;
        };

        class AtomicGroup : public virtual Group
        {
                public:
        };

        class TextEdit : public virtual Part
        {
                public:

                virtual void useModel (TextModel_ptr model) = 0;
                virtual void updateModel () = 0;
        };
        
        class TextView : public virtual Part
        {
                public:

                virtual void useModel (TextModel_ptr model) = 0;
                virtual void show (int blocknr, int vblocknr) = 0;
        };

        class ListModel 
        {
                public:
                
                virtual void fillCell (int x, int y, TextMessage_ptr m) = 0;
        };
        
        class ListView : public virtual Part
        {
                public:

                virtual void useModel (ListModel_ptr model) = 0;
                virtual void show (int x, int y) = 0;
                virtual void setSize (int w, int h) = 0;
        };
        
        class InteractionFactory : public virtual Factory
        {
                public:

                virtual AtomicMessage_ptr newAtomicMessage (Component_ptr composite,
                                                            string name,
                                                            Group_ptr g) = 0;

                virtual ElementMessage_ptr newElementMessage (Component_ptr composite,
                                                              string name,
                                                              Group_ptr g) = 0;
                
                virtual TextMessage_ptr newTextMessage (Component_ptr composite,
                                                        string name,
                                                        Group_ptr g) = 0;
                
                virtual ElementChoice_ptr newElementChoice (Component_ptr composite,
                                                            string name,
                                                            Group_ptr g) = 0;
                
                virtual TextChoice_ptr newTextChoice (Component_ptr composite,
                                                      string name,
                                                      Group_ptr g) = 0;

                virtual AtomicFlowControl_ptr newAtomicFlowControl (Component_ptr composite,
                                                                    string name,
                                                                    Group_ptr g) = 0;

                virtual AtomicGroup_ptr newAtomicGroup (Component_ptr composite,
                                                        string name,
                                                        Group_ptr g) = 0;                

                virtual TextEdit_ptr newTextEdit (Component_ptr composite,
                                                  string name,
                                                  Group_ptr g,
                                                  TextModel_ptr text) = 0;

                virtual TextView_ptr newTextView (Component_ptr composite,
                                                  string name,
                                                  Group_ptr g,
                                                  TextModel_ptr text) = 0;
                
                virtual ListView_ptr newListView (Component_ptr composite,
                                                  string name,
                                                  Group_ptr g,
                                                  ListModel_ptr list) = 0;
        };


        // Graphical Interaction Objects
        /*        
        class TextView : public Part_i
        {
                public:
                
                virtual int  getWidth () = 0;
                virtual int  getHeight () = 0;
                virtual void setPos (long pos) = 0;
                virtual void useModel (TextModel_ptr model) = 0;
                virtual EditTextAdapter_ptr getEditTextAdapter () = 0;
        };        
        
        class EditTextController
        {
                public:
                
                virtual void cursor (long pos) = 0;
                virtual void mark (long start, long end) = 0;
                virtual void keypressed (long pos, unichar c) = 0;
        };

        class EditTextAdapter
        {
                public:
                
                virtual void useController (EditTextController_ptr ctrl) = 0;
                virtual void onCursor (Callback_ptr cb) = 0;
                virtual void onMark (Callback_ptr cb) = 0;
                virtual void onKeypressed (Callback_ptr cb) = 0;
                };*/        
};

#endif














