// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#include "Interaction_X_Implementations.hh"

Interaction_X_Implementations::InteractionXFactory_i*
Interaction_X_Implementations::interaction_x_factory = new Interaction_X_Implementations::InteractionXFactory_i;

Interaction_X_Implementations::XPart_i::XPart_i (Component_ptr c,
                                                 string n,
                                                 Group_ptr g)
                : Component_i (c,n), ClassType (Latin1("XPart_i")),
                  group_ (g),
                  foreground (this, Latin1("foreground"), Latin1("")),
                  background (this, Latin1("background"), Latin1(""))
{
        screen = window_x_factory->getScreen ();

        gc = window_x_factory->newGC (this, screen);
        foor.scale (1,1);
        font = window_x_factory->newFont (this, Latin1("forall"), foor);
        string fgc = foreground.value().trimends();
        string bgc = background.value().trimends();

        if (fgc.size()!=0)
        {
                Color fg = screen->getColorByName (fgc);
                fgc.print (stderr, "Foreground>>", "<<\n");
                gc->setForeground (fg.pixel);
        }
        
        if (bgc.size()!=0)
        {
                Color bg = screen->getColorByName (bgc);
                bgc.print (stderr, "Background>>", "<<\n");
                gc->setBackground (bg.pixel);
        }
        
}

void
Interaction_X_Implementations::XPart_i::activate ()
{
        window_->mapAll ();
        window_->mapWindow ();        
        window_x_factory->sync ();                        
}

void
Interaction_X_Implementations::XPart_i::deactivate ()
{
        window_->unmapWindow ();
        window_x_factory->sync ();                        
}

void
Interaction_X_Implementations::XPart_i::moveToGroup (Group_ptr g)
{
}

Interaction_Interfaces::Group_ptr
Interaction_X_Implementations::XPart_i::parent ()
{
        return group_;
}



Interaction_X_Implementations::XAtomicGroup_i::XAtomicGroup_i (Component_ptr c,
                                                   string n,
                                                   Group_ptr g)
                : XPart_i (c,n,g), ClassType (Latin1("XAtomicGroup_i"))
{
        XGroup_ptr gp = dynamic_cast<XGroup_ptr>(g);
        if (gp!=0) window_parent_ = gp->getWindow ();
        else window_parent_ = screen->getRootWindow ();
        
        window_ = window_x_factory->newXWindow (this,
                                                Latin1("win"),
                                                window_parent_,
                                                0, 0,
                                                256,
                                                256);
}

void
Interaction_X_Implementations::XAtomicGroup_i::addPart (Part_ptr c)
{
}

void
Interaction_X_Implementations::XAtomicGroup_i::dropPart (Part_ptr c)
{
}

void
Interaction_X_Implementations::XAtomicGroup_i::focus (Part_ptr c)
{
}

void
Interaction_X_Implementations::XAtomicGroup_i::unfocus (Part_ptr c)
{
}

Window_X_Interfaces::XWindow_ptr
Interaction_X_Implementations::XPart_i::getWindow ()
{
        return window_;
}

Interaction_X_Implementations::XAtomicMessage_i::XAtomicMessage_i (Component_ptr c,
                                                                   string n,
                                                                   Group_ptr g)
                : XPart_i (c,n,g), ClassType (Latin1("XAtomicMessage_i")),
                  content (this, Latin1("content"), Latin1(""))
{                        
        XGroup_ptr gp = dynamic_cast<XGroup_ptr> (g);
        text = window_x_factory->newText (this);
        text->addItem (content.value().trimends(), font);
        width_ = text->width ()+10;
        height_ = text->height ()+10;
        
        window_ = window_x_factory->newXWindow (this, Latin1("win"),
                                                gp->getWindow(),
                                                0, 0,
                                                width_,
                                                height_);

        window_->onExpose (call (this, &XAtomicMessage_i::redraw));
}

void
Interaction_X_Implementations::XAtomicMessage_i::redraw (XCallback_i *cb)
{
        window_->drawRectangle (gc, 0, 0, width_-1, height_-1);
        window_->drawText (gc, text, 5, height_-1-5, foor);
        window_x_factory->sync ();
}

Interaction_X_Implementations::XTextChoice_i::XTextChoice_i (Component_ptr c,
                                                             string n,
                                                             Group_ptr g)
                : XPart_i (c,n,g), ClassType (Latin1("XTextChoice_i")),
                  content (this, Latin1("content"), Latin1(""))
{                        
        XGroup_ptr gp = dynamic_cast<XGroup_ptr> (g);
        text = window_x_factory->newText (this);
        text->addItem (content.value().trimends(), font);
        width_ = text->width ()+10;
        height_ = text->height ()+10;
        gcwhite = window_x_factory->newGC (this, screen);
        gcwhite->setForeground (0xffffffff);
        gcwhite->setBackground (0);
        textdata = Latin1 ("");
        window_ = window_x_factory->newXWindow (this, Latin1("win"),
                                                gp->getWindow(),
                                                50, 10,
                                                300,
                                                50);

        window_->onExpose (call (this, &XTextChoice_i::redraw));
        window_->onKeyPress (call (this, &XTextChoice_i::keypressed));
}


void
Interaction_X_Implementations::XTextChoice_i::redraw (XCallback_i *cb)
{
        window_->drawRectangle (gc, 0, 0, window_->width()-1,
                               window_->height()-1);
        window_->fillRectangle (gcwhite,
                               1, 1, window_->width()-2,
                               window_->height()-2);
        
        text = window_x_factory->newText (this);
        text->addItem (textdata, font);
        
        window_->drawText (gc, text, 10,10+text->height()-1,foor);
        
        window_x_factory->sync ();        
}

void
Interaction_X_Implementations::XTextChoice_i::keypressed (XCallback_i *cb)
{
        unichar ascii  = window_x_factory->keycodeToUnichar (cb->event.xkey.keycode);
        cerr << "Keypressed with state=" << cb->event.xkey.state
             << " keycode=" << cb->event.xkey.keycode
             << " unicode=" <<  ascii <<  endl;
        if (ascii==8)
                textdata = textdata.substring (0, textdata.size()-1);
        else                
                textdata = textdata+ascii;
        redraw (cb);
}

Interaction_X_Implementations::InteractionXFactory_i::InteractionXFactory_i () 
                : Factory_i (0,Latin1("interactionxfactory")),
                  ClassType (Latin1("InteractionXFactory"))
{
}

Interaction_X_Interfaces::XAtomicGroup_ptr
Interaction_X_Implementations::InteractionXFactory_i::newXRootGroup (Component_ptr composite,
                                                                     string name)
{
        return new XAtomicGroup_i (composite, name, NULL);
}

Interaction_Interfaces::AtomicMessage_ptr
Interaction_X_Implementations::InteractionXFactory_i::newAtomicMessage (Component_ptr composite,
                                                                        string name,
                                                                        Group_ptr g)
{
        return new XAtomicMessage_i (composite, name, g);
}

                
Interaction_Interfaces::ElementMessage_ptr
Interaction_X_Implementations::InteractionXFactory_i::newElementMessage (Component_ptr composite,
                                                                         string name,
                                                                         Group_ptr g)
{
        return NULL;
}


Interaction_Interfaces::TextMessage_ptr
Interaction_X_Implementations::InteractionXFactory_i::newTextMessage (Component_ptr composite,
                                                                      string name,
                                                                      Group_ptr g)
{
        return NULL;
}

Interaction_Interfaces::ElementChoice_ptr
Interaction_X_Implementations::InteractionXFactory_i::newElementChoice (Component_ptr composite,
                                                                        string name,
                                                                        Group_ptr g)
{
        return NULL;
}

Interaction_Interfaces::TextChoice_ptr
Interaction_X_Implementations::InteractionXFactory_i::newTextChoice (Component_ptr composite,
                                                                     string name,
                                                                     Group_ptr g)
{
        return new XTextChoice_i (composite, name, g);
}

Interaction_Interfaces::AtomicFlowControl_ptr
Interaction_X_Implementations::InteractionXFactory_i::newAtomicFlowControl (Component_ptr composite,
                                                                            string name,
                                                                            Group_ptr g)
{
        return NULL;
}


Interaction_Interfaces::AtomicGroup_ptr
Interaction_X_Implementations::InteractionXFactory_i::newAtomicGroup (Component_ptr composite,
                                                                      string name,
                                                                      Group_ptr g)
{
        return new XAtomicGroup_i (composite, name, g);
}

Interaction_Interfaces::TextEdit_ptr
Interaction_X_Implementations::InteractionXFactory_i::newTextEdit (Component_ptr composite,
                                                                   string name,
                                                                   Group_ptr g,
                                                                   TextModel_ptr model)
{
        return NULL;        
}


Interaction_Interfaces::TextView_ptr
Interaction_X_Implementations::InteractionXFactory_i::newTextView (Component_ptr composite,
                                                                   string name,
                                                                   Group_ptr g,
                                                                   TextModel_ptr model)
{
        return NULL;        
}















