// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _Interaction_X_Implementations_hh
#define _Interaction_X_Implementations_hh

#include"Interaction_X_Interfaces.hh"
#include"Component_Implementations.hh"
#include"Window_X_Implementations.hh"

namespace Interaction_X_Implementations
{
        using namespace Interaction_X_Interfaces;
        using namespace Component_Implementations;
        using namespace Window_X_Implementations;

        class XPart_i : public virtual XPart,
                        public Component_i
        {
                private:
                
                Group_ptr group_;

                protected:
                
                XWindow_ptr window_parent_, window_;
                bool active_;
                
                String foreground;
                String background;                
                
                XScreen_ptr screen;
                Orientation foor;
                XFont_ptr   font;
                XGC_ptr     gc;
                
                public:

                XPart_i (Component_ptr c,
                         string n,
                         Group_ptr g);

                Group_ptr parent ();
                void moveToGroup (Group_ptr g);
                void activate ();
                void deactivate ();
                void editPart (Part *p, string s) {}
                void useEditor (Part *p) {}
                
                
                XWindow_ptr getWindow ();
        };

        class XAtomicMessage_i : public virtual XAtomicMessage,
                                 public XPart_i
        {
                XWindow_ptr window_;
                XText_ptr   text;
                int width_, height_;
                
                String content;
                
                public:
                
                XAtomicMessage_i (Component_ptr c,
                                  string n,
                                  Group_ptr g);

                void redraw (XCallback_i *cb);
        };

        class XTextChoice_i : public virtual XTextChoice,
                              public XPart_i
        {
                XWindow_ptr window_;
                XText_ptr   text;
                int width_, height_;
                XGC_ptr gcwhite;
                
                String content;
                string textdata;
                
                public:
                
                XTextChoice_i (Component_ptr c,
                               string n,
                               Group_ptr g);

                void redraw (XCallback_i *cb);
                void keypressed (XCallback_i *cb);
                string getText ()
                {
                        return textdata;
                }
                
                void setText (string t) 
                {
                        textdata = t;
                }
                
        };
        
        class XAtomicGroup_i : public virtual XAtomicGroup,
                               public XPart_i
        {
                public:

                XAtomicGroup_i (Component_ptr c,
                                const string n,
                                Group_ptr g);
                
                XAtomicGroup_i (Component_ptr c,
                                const string n);

                void addPart (Part_ptr part);
                void dropPart (Part_ptr part);                
                void focus (Part_ptr part);
                void unfocus (Part_ptr part);                
        };
        
        /*
        class XFlowControl_i : public virtual XFlowControl,
                               public XPart_i
        {
                XWindow_ptr window_;
                XText_ptr   text;
                int width_, height_;
                Action_ptr callback_;
                
                String message;
                
                public:
                
                XButton_i (Component_ptr c,
                           string n,
                           Group_ptr g);
                void onAction (Action_ptr cb);
                void doAction (XCallback_i *cb);                
                void redraw (XCallback_i *cb);
        };
        */

        class InteractionXFactory_i : public virtual InteractionXFactory,
                                      public Factory_i
        {
                public:
                
                InteractionXFactory_i ();
                
                XAtomicGroup_ptr newXRootGroup (Component_ptr container,
                                                string name);
                
                AtomicMessage_ptr newAtomicMessage (Component_ptr container,
                                                    string name,
                                                    Group_ptr g);
                
                ElementMessage_ptr newElementMessage (Component_ptr container,
                                                      string name,
                                                      Group_ptr g);
                
                TextMessage_ptr newTextMessage (Component_ptr container,
                                                string name,
                                                Group_ptr g);
                
                ElementChoice_ptr  newElementChoice (Component_ptr container,
                                                     string name,
                                                     Group_ptr g);                
                
                TextChoice_ptr  newTextChoice (Component_ptr container,
                                               string name,
                                               Group_ptr g);
                
                AtomicFlowControl_ptr newAtomicFlowControl (Component_ptr container,
                                                            string name,
                                                            Group_ptr g);
                
                
                AtomicGroup_ptr newAtomicGroup (Component_ptr container,
                                                string name,
                                                Group_ptr g);

                TextEdit_ptr newTextEdit (Component_ptr container,
                                          string name,
                                          Group_ptr g,
                                          TextModel_ptr model);
                
                TextView_ptr newTextView (Component_ptr container,
                                          string name,
                                          Group_ptr g,
                                          TextModel_ptr model);
                
                ListView_ptr newListView (Component_ptr composite,
                                          string name,
                                          Group_ptr g,
                                          ListModel_ptr)
                {
                        return NULL;
                }
                
        };

        extern InteractionXFactory_i *interaction_x_factory; 
};

#endif











