// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _Interaction_X_Interfaces_hh
#define _Interaction_X_Interfaces_hh

#include "Interaction_Interfaces.hh"
#include "Window_X_Interfaces.hh"

namespace Interaction_X_Interfaces
{
        using namespace Interaction_Interfaces;
        using namespace Window_X_Interfaces;        
        
        class InteractionXFactory;
        typedef InteractionXFactory *InteractionXFactory_ptr;
        
        class XPart;
        typedef XPart *XPart_ptr;
        class XMessage;
        typedef XMessage *XMessage_ptr;
        class XAtomicMessage;
        typedef XAtomicMessage *XAtomicMessage_ptr;
        class XElementMessage;
        typedef XElementMessage *XElementMessage_ptr;
        class XTextMessage;
        typedef XTextMessage *XTextMessage_ptr;

        class XGroup;        
        typedef XGroup *XGroup_ptr;
        class XAtomicGroup;        
        typedef XAtomicGroup *XAtomicGroup_ptr;

        class XFlowControl;
        typedef XFlowControl *XFlowControl_ptr;
        class XAtomicFlowControl;
        typedef XAtomicFlowControl *XAtomicFlowControl_ptr;

        class XChoice;
        typedef XChoice *XChoice_ptr;
        class XElementChoice;
        typedef XElementChoice *XElementChoice_ptr;                
        class XTextChoice;
        typedef XTextChoice *XTextChoice_ptr;

        class XPart : public virtual Part
        {
                public:

                virtual XWindow_ptr getWindow () = 0;
        };

        
        class XMessage : public virtual Message,
                         public virtual XPart
        {
        };

        class XAtomicMessage : public virtual AtomicMessage,
                               public virtual XMessage
        {
        };

        class XElementMessage : public virtual ElementMessage,
                                public virtual XMessage
        {
        };
        
        class XTextMessage : public virtual TextMessage,
                             public virtual XMessage
        {
        };        
        

        
        class XChoice : public virtual Choice,
                        public virtual XPart
        {
        };

        class XElementChoice : public virtual ElementChoice,
                               public virtual XChoice
        {
        };

        
        class XTextChoice : public virtual TextChoice,
                            public virtual XChoice
        {
        };
        
        class XFlowControl : public virtual FlowControl,
                             public virtual XPart
        {
        };
        
        class XAtomicFlowControl : public virtual AtomicFlowControl,
                                   public virtual XFlowControl
        {
        };
        
        class XGroup : public virtual Group,
                       public virtual XPart
        {
        };

        class XAtomicGroup : public virtual AtomicGroup,
                             public virtual XGroup
        {
        };
        
        class InteractionXFactory : public virtual InteractionFactory
        {
                public:

                virtual XAtomicGroup_ptr newXRootGroup (Component_ptr container,
                                                        string name) = 0;
        };
};

#endif






/*

        class XTableCallback : public virtual TableCallback
        {
                public:

                virtual void setRow (int r) = 0;
                virtual void setRowState (TableRowState ts) = 0;
                virtual XPart_ptr getXColElement (int c) = 0;
        };
        typedef XTableCallback *XTableCallback_ptr;





*/

