// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#include"Window_Http_Implementations.hh"
//#include"zlib.h"

using namespace Window_Http_Implementations;

Window_Http_Implementations::WindowHttpFactory_i *
Window_Http_Implementations::window_http_factory = new Window_Http_Implementations::WindowHttpFactory_i;

long Window_Http_Implementations::Window_i::id_count_ = 0;

Window_Http_Implementations::Window_i::Window_i (Component_ptr composite,
                                                 string n,
                                                 HttpWindow_ptr parent,
                                                 HttpScreen_ptr screen)
                : Component_i (composite, n), ClassType (Latin1("WebWindow"))
{
        expose_ = NULL;
        
        parent_ = parent;
        if (parent!=NULL)
        {
                parent->addWindowChild (this);
                screen_ = parent->getScreen ();
        }
        else screen_ = screen;
        id_count_++;
        string nn;
        nn.asString ((int)id_count_);
        id_ = n+nn;

        setValue (Latin1("objectid"), id ());        
        setValue (Latin1("edithook"), Latin1(" "));

        if (screen_!=NULL) screen_->registerWindow (this);
        is_mapped_ = false;
}

void Window_Http_Implementations::Window_i::setMappedValue (string value)
{
        mapped_value_ = value;
}

void Window_Http_Implementations::Window_i::setUnmappedValue (string value)
{
        unmapped_value_ = value;
}

void Window_Http_Implementations::Window_i::focusWindow ()
{
        if (parent_!=NULL)
        {
                parent_->raise (this);
        }
}

void Window_Http_Implementations::Window_i::unfocusWindow ()
{
        if (parent_!=NULL)
        {
                parent_->lower (this);
        }
}

void Window_Http_Implementations::Window_i::mapWindow ()
{
        is_mapped_ = true;
}

void Window_Http_Implementations::Window_i::unmapWindow ()
{
        is_mapped_ = false;
}

void Window_Http_Implementations::Window_i::onExpose (HttpCallback_ptr cb)
{
        expose_ = cb;
}

void Window_Http_Implementations::Window_i::onMessage (string msg, HttpCallback_ptr cb)
{
        messages_[msg] = cb;
}

void Window_Http_Implementations::Window_i::addWindowChild (Window_ptr c)
{
        children_.push_back (c);
}

void Window_Http_Implementations::Window_i::dropWindowChild (Window_ptr c)
{
        list<Window_ptr>::iterator i;
        for (i=children_.begin(); i!=children_.end(); ++i)
        {
                if (*i==c)
                {                        
                        children_.erase (i);
                        return;
                }
        }        
}

void Window_Http_Implementations::Window_i::dropAllChildren ()
{
        children_.clear ();
}

void Window_Http_Implementations::Window_i::moveToParent (Window_ptr p)
{
        HttpWindow_ptr hp = dynamic_cast<HttpWindow_ptr>(p);
        if (parent_!=NULL) parent_->dropWindowChild (this);
        hp->addWindowChild (this);
        parent_ = hp;
}

Window_ptr Window_Http_Implementations::Window_i::parent ()
{
        return parent_;
}

list<Window_ptr>&
Window_Http_Implementations::Window_i::children ()
{
        return children_;
}

string
Window_Http_Implementations::Window_i::render (int indent)
{
        map<string,string> values;
        map<string,int> ordering;
        list<string> keys;
        string text = Latin1(""); //unmapped_value_;

        if (is_mapped_==true)
        {
                //                fprintf (stderr, ">");
                //    for (int j=0; j<indent; ++j) fprintf (stderr, " ");
                //id().print (stderr, "", " ");
                //getNameTag().print (stderr, "", "\n");                
                
                if (expose_!=NULL) expose_->invoke ();
                int count=0;
                list<Window_ptr>::iterator i;
                // Render all children and store them.
                for (i=children_.begin(); i!=children_.end(); ++i)
                {
                        HttpWindow_ptr c = dynamic_cast<HttpWindow_ptr> (*i);
                        string n = c->getNameTag ();
                        if (n.size()==0) n = c->path ();
                        values[n] = c->render (indent+8);
                        ordering[n] = count++;
                        keys.push_back (n);
                }
                // Now insert the children in the text.
                text = substituteChildren (mapped_value_, values, ordering, keys);
                // Insert static values
                text = substitute (text, Latin1("%|"), Latin1("|%"), staticvalues_);
        }

        /*
        for (int j=0; j<indent; ++j) fprintf (stderr, " ");
        text.print(stderr, "", "\n\n"); */
        
        return text;
}

void Window_Http_Implementations::Window_i::handleMessage (string msg)
{
        if (is_mapped_==false) return;
        HttpCallback_ptr cb = messages_[msg];
        if (cb!=NULL)
        {
                cb->setMessage (msg);
                cb->invoke ();
        }
        else
        {
                cb = messages_[Latin1("")];                
                if (cb!=NULL)
                {
                        cb->setMessage (msg);
                        cb->invoke ();
                }                
        }
}

void Window_Http_Implementations::Window_i::raise (Window_ptr w)
{
        list<Window_ptr>::iterator i;
        for (i=children_.begin(); i!=children_.end(); ++i)
        {
                if (*i == w)
                {
                        children_.erase (i);
                        children_.push_front (w);
                        break;
                }
        }
}

void Window_Http_Implementations::Window_i::lower (Window_ptr w)
{
        list<Window_ptr>::iterator i;
        for (i=children_.begin(); i!=children_.end(); ++i)
        {
                if (*i == w)
                {
                        children_.erase (i);
                        children_.push_back (w);
                        break;
                }
        }        
}

void Window_Http_Implementations::Window_i::addEditHook (string p,
                                                         HttpCallback_ptr appearance_cb,
                                                         HttpCallback_ptr content_cb)
{
        String edit_hook_appearance (this, Latin1("edit_hook_appearance"), Latin1 (""));
        string hook = edit_hook_appearance.value();
        map<string,string> m;
        m[Latin1("objectid")] = id ();
        m[Latin1("name")] = p;
        hook = substitute (hook, Latin1("%|"), Latin1("|%"), m);
        setValue (Latin1("edithook"), hook);        
        onMessage (Latin1("appearancehook"), appearance_cb);
        onMessage (Latin1("contenthook"), content_cb);        
}
        
void Window_Http_Implementations::Window_i::setNameTag (string name)
{
        name_tag_ = name;
}

string Window_Http_Implementations::Window_i::getNameTag ()
{
        return name_tag_;
}

void Window_Http_Implementations::Window_i::setValue (string key, string value)
{
        staticvalues_[key]=value;
}

string  Window_Http_Implementations::Window_i::getValue (string key)
{
        return staticvalues_[key];
}

HttpScreen_ptr Window_Http_Implementations::Window_i::getScreen ()
{
        return screen_;
}

Window_Http_Implementations::HttpGraphicsServer_i::HttpGraphicsServer_i (Component_ptr composite,
                                                           string name,
                                                           int port)
                : Component_i (composite, name), ClassType (Latin1("HttpGraphicsServer_i")), port_ (port),
                  sitenotactive_ (this, Latin1("sitenotactive"), Latin1("Site is not active!")),
                  screenframe_ (this, Latin1("screenframe"), Latin1("No database installed.")),
                  connect_cb_ (NULL)
{
        server = new Tcp::TCPServer (port_);
        server->useHandler (this);
        clean_up_.start (call (this, &HttpGraphicsServer_i::cleanupThread));
}

void
Window_Http_Implementations::HttpGraphicsServer_i::cleanupThread (Callback_i *cb)
{
        time_t last = time(NULL), curr;
        list<HttpScreen_i*> toberemoved;
        
        //        fprintf (stderr, "Victor commencer premier fois...\n");
        while (clean_up_.running())
        {                
                toberemoved.clear();
                do
                {
                        sleep (10000);
                        curr = time(NULL);
                } while (difftime (curr, last)<10000);
                last = curr;
                /*
                fprintf (stderr, "Victor interieur commencer...\n");
                map<string,HttpScreen_i*>::iterator i;

                for (i=screens.begin(); i!=screens.end(); ++i)
                {
                        HttpScreen_i *s = (*i).second;
                        if (s->hasTimedOut ())
                        {
                                toberemoved.push_back (s);
                        }
                }

                disconnect_cb_->setHttpGraphicsServer (this);
                list<HttpScreen_i*>::iterator j;
                for (j=toberemoved.begin(); j!=toberemoved.end(); ++j)
                {
                        disconnect_cb_->setHttpScreen (*j);
                        disconnect_cb_->invoke ();
                }
                
                fprintf (stderr, "Victor fini.\n");
                */
        }
}

void
Window_Http_Implementations::HttpGraphicsServer_i::onConnectRequest (HttpConnectRequest_ptr cb)
{
        connect_cb_ = cb;
}

void
Window_Http_Implementations::HttpGraphicsServer_i::onDisconnectRequest (HttpConnectRequest_ptr cb)
{
        disconnect_cb_ = cb;
}

void Window_Http_Implementations::HttpGraphicsServer_i::registerScreen (HttpScreen_ptr screen)
{
        screens[screen->id()] = dynamic_cast<HttpScreen_i*>(screen);
        screen->id().print (stderr, "Registered screen=", "\n");
}

void Window_Http_Implementations::HttpGraphicsServer_i::deregisterScreen (HttpScreen_ptr screen)
{        
        screen->id().print (stderr, "Deregistering screen=", "\n");
        map<string, HttpScreen_i*>::iterator i;
        i = screens.find (screen->id());
        screens.erase (i);
}

char *gzheader = "HTTP/1.1 200 OK\n"
"Server: Unknown\n"
// "Content-Length: %d\n"
"Connection: close\n"
"Content-Type: text/html;\n" // charset=utf-8\n
"Content-Encoding: x-gzip\n\n";

char *header = "HTTP/1.1 200 OK\n"
"Server: Unknown\n"
// "Content-Length: %d\n"
"Connection: close\n"
"Content-Type: text/html;\n\n"; // charset=utf-8\n\n

char *gifheader = "HTTP/1.1 200 OK\n"
"Server: Unknown\n"
//"Content-Length: %d\n"
"Connection: close\n"
"Content-Type: image/gif;\n\n"; // charset=utf-8\n\n

char *cssheader = "HTTP/1.1 200 OK\n"
"Server: Unknown\n"
//"Content-Length: %d\n"
"Connection: close\n"
"Content-Type: text/css;\n\n"; // charset=utf-8\n\n

Mutex single (Latin1("single"));

void Window_Http_Implementations::HttpGraphicsServer_i::handle (Client *c)
{
        single.lock();
        map<string,string> values;
        HttpWindow_ptr root = NULL;
        char inn[65536];
        int n = c->readdata (inn, 60000);
        inn[n] = 0;
        parseRequest (inn, values);
        string cookie = values[Latin1("cookie")];
        string state = values[Latin1("state")];
        cookie.print (stderr, "COOKIE=", "\n");
        state.print (stderr, "STATE=", "\n");        
        
        map<string,HttpScreen_i*>::iterator i = screens.find (cookie);
        if (i!=screens.end())
        {
                fprintf (stderr, "Found screen.\n");
                root = (*i).second->handleRequest (values);
        }
        
        if (root!=NULL)
        {
                string s = root->render ();
                char page[65536];
                s.storeAsLatin1 (page);

                /*
                write (fd, gzheader, strlen (gzheader));
                gzFile gz = gzdopen (dup(fd), "w");
                int ss = gzwrite (gz, page, strlen (page));
                gzclose (gz); */
                
                c->writedata (header, strlen (header));
                c->writedata (page, strlen (page));
                c->writedata ("\n", 1);
        }
        else
        if (cookie==Latin1("Html"))
        {
                //                state.print (stderr, "file=","\n");
                char *buffer, head[256]; // ATTENTION
                int n;
                getFile (state, &buffer, &n);
                // fprintf (stderr, "Got file with %d bytes.\n", n);
                
                char *h = header;
                string ending = state.substring (state.size()-4);
                //                ending.print (stderr, "Ending ","\n");
                if (ending==Latin1(".css"))
                        h = cssheader;
                else
                        if (ending==Latin1(".gif"))
                                h = gifheader;
                
                sprintf (head, h, n);
                c->writedata (head, strlen (head));
                c->writedata (buffer, n);
        }
        else    
        {
                string v = sitenotactive_.value ();

                if (connect_cb_!=NULL)
                {
                        connect_cb_->setHttpGraphicsServer (this);
                        connect_cb_->setApplication (cookie);
                        //  cookie.print (stderr, "COOKIE=","\n");
                        connect_cb_->invoke ();
                        HttpScreen_ptr screen = connect_cb_->getHttpScreen ();
                        map<string,string> values;
                        values[Latin1("screen")] = screen->id();
                        v = substitute (screenframe_.value(), Latin1("%|"), Latin1("|%"), values);
                }
                
                char page[65536];
                v.storeAsLatin1 (page);
                c->writedata (header, strlen (header));
                c->writedata (page, strlen (page));
                c->writedata ("\n", 1);
        }
        single.unlock ();
}


void
Window_Http_Implementations::HttpGraphicsServer_i::getFile (string fn, char **buf, int *n)
{
        *buf = file_cache_buf_[fn];
        if (*buf==NULL)
        {
                FILE *f;
                char name[256];  // ATTENTION
                *buf = new char[65536];
                fn.storeAsLatin1 (name);
                //  fprintf (stderr, "Loading file >%s< \n", name);
                f = fopen (name, "r");
                if (f!=NULL)
                {                        
                        *n = fread (*buf, 1, 65535, f);
                        fclose (f);
                        file_cache_buf_[fn] = *buf;
                        file_cache_n_[fn] = *n;
                }
        }
        
        *n = file_cache_n_[fn];
}


Window_Http_Implementations::HttpScreen_i::HttpScreen_i (HttpGraphicsServer_ptr composite, string name, string id) 
                : Component_i (composite, name), ClassType (Latin1("HttpScreen_i"))
{
        id_ = id;
        count_ = 0;
        root_window = window_http_factory->newHttpWindowOnScreen (this, Latin1("root"), this);
        root_window->setMappedValue (Latin1("%[CHILDREN]%"));
        root_window->mapWindow ();
        composite->registerScreen (this);
        last_time_viewed_ = time (NULL);
}

bool
Window_Http_Implementations::HttpScreen_i::hasTimedOut ()
{
        time_t current = time(NULL);
        double d = difftime (current, last_time_viewed_);
        return (d>10.0);
}

void
Window_Http_Implementations::HttpScreen_i::touch ()
{
        last_time_viewed_ = time (NULL);
}

Component_ptr
Window_Http_Implementations::HttpScreen_i::cleanup ()
{        
        HttpGraphicsServer_i *server = dynamic_cast<HttpGraphicsServer_i*>(composite());
        server->deregisterScreen (this);
        return this;
}

string
Window_Http_Implementations::HttpScreen_i::id ()
{
        return id_;
}

void
Window_Http_Implementations::HttpScreen_i::registerWindow (HttpWindow_ptr window)
{
        Window_i *w = dynamic_cast<Window_i*>(window);
        id_to_object_[w->id()] = w;
        //        w->id().print(stderr, "Registered window ", "\n");
}

HttpWindow_ptr
Window_Http_Implementations::HttpScreen_i::getRootWindow ()
{
        return root_window;
}

HttpWindow_ptr
Window_Http_Implementations::HttpScreen_i::handleRequest (map<string,string> &values)
{
        map<string,string> buttons;
        map<string,string>::iterator i;
        bool done=false;

        touch ();
        i = values.find (Latin1("appearancehook"));
        if (i!=values.end ())
        {
                Window_i *w = id_to_object_[(*i).second];
                w->handleMessage (Latin1("appearancehook"));
                done=true;
        }
        i = values.find (Latin1("contenthook"));
        if (!done && i!=values.end ())
        {
                Window_i *w = id_to_object_[(*i).second];
                w->handleMessage (Latin1("contenthook"));
                done=true;
        }

        if (!done)
        {
                for (i=values.begin(); i!=values.end(); ++i)
                {
                        Window_i *w = id_to_object_[(*i).first];
                        if (w!=NULL)
                        {
                                if ((*i).first.substring(0,2)==Latin1("zw"))
                                {
                                        buttons[(*i).first] = (*i).second;
                                }
                                else
                                {
                                        w->handleMessage ((*i).second);
                                }
                        }
                }
                for (i=buttons.begin(); i!=buttons.end(); ++i)
                {
                        Window_i *w = id_to_object_[(*i).first];
                        if (w!=NULL)
                        {
                                w->handleMessage ((*i).second);
                        }
                }
        }
        
        string countstate;
        countstate.asString (count_);
        count_++;
        root_window->setValue (Latin1("state"), countstate);        
        return root_window;
}


HttpGraphicsServer_ptr
Window_Http_Implementations::HttpConnectRequest_i::getHttpGraphicsServer ()
{
        return server_;
}

void
Window_Http_Implementations::HttpConnectRequest_i::setHttpGraphicsServer (HttpGraphicsServer_ptr server)
{
        server_ = server;
}

HttpScreen_ptr
Window_Http_Implementations::HttpConnectRequest_i::getHttpScreen ()
{
        return screen_;
}

void
Window_Http_Implementations::HttpConnectRequest_i::setHttpScreen (HttpScreen_ptr screen)
{
        screen_  = screen;
}

string
Window_Http_Implementations::HttpConnectRequest_i::getApplication ()
{
        return application_;
}

void
Window_Http_Implementations::HttpConnectRequest_i::setApplication (string app)
{
        application_  = app;
}


Window_Http_Implementations::WindowHttpFactory_i::WindowHttpFactory_i () 
                : Factory_i (NULL, Latin1("httpfactory")), ClassType (Latin1("HttpFactory"))
{
}

HttpGraphicsServer_ptr
Window_Http_Implementations::WindowHttpFactory_i::newHttpGraphicsServer (Component_ptr composite,
                                                        string n,
                                                        int port)
{
        HttpGraphicsServer_i *server = new HttpGraphicsServer_i (composite, n, port);
        return server; 
};

HttpScreen_ptr
Window_Http_Implementations::WindowHttpFactory_i::newHttpScreen (HttpGraphicsServer_ptr composite,
                                                       string n,
                                                       string id)
{
        HttpScreen_i *screen = new HttpScreen_i (composite, n, id);
        return screen;
};

Window_ptr
Window_Http_Implementations::WindowHttpFactory_i::newWindow (Component_ptr composite,
                                                             string n,
                                                             Window_ptr parent)
{
        return newHttpWindow (composite, n, parent);
};

HttpWindow_ptr
Window_Http_Implementations::WindowHttpFactory_i::newHttpWindow (Component_ptr composite,
                                                                 string n,
                                                                 Window_ptr parent)
{
        Window_i *w = new Window_Http_Implementations::Window_i (composite,
                                                                 n,
                                                                 dynamic_cast<HttpWindow_ptr>(parent),
                                                                 NULL);
        return w;
};

HttpWindow_ptr
Window_Http_Implementations::WindowHttpFactory_i::newHttpWindowOnScreen (Component_ptr composite,
                                                                         string n,
                                                                         HttpScreen_ptr screen)
{
        Window_i *w = new Window_Http_Implementations::Window_i (composite,
                                                                 n,
                                                                 NULL,
                                                                 screen);        
        return w;
};















