// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

// Copyright (C) 1999 Fredrik hrstrm

#ifndef _Window_Http_Implementations_hh
#define _Window_Http_Implementations_hh

#include<list>
#include<map>
#include<ctime>
#include"string.hh"

#include"Component_Implementations.hh"
#include"Window_Http_Interfaces.hh"
#include"tcp.hh"

namespace Window_Http_Implementations
{        
        using namespace Window_Http_Interfaces;
        using namespace Component_Implementations;
        using namespace Tcp;
        
        class HttpGraphicsServer_i;
        class HttpScreen_i;
        class HttpConnectRequest_i;
        class Window_i;
        class WindowHttpFactory_i;

        class HttpGraphicsServer_i : public virtual HttpGraphicsServer,
                              public virtual Handler,
                              public Component_i
        {
                TCPServer *server;
                map<string, HttpScreen_i*> screens;
                Thread clean_up_;
                
                int port_;
                String sitenotactive_, screenframe_;
                HttpConnectRequest_ptr connect_cb_, disconnect_cb_;

                map<string,char*> file_cache_buf_;
                map<string,int> file_cache_n_;                
                
                public:
                
                HttpGraphicsServer_i (Component_ptr composite,
                               string name,
                               int port);

                void cleanupThread (Callback_i *cb);
                
                void onConnectRequest (HttpConnectRequest_ptr cb);
                void onDisconnectRequest (HttpConnectRequest_ptr cb);
                void registerScreen (HttpScreen_ptr screen);
                void deregisterScreen (HttpScreen_ptr screen);
                void handle (Client *c);
                void getFile (string filename, char **buf, int *n);
        };
        
        class HttpScreen_i : public virtual HttpScreen,
                             public Component_i 
        {
                // Unique identification name.
                string id_;
                // A screen has a root window.
                HttpWindow_ptr root_window;
                // Give all objects belonging to this screen a short name.
                map<string,Window_i*> id_to_object_;
                // Used to check if user is backstepping.
                int count_;
                // Last time this screen was viewed.
                time_t last_time_viewed_;

                public:
                
                HttpScreen_i (HttpGraphicsServer_ptr composite, string name, string id);
                bool hasTimedOut ();
                void touch ();
                Component_ptr cleanup ();
                
                string id ();
                HttpWindow_ptr getRootWindow ();
                void registerWindow (HttpWindow_ptr window);
                HttpWindow_ptr handleRequest (map<string,string> &values);
        };

        class HttpConnectRequest_i : public virtual HttpConnectRequest,
                                     public Callback_i
        {
                HttpGraphicsServer_ptr server_;
                HttpScreen_ptr screen_;
                string application_;

                public:
                HttpConnectRequest_i (Component_i *composite,
                                      SysCaller *sc)
                                : Callback_i (composite, sc), ClassType (Latin1("HttpConnectRequest_i"))
                {}

                HttpGraphicsServer_ptr getHttpGraphicsServer ();
                void setHttpGraphicsServer (HttpGraphicsServer_ptr server);
                HttpScreen_ptr getHttpScreen ();
                void setHttpScreen (HttpScreen_ptr screen);
                string getApplication ();                
                void setApplication (string app);
        };
                
        struct HttpCallback_i : public virtual HttpCallback,
                                public Callback_i
        {
                string message_;                
                
                HttpCallback_i (Component_i *composite,
                                SysCaller *sc)
                                : Callback_i (composite, sc), ClassType (Latin1("HttpCallback_i"))
                {}

                string getMessage () 
                {
                        return message_;
                }

                void setMessage (string m)
                {
                        message_ = m;
                }                
        };
        
        struct Window_i : public HttpWindow,
                          public Component_i
        {
                Window_i (Component_ptr composite,
                          string n,
                          HttpWindow_ptr parent,
                          HttpScreen_ptr screen);
                
                void setMappedValue (string value);
                void setUnmappedValue (string value);

                void focusWindow ();
                void unfocusWindow ();
                void mapWindow ();
                void unmapWindow ();

                void onExpose (HttpCallback *cb);
                void onMessage (string msg, HttpCallback_ptr cb);
                list<Window_ptr> &children ();
                void addWindowChild (Window_ptr c);
                void dropWindowChild (Window_ptr p);
                void dropAllChildren ();
                void moveToParent (Window_ptr c);
                Window_ptr parent ();

                string render (int indent=0);
                string id () { return id_; }
                void setNameTag (string name);
                string getNameTag ();
                void setValue (string key, string value);
                string getValue (string key);
                HttpScreen_ptr getScreen ();                
                
                void handleMessage (string msg);
                void raise (Window_ptr w);
                void lower (Window_ptr w);
                void addEditHook (string p,
                                  HttpCallback_ptr appearance_cb,
                                  HttpCallback_ptr content_cb);                

                private:

                string mapped_value_, unmapped_value_;

                bool is_mapped_;
                HttpCallback_ptr expose_;
                map<string,HttpCallback_ptr> messages_;

                map<string,string> staticvalues_;

                list<Window_ptr> children_;
                HttpScreen_ptr screen_;
                HttpWindow_ptr parent_;
                // A unique id generated so messages can be sent
                // directly to this object.
                string id_;
                // The children windows of of a window must all have
                // different names, so substitution works all right.
                // The name_tag is from the beginning the same as the
                // object id. A composite usually sets it to the object's
                // last path name with the composites name appended.
                string name_tag_;

                static long id_count_;
        };
        
        struct WindowHttpFactory_i : public virtual WindowHttpFactory,
                                     public Factory_i
        {
                WindowHttpFactory_i ();

                HttpGraphicsServer_ptr newHttpGraphicsServer (Component_ptr composite,
                                                string n,
                                                int port);

                HttpScreen_ptr newHttpScreen (HttpGraphicsServer_ptr composite,
                                              string n,
                                              string id);
                
                Window_ptr newWindow (Component_ptr composite,
                                      string n,
                                      Window_ptr parent);

                HttpWindow_ptr newHttpWindow (Component_ptr composite,
                                              string n,
                                              Window_ptr parent);

                HttpWindow_ptr newHttpWindowOnScreen (Component_ptr composite,
                                                      string n,
                                                      HttpScreen_ptr screen);
        };

        extern WindowHttpFactory_i *window_http_factory;        
};

#endif









