// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _Window_Http_Interfaces_hh
#define _Window_Http_Interfaces_hh

#include<list>
#include"string.hh"

#include"Component_Interfaces.hh"
#include"Window_Interfaces.hh"

namespace Window_Http_Interfaces
{
        using namespace Window_Interfaces;
        
        class HttpCallback;
        typedef HttpCallback *HttpCallback_ptr;
        class HttpWindow;
        typedef HttpWindow *HttpWindow_ptr;
        class HttpGraphicsServer;        
        typedef HttpGraphicsServer *HttpGraphicsServer_ptr;        
        class HttpScreen;        
        typedef HttpScreen *HttpScreen_ptr;
        class HttpConnectRequest;        
        typedef HttpConnectRequest *HttpConnectRequest_ptr;
        class WindowHttpFactory;
        typedef WindowHttpFactory *WindowHttpFactory_ptr;
        
        struct HttpCallback : public virtual Callback
        {
                virtual string getMessage () = 0;
                virtual void setMessage (string msg) = 0;                
        };
        
        class HttpWindow : public virtual Window
        {
                public:
                
                virtual void setMappedValue (string value) = 0;
                virtual void setUnmappedValue (string value) = 0;
                
                virtual void onExpose (HttpCallback *cb) = 0;
                virtual void onMessage (string msg, HttpCallback_ptr cb) = 0;

                virtual string render (int indent=0) = 0;
                virtual string id () = 0;
                virtual void setNameTag (string name) = 0;
                virtual string getNameTag () = 0;
                virtual void setValue (string key, string value) = 0;
                virtual string getValue (string key) = 0;
                virtual HttpScreen_ptr getScreen () = 0;
                virtual void raise (Window_ptr w) = 0;
                virtual void lower (Window_ptr w) = 0;
                virtual void addEditHook (string p, HttpCallback_ptr appearance_cb,
                                          HttpCallback_ptr content_cb) = 0;
        };
        
        
        struct HttpGraphicsServer : public virtual GraphicsServer
        {
                virtual void onConnectRequest (HttpConnectRequest_ptr cb) = 0;
                virtual void onDisconnectRequest (HttpConnectRequest_ptr cb) = 0;
                virtual void registerScreen (HttpScreen_ptr screen) = 0;
        };

        struct HttpScreen : public virtual Screen
        {
                virtual string id () = 0;
                virtual HttpWindow_ptr getRootWindow () = 0;
                virtual void registerWindow (HttpWindow_ptr window) = 0;
                virtual HttpWindow_ptr handleRequest (map<string,string> &values) = 0;
                virtual bool hasTimedOut () = 0;
                virtual void touch () = 0;
        };

        struct HttpConnectRequest : public virtual Callback
        {
                virtual HttpGraphicsServer_ptr getHttpGraphicsServer () = 0;
                virtual void setHttpGraphicsServer (HttpGraphicsServer_ptr server) = 0;
                virtual HttpScreen_ptr getHttpScreen () = 0;
                virtual void setHttpScreen (HttpScreen_ptr screen) = 0;
                virtual string getApplication () = 0;
                virtual void setApplication (string app) = 0;                
        };

        class WindowHttpFactory : public virtual WindowFactory
        {
                public:

                virtual HttpGraphicsServer_ptr newHttpGraphicsServer (Component_ptr composite,
                                                        string n,
                                                        int port) = 0;
                
                virtual HttpScreen_ptr newHttpScreen (HttpGraphicsServer_ptr composite,
                                                      string name,
                                                      string id) = 0;

                virtual HttpWindow_ptr newHttpWindow (Component_ptr composite,
                                                      string n,
                                                      Window_ptr parent) = 0;

                virtual HttpWindow_ptr newHttpWindowOnScreen (Component_ptr composite,
                                                              string n,
                                                              HttpScreen_ptr screen) = 0;
        };
        typedef WindowHttpFactory *WindowHttpFactory_ptr;        
};

#endif









