// Copyright (C) 1999 Fredrik hrstrm
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _win_hh
#define _win_hh

#include <list>
#include "string.hh"

#include "Component_Interfaces.hh"

namespace Window_Interfaces
{
        using namespace Component_Interfaces;
        
        struct Drawable;
        typedef Drawable *Drawable_ptr;

        struct Pixmap;
        typedef Pixmap *Pixmap_ptr;                

        struct Window;
        typedef Window *Window_ptr;                

        struct WindowFactory;
        typedef WindowFactory *WindowFactory_ptr;

        struct GraphicsServer;
        typedef GraphicsServer *GraphicsServer_ptr;

        struct Screen;
        typedef Screen *Screen_ptr;

        struct Windowref
        {
                string display;
                long port;
                long id;
        };

        struct Color
        {
                Color (unsigned long p,
                        unsigned short r,
                        unsigned short g,
                        unsigned short b) 
                                : pixel (p), red (r), green (g), blue (b)
                {}
                
                unsigned long pixel;            /* pixel value */
                unsigned short red, green, blue;/* rgb values */
                char flags;                     /* DoRed, DoGreen, DoBlue */
                char pad;
        };

        struct Orientation
        {
                Orientation (float aa=1.0, float bb=0.0,
                             float cc=0.0, float dd=1.0)
                                : a(aa),b(bb),c(cc),d(dd) { }
                
                Orientation& rotate (float rads);
                Orientation& rotate90 ();
                Orientation& rotate180 ();
                Orientation& rotate270 ();
                
                Orientation& scale (float x, float y);
                Orientation& mul (Orientation o);
                
                string       asString ();
                
                float a,b,c,d;
        };
        
        struct Drawable : public virtual Component
        {
        };

        struct Pixmap : public virtual Drawable
        {
        };

        struct Window : public virtual Drawable
        {
                virtual void focusWindow () = 0;
                virtual void unfocusWindow () = 0;
                virtual void mapWindow () = 0;
                virtual void unmapWindow () = 0;

                virtual list<Window_ptr> &children () = 0;
                virtual void addWindowChild (Window_ptr c) = 0;
                virtual void dropWindowChild (Window_ptr c) = 0;
                virtual void dropAllChildren () = 0;
                virtual void moveToParent (Window_ptr c) = 0;
                virtual Window_ptr parent () = 0;
        };
                
        struct GraphicsServer : public virtual Component
        {
        };

        struct Screen : public virtual Component
        {
        };
        
        struct WindowFactory : public virtual Factory
        {
                virtual Window_ptr newWindow (Component_ptr composite,
                                              string n,
                                              Window_ptr parent) = 0;
        };
};


#endif














