// Copyright (C) 1999 Fredrik hrstrm
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#include"Window_X_Implementations.hh"

Window_X_Implementations::WindowXFactory_i *
Window_X_Implementations::window_x_factory = new WindowXFactory_i;

void
Window_X_Implementations::X_Window::handleEvent (x_eventstruct event)
{
        protect_events_.lock ();
        events_.push_back (event);	
        protect_events_.unlock ();
        semaphore_.post ();	
}

void Window_X_Implementations::X_Window::sendCallbacks (Callback_i *cb)
{
        x_eventstruct event;
        bool stillempty;

        for (;;)
        {
                do 
                {
                        semaphore_.wait ();
                        protect_events_.lock ();
                        stillempty = events_.empty();
                        if (stillempty==true) protect_events_.unlock ();
                }
                while (stillempty == true);

                event = events_.front ();                
                events_.pop_front ();
                protect_events_.unlock ();
                
                Window_X_Implementations::XCallback_i *c = callbacks_[event.type];
                if (c!=0)
                {
                        c->event = event;
                        c->invoke ();
                }
        }
}


void Window_X_Implementations::X_DataReceiver::lock ()
{
        protect_x_.lock ();
}

void Window_X_Implementations::X_DataReceiver::unlock ()
{
        //        if (XPending (server_)) controller_->shake();
        protect_x_.unlock ();
}

void Window_X_Implementations::X_DataReceiver::sync ()
{
        XFlush (server_);
}

Window_X_Implementations::X_DataReceiver::X_DataReceiver (string url)
                :       Component_i (0,Latin1("X_DataReceiver")),
                        ClassType (Latin1("X_DataReceiver")),
                        protect_x_ (Latin1("protect_x")),
                        protect_connection_(Latin1("protect_connection")),
                        caller_(0,string(Latin1("X_Datareceiver_thread")))                        
{ 
        char buffer[64];
        UniToUtf (url,buffer);
        protect_x_.lock ();
        server_ = XOpenDisplay (buffer);
        
        nof_screens_ = ScreenCount (server_);
        screens_ = new Window_X_Implementations::XScreen_i* [nof_screens_];
        
        protect_x_.unlock ();
        
        for (int i=0; i<nof_screens_; ++i)
        {
                screens_[i] = new Window_X_Implementations::XScreen_i (0,Latin1(""),server_,
                                                                       ScreenOfDisplay(server_, i),
                                                                       i,this);
        }
        default_screen_ = DefaultScreen (server_);
        caller_.start (call (this, &X_DataReceiver::receiveLoop));
}

Window_X_Implementations::X_DataReceiver::~X_DataReceiver ()
{
        protect_x_.lock ();
        XCloseDisplay (server_);
        protect_x_.unlock ();
}

void 
Window_X_Implementations::X_DataReceiver::addCallback (Window_X_Implementations::XWindow_i *w, 
                                                       Window_X_Implementations::XCallback_i *callback, 
                                                       int event_type)
{    
        protect_connection_.lock ();
        X_Window *win = windows_[w->drawable_];
        if (win==0)
        {
                win = new X_Window (w);
                windows_[w->drawable_] = win;	
        }
        win->callbacks_ [event_type] = callback;
        protect_connection_.unlock ();
}

void 
Window_X_Implementations::X_DataReceiver::removeCallback (Window_X_Implementations::XWindow_i *w, 
                                                          int event_type)
{    
        protect_connection_.lock ();
        X_Window *win = windows_[w->drawable_];
        if (win!=0) win->callbacks_.erase (event_type);
        protect_connection_.unlock ();
}

void 
Window_X_Implementations::X_DataReceiver::receiveLoop (Callback_i *cb)
{
        XEvent event;
        X_Window *w;
        
        while (caller_.running ())
        {
                XNextEvent (server_, &event);

                switch (event.type)
                {
                    case Expose: 
                            if (event.xexpose.count) break;
                            protect_connection_.lock();
                            w = windows_[event.xany.window];
                            protect_connection_.unlock();
                            w->handleEvent (event);
                            break;
                    case ButtonPress:
                            protect_connection_.lock();
                            w = windows_[event.xany.window];
                            protect_connection_.unlock();
                            w->handleEvent (event);
                            break;
                    case KeyPress:
                            protect_connection_.lock();
                            w = windows_[event.xany.window];
                            protect_connection_.unlock();
                            w->handleEvent (event);
                            break;
                    case EnterNotify:
                            protect_connection_.lock();
                            w = windows_[event.xany.window];
                            protect_connection_.unlock();
                            w->handleEvent (event);
                            break;
                    case LeaveNotify:
                            protect_connection_.lock();
                            w = windows_[event.xany.window];
                            protect_connection_.unlock();
                            w->handleEvent (event);
                            break;
                    case CreateNotify:
                            protect_connection_.lock();
                            w = windows_[event.xany.window];
                            protect_connection_.unlock();
                            w->handleEvent (event);
                            break;
                    case MotionNotify: 
                            protect_connection_.lock();
                            w = windows_[event.xany.window];
                            protect_connection_.unlock();
                            w->handleEvent (event);
                            break; 
                }
        }
}

inline
void swap (unsigned char &a, unsigned char &b)
{
        unsigned char tmp = a;
        a = b;
        b = tmp;
}
 
Window_X_Implementations::XCallback_i::XCallback_i (Component_i *c=0,
                               SysCaller *b=0)
                : Callback_i (c, b), ClassType (Latin1("XCAllback")) 
{
}

int
Window_X_Implementations::XCallback_i::eventType ()
{
        return event.type;        
}

int
Window_X_Implementations::XCallback_i::x ()
{
        return event.xbutton.x;
        
}
 
int
Window_X_Implementations::XCallback_i::y ()
{
        return event.xbutton.y;
}


int Window_X_Implementations::XAtom_i::id ()
{
        return atom_;
}
 

string Window_X_Implementations::XAtom_i::name ()
{
        if (name_!=Latin1("")) return name_;
        connection_->lock ();
        char *n = XGetAtomName (server_, atom_);
        connection_->unlock ();
        if (n==0) return Latin1("NO SUCH ATOM!!!!!");
        name_ = Latin1(n);
        XFree (n);
        return name_;
}

// Window_X_Implementations::Property //////////////////////////////////////////////////////////////

int 
Window_X_Implementations::XProperty_i::nofElements ()
{
        return nof_items_returned;
}

int
Window_X_Implementations::XProperty_i::getInteger (int i)
{
        if (return_type->id()!=XA_INTEGER)
        {
                property->name().print (stderr,"Trying to get integer from property ", "");
                return_type->name ().print (stderr, " which has type ", "\n");
                exit (-1);
        }
        switch (return_format)
        {
            case 8: return *(buf+i);
            case 16: return *((short*)buf+i);
            case 32: return *((int*)buf+i);
        }
        return 0;        
}

string
Window_X_Implementations::XProperty_i::getString ()
{
        if (return_type->id()!=XA_STRING)
        {
                property->name().print (stderr,"Trying to get string from property ",""); 
                return_type->name ().print (stderr, " which has type ", "\n");
                exit (-1);
        }
        string r;
        switch (return_format)
        {
            case 8: r.use8bitData ((char*)buf, nof_items_returned); return r;
                    
            case 16: r.useUniData ((unichar*)buf, nof_items_returned); return r;
                    
            case 32: return Latin1("Cannot handle 32 bit unicode characters!!!!"); 
        }        
        return Latin1("");        
}

Window_X_Interfaces::XAtom_ptr
Window_X_Implementations::XProperty_i::getAtom (int i)
{
        if (return_type->id()!=XA_ATOM)
        {
                property->name().print (stderr,"Trying to get atom from property ",""); 
                return_type->name ().print (stderr, " which has type ", "\n");                
                exit (-1);
        }
        string r;
        if (return_format==8 || return_format==16) 
        {
                fprintf (stderr, "ATOM must be stored in 32 bit format!\n");
                exit (-1);
        }
        return new XAtom_i (*((::Atom*)buf+i), property->server_, property->connection_);
}


int
Window_X_Implementations::XProperty_i::getIOR (int i)
{
        if (return_format==8 || return_format==16)
        {
                fprintf (stderr, "NOT IOR FORMAT\n" );
                exit (-1);
        }
    
        return *((int*)buf+i);
}


// Window_X_Implementations::Visual ////////////////////////////////////////////////////////////////

int Window_X_Implementations::XVisual_i::depth () 
{
        return depth_;
}

int Window_X_Implementations::XVisual_i::typeOfVisual ()
{
        return visual_->c_class;
}

unsigned long Window_X_Implementations::XVisual_i::redMask ()
{
        return visual_->red_mask;
}

unsigned long Window_X_Implementations::XVisual_i::greenMask ()
{
        return visual_->green_mask;
}

unsigned long Window_X_Implementations::XVisual_i::blueMask ()
{
        return visual_->blue_mask;
}

int Window_X_Implementations::XVisual_i::bitsPerRGB ()
{
        return visual_->bits_per_rgb;
}

int Window_X_Implementations::XVisual_i::nofMapEntries ()
{
        return visual_->map_entries;
}

// Window_X_Implementations::Screen ////////////////////////////////////////////////////////////////

Window_X_Implementations::XScreen_i::XScreen_i (Component_ptr c, 
                         string n, 
                         ::Display *server,
                         ::Screen *screen,
                         int nr,
                         X_DataReceiver *connection) 
                : Component_i (c,n), ClassType (Latin1("Screen")), server_ (server), screen_ (screen),
                  connection_ (connection),
                  screen_nr_ (nr),  
                  dga_mapped_ (false), dga_in_use_ (false) 
{ 
        connection_->lock();
        
        Bool b = XF86VidModeGetAllModeLines (server_,
                                             screen_nr_,
                                             &nof_video_modes_,
                                             &video_modes_); 

        connection_->unlock();
        nof_visuals_ = 0;
        for (int i=0; i<screen_->ndepths; ++i)
        {
                nof_visuals_ += screen_->depths[i].nvisuals;
        }
        visuals_ = new XVisual_i[nof_visuals_];
        int t=0;
        for (int i=0; i<screen_->ndepths; ++i)
        {
                for (int j=0; j<screen_->depths[i].nvisuals; ++j)
                {
                        visuals_[t].depth_ = screen_->depths[i].depth;
                        visuals_[t].visual_ = &(screen_->depths[i].visuals[j]);
                        t++;
                }	    
        }

        cmap_ = DefaultColormap (server_, nr);                                
}

int Window_X_Implementations::XScreen_i::width () { return WidthOfScreen (screen_); }
int Window_X_Implementations::XScreen_i::height () { return HeightOfScreen (screen_); }
int Window_X_Implementations::XScreen_i::widthmm () { return WidthMMOfScreen (screen_); }
int Window_X_Implementations::XScreen_i::heightmm () { return HeightMMOfScreen (screen_); }

Window_Interfaces::Color 
Window_X_Implementations::XScreen_i::getBlackPixel ()
{
        return Color (BlackPixelOfScreen(screen_),
                      0,0,0);
}

Window_Interfaces::Color
Window_X_Implementations::XScreen_i::getWhitePixel ()
{
        return Color (WhitePixelOfScreen(screen_),
                      255,255,255);
} 

Window_Interfaces::Color
Window_X_Implementations::XScreen_i::getColorByRGB (int red, int green, int blue) 
{
        ::XColor xc;
        xc.red = red;
        xc.green = green;
        xc.blue = blue;
        XAllocColor (server_, cmap_, &xc);
        
        return Color(xc.pixel,xc.red,xc.green,xc.blue);        
}

Window_Interfaces::Color
Window_X_Implementations::XScreen_i::getColorByName (string name) 
{
        char cn[256];
        UniToUtf (name, cn);
        
        XColor sd, ed;
        
        XAllocNamedColor (server_,
                          cmap_,
                          cn,
                          &sd, &ed);

        return Color (ed.pixel,ed.red,ed.green,ed.blue);        
}

Window_X_Interfaces::XWindow_ptr
Window_X_Implementations::XScreen_i::getRootWindow ()
{    
        XWindow_ptr w = new XWindow_i (composite(),Latin1("root"),

                                       0,
                                     0,0,
                                     0,0,
                                     RootWindowOfScreen (screen_),
                                     screen_,
                                     server_,
                                     connection_);
        return w;
}

int Window_X_Implementations::XScreen_i::nofVisuals ()
{
        return nof_visuals_;
}

Window_X_Interfaces::XVisual_ptr
Window_X_Implementations::XScreen_i::visual (int n)
{
        if (n<0 || n>=nof_visuals_) return 0;
        return &visuals_[n];
}


int Window_X_Implementations::XScreen_i::nofVideoModes ()
{
        return nof_video_modes_;
}

XF86VidModeModeInfo* Window_X_Implementations::XScreen_i::videoModeInfo (int n)
{
        if (n<0 || n>=nof_video_modes_) return 0;
        return video_modes_[n];
}

void Window_X_Implementations::XScreen_i::setVideoMode (int i)
{
        connection_->lock();
        Bool b = XF86VidModeSwitchToMode (server_,
                                          screen_nr_,
                                          video_modes_[i]);
        connection_->unlock();
        connection_->sync ();
}

int Window_X_Implementations::XScreen_i::getBestVideoMode (int x, int y)
{
        int best=-1, ydiff=999999999, xdiff=999999999;

        for (int i=0; i<nof_video_modes_; ++i)
        {
                int dx = video_modes_[i]->hdisplay-x;
                int dy = video_modes_[i]->vdisplay-y;
                if (dx>=0 && xdiff>dx &&
                    dy>=0 && ydiff>dy) 
                {
                        xdiff = dx;
                        ydiff = dy;
                        best = i;
                }
        }
        return best;
}

void Window_X_Implementations::XScreen_i::setViewPort (int x, int y)
{
        connection_->lock();
        Bool b = XF86VidModeSetViewPort (server_, 
                                         screen_nr_,
                                         x, y);
        connection_->unlock();
        connection_->sync ();
}

bool
Window_X_Implementations::XScreen_i::hasDGA ()
{
        int flags;
        connection_->lock ();
        XF86DGAQueryDirectVideo (server_, screen_nr_, &flags);
        connection_->unlock ();
        return (flags & XF86DGADirectPresent)!=0;
}

void Window_X_Implementations::XScreen_i::getDGA (char **addr, int *width, int *bankSize, int *memSize)
{
        if (!dga_mapped_)
        {
                connection_->lock ();
                XF86DGAGetVideo (server_, screen_nr_, &dga_addr_, &dga_width_, 
                                 &dga_bank_size_, &dga_mem_size_);
                connection_->unlock ();
                dga_mapped_ = true;
        }

        *addr = dga_addr_;
        *width = dga_width_;
        *bankSize = dga_bank_size_;
        *memSize = dga_mem_size_;
}

//  Method: useDGA
// Purpose: Enables or disables the use of DGA.
//          XF86DGADirectGraphics          enable Direct Video mode
//          XF86DGADirectMouse             enable reporting of pointer movement
//                                         as relative motion
//          XF86DGADirectKeyb              enable direct reporting of keyboard
//                                         events
void Window_X_Implementations::XScreen_i::useDGA (int flags)
{
        if (flags==0 && dga_in_use_==false) return;
        connection_->lock ();
        //    if (dga_in_use_==false) XF86DGAForkApp (screen_nr_);  
        XF86DGADirectVideo (server_, screen_nr_, flags);
        if (flags!=0) dga_in_use_ = true;
        else          dga_in_use_ = false;
        connection_->sync ();
        connection_->unlock ();
}

// Window_X_Implementations::Colormap ///////////////////////////////////////////////////////////////
/*
Window_X_Implementations::Colormap_i::Colormap_i (Component_i * c, 
                             string n,
                             ::Server *d,
                             ::Screen *s)
                : Component_i (c,n), ClassType (c, Latin1("Colormap")),
                  server_(d), screen_ (s)
{
        cmap = DefaultColormap (server_,
                                screen_);                        
}

Color
Window_X_Implementations::Colormap_i::getColor (int red, int green, int blue) 
{
        return Color(0,red,green,blue);        
}

Color Window_X_Implementations::Colormap_i::getColor (string name) 
{
        char cn[256];
        UniToUtf (name, cn);
        
        XColor sd, ed;
        
        XAllocNamedColor (server,
                          colormap,
                          cn,
                          &sd, &ed);
}
*/

Window_Interfaces::Orientation&
Window_Interfaces::Orientation::rotate (float rad)
{
        float x = cos (rad);
        float y = sin (rad);
        float z = -y;
        float w = x;

        float aa = a*x+b*z;
        float bb = a*y+b*w;
        float cc = c*x+d*z;
        float dd = c*y+d*w;

        a = aa;
        b = bb;
        c = cc;
        d = dd;

        return *this;
}

Window_Interfaces::Orientation&
Window_Interfaces::Orientation::rotate90 ()
{
        float x = 0;
        float y = 1;
        float z = -y;
        float w = x;

        float aa = a*x+b*z;
        float bb = a*y+b*w;
        float cc = c*x+d*z;
        float dd = c*y+d*w;

        a = aa;
        b = bb;
        c = cc;
        d = dd;

        return *this;
}

Window_Interfaces::Orientation&
Window_Interfaces::Orientation::rotate180 ()
{
        float x = -1;
        float y = 0;
        float z = -y;
        float w = x;

        float aa = a*x+b*z;
        float bb = a*y+b*w;
        float cc = c*x+d*z;
        float dd = c*y+d*w;

        a = aa;
        b = bb;
        c = cc;
        d = dd;

        return *this;
}

Window_Interfaces::Orientation&
Window_Interfaces::Orientation::rotate270 ()
{
        float x = 0;
        float y = -1;
        float z = -y;
        float w = x;

        float aa = a*x+b*z;
        float bb = a*y+b*w;
        float cc = c*x+d*z;
        float dd = c*y+d*w;

        a = aa;
        b = bb;
        c = cc;
        d = dd;

        return *this;
}

Window_Interfaces::Orientation&
Window_Interfaces::Orientation::scale (float xs, float ys)
{
        float x = xs;
        float y = 0;
        float z = 0;
        float w = ys;

        float aa = a*x+b*z;
        float bb = a*y+b*w;
        float cc = c*x+d*z;
        float dd = c*y+d*w;

        a = aa;
        b = bb;
        c = cc;
        d = dd;

        return *this;
}

Window_Interfaces::Orientation&
Window_Interfaces::Orientation::mul (Orientation o)
{
        float x = o.a;
        float y = o.b;
        float z = o.c;
        float w = o.d;

        float aa = a*x+b*z;
        float bb = a*y+b*w;
        float cc = c*x+d*z;
        float dd = c*y+d*w;

        a = aa;
        b = bb;
        c = cc;
        d = dd;

        return *this;
}

string
Window_Interfaces::Orientation::asString ()
{
        string aa,bb,cc,dd;
        aa.asString(a);
        bb.asString(b);
        cc.asString(c);
        dd.asString(d);

        string tmp = Latin1("[")+aa+Latin1(" ")+bb+Latin1(" ")+cc+Latin1(" ")+dd+Latin1("]");
        return tmp.replaceSubstrings (Latin1("-"),Latin1("~"));
}

// Window_X_Implementations::Font ////////////////////////////////////////////////////////////////////

Window_X_Implementations::XFont_i::XFont_i (Component_ptr c,
                                            string n)
                : Component_i (c,n), ClassType (Latin1("XFont_i"))
{
}

int Window_X_Implementations::XFont_i::ascent () { return ascent_; }
int Window_X_Implementations::XFont_i::descent () { return descent_; }
void Window_X_Implementations::XFont_i::addToText (string in, XText_ptr o)
{
        Window_X_Implementations::XText_i *out =
                  dynamic_cast<Window_X_Implementations::XText_i*>(o);
        
        string tmp;
        int l = in.size (), i=0, t=0;
        for (; i<l && t<nfonts_; ++t)
        {
                for (; in[i] && tables[t]->handle(in[i]); ++i) 
                {
                        tmp = tmp + in[i];
                }
                if (tmp.size())
                {
                        tmp = tables[t]->translate (tmp);
                        out->addInternalItem (tmp, fonts[t]);
                        tmp = string(Latin1(""));
                        t = -1;
                }
        }
}

void
Window_X_Implementations::XFont_i::load (::Display *d, 
                                         Orientation o)
{
        String fontname (this, Latin1("name"), Latin1(""));
        string fname;
        char cfontname[256];

        ascent_ = 0;
        descent_ = 0;        
        nfonts_ = 0;

        fname = fontname.value().trimends().replaceSubstring (Latin1("<SCALE>"), o.asString ());
        fname.storeAsLatin1 (cfontname);
        fonts [0] = XLoadQueryFont (d, cfontname);
        if (fonts[0]!=NULL)
        {
                tables [0] = new TranslateTable (Latin1("*"), 0, 255);
                XCharStruct xs = fonts[0]->max_bounds;
                if (xs.ascent>ascent_) ascent_= xs.ascent;
                if (xs.descent>descent_) descent_= xs.descent;
                fprintf (stderr, "Ascent = %d Descent = %d\n", xs.ascent, xs.descent);
                for (int f=43; f<=66; ++f)
                {
                        unsigned long fp;
                        XGetFontProperty(fonts[0], (::Atom)f, &fp);
                }
                nfonts_ = 1;                
        }
        else
                fprintf (stderr, "Could not load font >%s<", cfontname);
}

// Window_X_Implementations::Text ////////////////////////////////////////////////////////////////////

void Window_X_Implementations::XText_i::addItem (string s,
                                                 XFont_ptr font)
{
        font->addToText (s, this);
}

void Window_X_Implementations::XText_i::addInternalItem (string s, ::XFontStruct *font)
{
        if (font->ascent>ascent_) ascent_ = font->ascent;
        if (font->descent>descent_) descent_ = font->descent;
        height_ = ascent_+descent_;
        
        char buffer[2000];
        s.storeAsUtf (buffer);
        int w = XTextWidth (font, buffer, strlen (buffer));
        width_ += w;
        
        texts_ [nitems_] = s; 
        fonts_ [nitems_] = font;
        nitems_++;
}

int Window_X_Implementations::XText_i::width ()
{
        return width_;
}

int Window_X_Implementations::XText_i::height ()
{
        return height_;
}

int Window_X_Implementations::XText_i::ascent()
{
        return ascent_;
}

int Window_X_Implementations::XText_i::descent ()
{
        return descent_;
}

// Window_X_Implementations::GC /////////////////////////////////////////////////////////////////////

Window_X_Implementations::XGC_i::XGC_i (Component_ptr c,
                                        string n, 
                                        ::Display *server,
                                        ::Drawable drawable,
                                        WindowXFactory_i *factory)
                : ClassType (Latin1("GC")),
                  Component_i (c,n), server_ (server), factory_ (factory)
{
        gc_ = XCreateGC (server_,drawable,0,0);
}

void Window_X_Implementations::XGC_i::setForeground (int c)
{
        XSetForeground (server_, gc_, c);
}
void Window_X_Implementations::XGC_i::setBackground (int c)
{
        XSetBackground (server_, gc_, c);
}
void Window_X_Implementations::XGC_i::setLineAttributes (int line_width,
                                   int line_style,
                                   int end_style,
                                   int join_style)
{
        XSetLineAttributes (server_, gc_, 
                            line_width, line_style, 
                            end_style, join_style);
}
void Window_X_Implementations::XGC_i::setArcMode (int arc_mode)
{
        XSetArcMode (server_, gc_, arc_mode);
}

// Drawable //////////////////////////////////////////////////////////////////

void Window_X_Implementations::XDrawable_i::drawPoint (XGC_ptr ggc, int x, int y)
{
        XGC_i *gc = dynamic_cast<Window_X_Implementations::XGC_i*>(ggc);
        XDrawPoint (server_, drawable_, gc->gc_, x, y);
}
void Window_X_Implementations::XDrawable_i::drawLine (XGC_ptr ggc, int x1, int y1, int x2, int y2)
{
        XGC_i *gc = dynamic_cast<Window_X_Implementations::XGC_i*>(ggc);
        XDrawLine (server_, drawable_, gc->gc_, x1, y1, x2, y2);
}
void Window_X_Implementations::XDrawable_i::drawRectangle(XGC_ptr ggc, int x, int y, int w, int h)
{
        XGC_i *gc = dynamic_cast<Window_X_Implementations::XGC_i*>(ggc);        
        XDrawRectangle (server_, drawable_, gc->gc_, x, y, w, h);
}
void Window_X_Implementations::XDrawable_i::fillRectangle(XGC_ptr ggc, int x, int y, int w, int h)
{
        XGC_i *gc = dynamic_cast<Window_X_Implementations::XGC_i*>(ggc);
        XFillRectangle (server_, drawable_, gc->gc_, x, y, w, h);
}

#define IROUND(x) (int)((x)>0 ? (x)+0.5 : (x)-0.5)

void Window_X_Implementations::XDrawable_i::drawText (XGC_ptr ggc, XText_ptr ttext, int x, int y, Orientation o)
{
        double offset=0;
        int charno, char_metric_offset; 
        XChar2b c;
        char ch;
        char tmp;
        XFontStruct *fontinfo;
        string s;

        XGC_i *gc = dynamic_cast<Window_X_Implementations::XGC_i*>(ggc);
        XText_i *text = dynamic_cast<Window_X_Implementations::XText_i*>(ttext);

        for (int i=0; i<text->nitems_; ++i)
        {
                fontinfo = text->fonts_[i];
                XSetFont (server_, gc->gc_, fontinfo->fid);
                s = text->texts_[i];
                for (int j=0; j<s.size(); ++j)
                {	  
#ifdef DOSWAP
                        c.byte2 = s[j]&255;
                        c.byte1 = s[j]>>8;
#endif 
                        ch = s[j]&255;
                        charno = s[j]&255;
                        char_metric_offset = charno - fontinfo->min_char_or_byte2;

                        XDrawString (server_,
                                     drawable_,
                                     gc->gc_,
                                     IROUND((float)x+offset*o.a),
                                     IROUND((float)y-offset*o.b),
                                     &ch, 
                                     1);

                        int tmp; 
                        if (fontinfo->per_char != 0)
                        {
                                tmp = fontinfo->per_char[char_metric_offset].width;
                        }
                        else
                        {
                                tmp = fontinfo->min_bounds.width;
                        }
                        offset += tmp;
                }
        }
}

void Window_X_Implementations::XDrawable_i::drawDrawable(XGC_ptr ggc, 
                                                         XDrawable_ptr image, 
                                                         int x, int y, 
                                                         int w, int h, 
                                                         int sx, int sy)
{
}

void Window_X_Implementations::XDrawable_i::drawImage (XGC_ptr ggc, 
                                                       XImage_ptr iimage, 
                                                       int x, int y, 
                                                       int w, int h, 
                                                       int sx, int sy)
{
        XGC_i *gc = dynamic_cast<Window_X_Implementations::XGC_i*>(ggc);
        XImage_i *image = dynamic_cast<Window_X_Implementations::XImage_i*>(iimage);
        
        if (w<=0) w = image->width ();
        if (h<=0) h = image->height ();

        XPutImage (server_,
                   drawable_,
                   gc->gc_,
                   image->image_,
                   sx, sy,
                   x, y,
                   w, h);
}

// Window_X_Implementations::XPixmap_i /////////////////////////////////////////////////////////////////

Window_X_Implementations::XPixmap_i::XPixmap_i (Component_ptr c,
                         string n,
                         int width, int height,
                         ::Window window,
                         ::Screen *screen,
                         ::Display *server,
                         X_DataReceiver *connection)
                : XDrawable_i (c,n, window, screen, server, connection, 0),
                  ClassType (Latin1("Pixmap"))
{
}                

int Window_X_Implementations::XPixmap_i::width ()
{
        return 0;        
}

int Window_X_Implementations::XPixmap_i::height ()
{
        return 0;        
}

// Window_X_Implementations::XWindow_i //////////////////////////////////////////////////////////////

Window_X_Implementations::XWindow_i::XWindow_i (Component_ptr c,
                         string n,
                         XWindow_ptr pparent,
                         int x, int y,
                         int width, int height,
                         ::Window window,
                         ::Screen *screen,
                         ::Display *server,
                         X_DataReceiver *connection)
                : XDrawable_i (c,n,window,screen,server,connection,0),
                  ClassType (Latin1("Window")),
                  event_mask_ (0)
{
        Window_X_Implementations::XWindow_i *parent = dynamic_cast<Window_X_Implementations::XWindow_i*>(pparent);
        
        if (drawable_==0)
        {
                Integer rwidth (this, Latin1("width"), width);
                Integer rheight (this, Latin1("height"), height);
                
                screen_ = parent->screen_;
                cerr << "Window creating name=" << n << "" << endl;
                cerr << "Width=" << rwidth.value () << "," << width << endl;
                cerr << "Height=" << rheight.value () << "," << height << endl;
                
                drawable_ = XCreateSimpleWindow (server_,
                                                 parent->drawable_,
                                                 x, y, 
                                                 rwidth.value(),
                                                 rheight.value(),
                                                 0,
                                                 BlackPixelOfScreen (screen_),
                                                 WhitePixelOfScreen (screen_));
        }
        connection_->lock ();
        XGetWindowAttributes (server_, drawable_, &attributes_);
        connection_->unlock ();        
}

list<Window_Interfaces::Window_ptr> &
Window_X_Implementations::XWindow_i::children ()
{
        list<Window_ptr> *lw = new list<Window_ptr>;
        return *lw;
}

void
Window_X_Implementations::XWindow_i::addWindowChild (Window_Interfaces::Window_ptr w)
{
        // TODO
}

void
Window_X_Implementations::XWindow_i::dropWindowChild (Window_Interfaces::Window_ptr w)
{
        // TODO
}

void
Window_X_Implementations::XWindow_i::dropAllChildren ()
{
        // TODO
}

void
Window_X_Implementations::XWindow_i::moveToParent (Window_Interfaces::Window_ptr w)
{
        // TODO
}

Window_Interfaces::Window_ptr
Window_X_Implementations::XWindow_i::parent () 
{
        return NULL; // TODO
}

int Window_X_Implementations::XWindow_i::width () { return attributes_.width; }
int Window_X_Implementations::XWindow_i::height () { return attributes_.height; }
int Window_X_Implementations::XWindow_i::x () { return attributes_.x; }
int Window_X_Implementations::XWindow_i::y () { return attributes_.y; }
void Window_X_Implementations::XWindow_i::move (int x, int y) { XMoveWindow (server_,drawable_,x,y); }
void Window_X_Implementations::XWindow_i::resizepos (int w, int h, int x, int y) 
{ 
        if (x<100000 || y<100000)
                XResizeWindow (server_, drawable_, w, h);
        else
                XMoveResizeWindow (server_, drawable_, x, y, w, h);    
}
void Window_X_Implementations::XWindow_i::mapWindow ()
{
        XMapWindow (server_,drawable_);
}

void Window_X_Implementations::XWindow_i::mapAll ()
{
        XMapSubwindows (server_,drawable_);
        XMapWindow (server_,drawable_);
}

void Window_X_Implementations::XWindow_i::unmapWindow () { XUnmapWindow (server_,drawable_); }

void Window_X_Implementations::XWindow_i::focusWindow ()
{
        //        XUnmapWindow (server_,drawable_);
}

void Window_X_Implementations::XWindow_i::unfocusWindow ()
{
        //        XUnmapWindow (server_,drawable_);
}

void Window_X_Implementations::XWindow_i::clearArea (int x, int y, int w, int h, bool e)
{
        XClearArea (server_, drawable_, x, y, w, h, e?True:False);
}

void Window_X_Implementations::XWindow_i::clearWindow ()
{
        XClearWindow (server_, drawable_);
}

void Window_X_Implementations::XWindow_i::setWindowBackgroundPixmap (XPixmap_ptr pp)
{
        Window_X_Implementations::XPixmap_i *p = dynamic_cast<Window_X_Implementations::XPixmap_i*>(pp);
        
        if (p)
                XSetWindowBackgroundPixmap (server_,
                                            drawable_,
                                            p->drawable_);
        else
                XSetWindowBackgroundPixmap (server_,
                                            drawable_,
                                            None);      
}

void Window_X_Implementations::XWindow_i::shapeCombineMask (XPixmap_ptr mm)
{
        Window_X_Implementations::XPixmap_i *m = dynamic_cast<Window_X_Implementations::XPixmap_i*>(mm);

        XShapeCombineMask (server_,
                           drawable_,
                           ShapeBounding,
                           0,0,
                           m->drawable_,
                           ShapeSet);
}

void Window_X_Implementations::XWindow_i::onExpose (XCallback *ccb)
{
        Window_X_Implementations::XCallback_i *cb = dynamic_cast<Window_X_Implementations::XCallback_i*>(ccb);
        
        if (cb == 0)
        {
                expose_ = 0;
                connection_->removeCallback (this, Expose);
                event_mask_ &= ~ExposureMask; 
                XSelectInput (server_, drawable_, event_mask_);
                return;
        }	
        expose_ = cb;
        connection_->addCallback (this, cb, Expose);
        event_mask_ |= ExposureMask;
        XSelectInput (server_, drawable_, event_mask_);
}

void Window_X_Implementations::XWindow_i::onMotionNotify (XCallback *ccb)
{
        Window_X_Implementations::XCallback_i *cb = dynamic_cast<Window_X_Implementations::XCallback_i*>(ccb);
        
        if (cb == 0)
        {
                motion_notify_ = 0;
                connection_->removeCallback (this, MotionNotify);
                event_mask_ &= ~PointerMotionMask; 
                XSelectInput (server_, drawable_, event_mask_);
                return;
        }	
        motion_notify_ = cb;
        connection_->addCallback (this, cb, MotionNotify);
        event_mask_ |= PointerMotionMask;
        XSelectInput (server_, drawable_, event_mask_);
}

void Window_X_Implementations::XWindow_i::onButtonPress (XCallback *ccb)
{
        Window_X_Implementations::XCallback_i *cb = dynamic_cast<Window_X_Implementations::XCallback_i*>(ccb);
        
        if (cb == 0)
        {
                button_press_ = 0;
                connection_->removeCallback (this, ButtonPress);
                event_mask_ &= ~ButtonPressMask;	  
                XSelectInput (server_, drawable_, event_mask_);
                return;
        }	
        button_press_ = cb;
        connection_->addCallback (this,cb,ButtonPress);
        event_mask_ |= ButtonPressMask;
        XSelectInput (server_,drawable_, event_mask_);
}

void Window_X_Implementations::XWindow_i::onKeyPress (XCallback *ccb)
{
        Window_X_Implementations::XCallback_i *cb = dynamic_cast<Window_X_Implementations::XCallback_i*>(ccb);
        
        if (cb == 0)
        {
                button_press_ = 0;
                connection_->removeCallback (this, ButtonPress);
                event_mask_ &= ~KeyPressMask;
                XSelectInput (server_, drawable_, event_mask_);
                return;
        }	
        key_press_ = cb;
        connection_->addCallback (this,cb,KeyPress);
        event_mask_ |= KeyPressMask;
        XSelectInput (server_,drawable_, event_mask_);
}

void Window_X_Implementations::XWindow_i::onEnterNotify (XCallback *ccb)
{
        Window_X_Implementations::XCallback_i *cb = dynamic_cast<Window_X_Implementations::XCallback_i*>(ccb);
        
        if (cb == 0)
        {
                enter_notify_ = 0;
                connection_->removeCallback (this, EnterNotify);
                event_mask_ &= ~EnterWindowMask; 
                XSelectInput (server_, drawable_, event_mask_);
                return;
        }	
        enter_notify_ = cb;
        connection_->addCallback (this,cb,EnterNotify);
        event_mask_ |= EnterWindowMask;
        XSelectInput (server_,drawable_, event_mask_);
}

void Window_X_Implementations::XWindow_i::onLeaveNotify (XCallback *ccb)
{
        Window_X_Implementations::XCallback_i *cb = dynamic_cast<Window_X_Implementations::XCallback_i*>(ccb);
        
        leave_notify_ = cb;
        connection_->addCallback (this,cb,LeaveNotify);
        event_mask_ |= LeaveWindowMask;
        XSelectInput (server_,drawable_, event_mask_);
}

void Window_X_Implementations::XWindow_i::onCreateNotify (XCallback *ccb)
{
        Window_X_Implementations::XCallback_i *cb = dynamic_cast<Window_X_Implementations::XCallback_i*>(ccb);
        
        create_notify_ = cb;
        connection_->addCallback (this,cb,CreateNotify);
        event_mask_ |= SubstructureNotifyMask;
        XSelectInput (server_,drawable_, event_mask_);
}

list<Window_X_Interfaces::XAtom_ptr> Window_X_Implementations::XWindow_i::listOfProperties ()
{
        int n;
        list<XAtom_ptr > answer;
        connection_->lock ();
        ::Atom *atoms = XListProperties (server_, drawable_, &n);
        connection_->unlock ();
        if (atoms == NULL) return answer;

        for (int i=0; i<n; ++i)
        {
                answer.push_back (new XAtom_i (atoms[i], server_, connection_));
        }				     
        XFree (atoms);
        return answer;
}

Window_X_Interfaces::XProperty_ptr
Window_X_Implementations::XWindow_i::property (XAtom_ptr aatomic)
{
        int n;
        ::Atom actual_type_return;
        int actual_format_return;
        unsigned long nitems_return;
        unsigned long bytes_after_return;
        unsigned char *prop_return = 0;

        Window_X_Implementations::XAtom_i *atomic = dynamic_cast<Window_X_Implementations::XAtom_i*>(aatomic);
        
        connection_->lock ();
        int suc = XGetWindowProperty (server_, drawable_,
                                      atomic->id(),
                                      0, 65536,
                                      False,
                                      AnyPropertyType,
                                      &actual_type_return,
                                      &actual_format_return,
                                      &nitems_return,
                                      &bytes_after_return,
                                      &prop_return);
        connection_->unlock ();
        XAtom_i *rt = new XAtom_i (actual_type_return, server_, connection_);
        XProperty_i *p = new XProperty_i (atomic, rt, 
                                          actual_format_return, nitems_return,
                                          bytes_after_return, prop_return, 0);

        return p;
}

void Window_X_Implementations::XWindow_i::deleteProperty (XAtom_ptr atom)
{
        connection_->lock ();
        XDeleteProperty (server_, drawable_, atom->id());
        connection_->unlock ();
}

void Window_X_Implementations::XWindow_i::changeProperty (Atom property,
                                  Atom type,
                                  int format,
                                  int mode,
                                  char *data,
                                  int nelements)
{
        connection_->lock();
        XChangeProperty (server_, drawable_,
                         property,
                         type,
                         format,
                         mode,
                         (unsigned char*)data,
                         nelements);
        connection_->unlock();
}

void Window_X_Implementations::XWindow_i::grabPointer (int event_mask)
{
        connection_->lock ();
        int suc = XGrabPointer (server_, drawable_,
                                False, 
                                event_mask,
                                GrabModeAsync,
                                GrabModeAsync,
                                None,
                                None,
                                CurrentTime);
        XFlush (connection_->server_);
        connection_->unlock ();			    
        if (suc!=GrabSuccess) cout << "Grab not successfull! " << suc << endl;
}

void Window_X_Implementations::XWindow_i::unGrabPointer ()
{
        connection_->lock ();
        XUngrabPointer (server_, CurrentTime);
        XFlush (connection_->server_);
        connection_->unlock ();			    
}

void Window_X_Implementations::XWindow_i::setDropFlowback (Flowback *fb)
{
        XAtom_ptr drop_atom_, ior_atom_;
        WindowXFactory_i *xx = window_x_factory;
        
        drop_atom_ = xx->getAtom (Latin1("DROPFLOWBACKS"));
        ior_atom_ = xx->getAtom (Latin1("IOR"));

        drop_ = fb;
        int id = 42; // fb->id();        
        if (fb!=0)
        {   
                //               fb->setName ("AFlowback");
                changeProperty (drop_atom_->id(),
                                ior_atom_->id(),
                                32,
                                PropModeReplace,
                                (char*)&id,
                                1);    
        }
        else deleteProperty (drop_atom_);
}

// Window_X_Implementations::Image /////////////////////////////////////////////////////////////////

Window_X_Implementations::XImage_i::XImage_i (Component_ptr c,
                                            string n,
                                            XScreen_ptr sscreen,
                                            int width, int height)
                : Component_i (c,n), ClassType (Latin1(")Image"))
{
        XScreen_i *screen = dynamic_cast<XScreen_i*>(sscreen);
        screen_ = screen;
        server_ = screen->server_;
        
        size_ = width*height*(DefaultDepthOfScreen (screen_->screen_)/8);
        buffer_ = new char [size_];
        image_ = XCreateImage (server_,
                               DefaultVisualOfScreen (screen_->screen_),
                               DefaultDepthOfScreen (screen_->screen_),
                               ZPixmap,	// Format		  
                               0,		// offset
                               buffer_,
                               width,
                               height,
                               8,		// bitmap_pad
                               0);		// bytes_per_line 
}

char *Window_X_Implementations::XImage_i::buffer () { return buffer_; }
int Window_X_Implementations::XImage_i::size () { return size_; }
int Window_X_Implementations::XImage_i::width () { return image_->width; }
int Window_X_Implementations::XImage_i::height () { return image_->height; }

void Window_X_Implementations::XImage_i::drawImage (XGC_ptr gc, 
                                                    XDrawable_ptr drawable, 
                                                    int x, int y, 
                                                    int w=0, int h=0,
                                                    int sx=0, int sy=0)
{
}

// Factory ///////////////////////////////////////////////////////////////////

Window_X_Implementations::X_DataReceiver *Window_X_Implementations::WindowXFactory_i::connection_;

Window_X_Implementations::WindowXFactory_i::WindowXFactory_i ()
                : Factory_i (0,Latin1("factory")),
                  ClassType (Latin1("WindowXFactory_i")),
                  keysource_ (this, Latin1("keysource"))
{
        cerr << "Factory constructor!" << endl;
        
        if (!connection_)
        {
                connection_ = new X_DataReceiver (Latin1(""));
        }
        keymap_ = NULL;
}

void
Window_X_Implementations::WindowXFactory_i::useFactory (string url)
{
}

void
Window_X_Implementations::WindowXFactory_i::sync ()
{
        connection_->sync ();
}

int
Window_X_Implementations::WindowXFactory_i::nofScreens ()
{
        return connection_->nof_screens_;
}

Window_X_Interfaces::XScreen_ptr
Window_X_Implementations::WindowXFactory_i::getScreen (int n)
{ 
        if (n<0) n=connection_->default_screen_;
        return connection_->screens_[n];
}

Window_X_Interfaces::XWindow_ptr
Window_X_Implementations::WindowXFactory_i::getWindow (int wid)
{ 
        XWindow_i *w = new Window_X_Implementations::XWindow_i (composite(), Latin1(""),
                                                                0,
                                                                0,0,
                                                                0,0,
                                                                wid,
                                                                DefaultScreenOfDisplay (connection_->server_),
                                                                connection_->server_,
                                                                connection_);
        return w;
}

Window_X_Interfaces::XFont_ptr
Window_X_Implementations::WindowXFactory_i::newFont (Component_ptr c,
                                                     string n, 
                                                     Orientation o)
{
        Window_X_Implementations::XFont_i *f = new Window_X_Implementations::XFont_i (c,n);
        f->load (connection_->server_, o);

        return f;
}

Window_X_Interfaces::XAtom_ptr 
Window_X_Implementations::WindowXFactory_i::getAtom (string name)
{
        char buf[256];
        name.storeAsUtf (buf);

        ::Atom a = XInternAtom (connection_->server_, buf, False);
        return new XAtom_i (a, connection_->server_, connection_);
}

Window_X_Interfaces::XText_ptr
Window_X_Implementations::WindowXFactory_i::newText (Component_ptr composite)
{
        return new XText_i;        
}

Window_X_Interfaces::XGC_ptr 
Window_X_Implementations::WindowXFactory_i::newGC (Component_ptr composite,
                                            XScreen_ptr sscreen)
{
        Window_X_Implementations::XScreen_i *screen = dynamic_cast<Window_X_Implementations::XScreen_i*>(sscreen);        
        Window_X_Implementations::XGC_i *gc = new XGC_i (composite,Latin1("gc"),
                                  connection_->server_,
                                  RootWindowOfScreen(screen->screen_),
                                  this);

        return gc;
}

Window_Interfaces::Window_ptr
Window_X_Implementations::WindowXFactory_i::newWindow (Component_ptr composite,
                                                 string n,
                                                 Window_ptr p)
{
        return newXWindow (composite, n, dynamic_cast<XWindow_ptr>(p));
}

Window_X_Interfaces::XWindow_ptr
Window_X_Implementations::WindowXFactory_i::newXWindow (Component_ptr composite,
                                                 string n,
                                                 XWindow_ptr p,
                                                 int x, int y,
                                                 int width, int height)
{
        XWindow_i *w = new XWindow_i (composite, n,
                                    p,
                                    x,y,
                                    width,height,
                                    0,
                                    0,
                                    connection_->server_,
                                    connection_);
        
        return w;
}

Window_X_Interfaces::XImage_ptr
Window_X_Implementations::WindowXFactory_i::newImage (Component_ptr composite,
                          XScreen_ptr screen,
                          int width,
                          int height)
{
        XImage_ptr i = new XImage_i (composite,Latin1(""),
                                    screen,
                                    width,height);
    
        return i;
}

Window_X_Interfaces::XScreen_ptr
Window_X_Implementations::WindowXFactory_i::queryPointerSingle (::Window wid, 
                                    ::Window &r,
                                    ::Window &c,
                                    int &x, int &y,
                                    int &x_root, int &y_root, 
                                    unsigned int &mask)
{
        connection_->lock ();
        XQueryPointer (connection_->server_, 
                       wid,
                       &r,
                       &c,
                       &x_root, 
                       &y_root, 
                       &x,
                       &y, 
                       &mask);
        connection_->unlock ();
        return 0;
}

Window_X_Interfaces::XScreen_ptr
Window_X_Implementations::WindowXFactory_i::queryPointer (::Window w, 
                                                          ::Window &r,
                                                          ::Window &c,
                                                          int &x, int &y,
                                                          int &x_root, int &y_root, 
                                                          unsigned int &mask)
{    
        ::Window rr,cc, oldc;
        int xv, yv, xr, yr;
        unsigned int mm;
        // Fetch root window
        queryPointerSingle (w, rr, cc, xv, yv, xr, yr, mm);       
        w = rr;
        // Is inside child of root window?
        queryPointerSingle (w, rr, cc, xv, yv, xr, yr, mm);
        oldc = rr;
        // Recurse into sub windows if inside them.
        while (cc!=0)
        {
                w = cc;
                oldc = cc;
                // Yes, query where inside child.
                queryPointerSingle (w, rr, cc, xv, yv, xr, yr, mm);	  
        }
        c = oldc;
        x_root = xr; 
        y_root = yr;
        x = xv;
        y = yv;
        return 0;
}


unichar
Window_X_Implementations::WindowXFactory_i::keycodeToUnichar (int keycode)
{
        if (keymap_==NULL)
        {
                keymap_ = new map<int,int>;
                map<string,string> &foo = keysource_.value();
                map<string,string>::iterator i;
                for (i=foo.begin(); i!=foo.end(); ++i)
                {
                        int first = (*i).first.asInt();
                        int second = (*i).second.asInt ();
                        (*keymap_)[second] = first;
                        fprintf (stderr, "map keycode %d to unicode %d\n", second, first);
                }
        }

        return (*keymap_)[keycode];
}

















