// Copyright (C) 1999 Fredrik hrstrm
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _Window_X_Implementations_hh
#define _Window_X_Implementations_hh

#include <math.h>
#include <map>
#include "string.hh"
#include "buffer.hh"

#include"Component_Implementations.hh"
#include"Window_X_Interfaces.hh"

namespace Window_X_Implementations
{
        using namespace Window_X_Interfaces;
        using namespace Component_Implementations;

        struct XAtom_i;
        struct XProperty_i;
        struct XVisual_i;
        struct XText_i;
        
        struct XCallback_i;
        struct XScreen_i;
        struct XFont_i;
        struct XGC_i;
        struct XDrawable_i;
        struct XPixmap_i;
        struct XWindow_i;
        struct XImage_i;
        struct WindowXFactory_i;

        struct X_Window;
        struct X_DataReceiver;

        struct X_Window : public Component_i
        {
                typedef ::XEvent x_eventstruct;
        
                XWindow_i             *window_;
                Mutex                 protect_events_;
                Semaphore             semaphore_;	     
                Thread                thread_;
                map<int,XCallback_i*> callbacks_;
                list<x_eventstruct>   events_;

                X_Window (XWindow_i *w)
                                : Component_i (0,Latin1("")), ClassType (Latin1("X_Window")), window_ (w),
                                  protect_events_ (Latin1("protect_events")), semaphore_ (Latin1("XSEM")),
                                  thread_(0,string(Latin1("X_thread")))
                {
                        thread_.start (call (this, &X_Window::sendCallbacks));	    
                }    	     
                void handleEvent (x_eventstruct event);                
                void sendCallbacks (Callback_i *cb);                
        };
                
        class X_DataReceiver : public Component_i
        {
                public:
                
                RecursiveMutex protect_x_;
                Mutex protect_connection_; 
                ::Display *server_;
                map<int,X_Window*> windows_;
                int nof_screens_;
                int default_screen_;
                XScreen_i **screens_;
                Thread caller_;
                
                X_DataReceiver (string url);
                ~X_DataReceiver ();
                
                void connect (string u);
                void accept (string u);
                void close();
                void receiveLoop (Callback_i *cb);
                
                X_Window *findWindow (int wid);
                
                void addCallback (XWindow_i *w, 
                                  XCallback_i *c, 
                                  int type);
                
                void removeCallback (XWindow_i *w,
                                     int type);
                void lock ();
                
                void unlock ();
                
                void sync ();                
        };
        
        class XAtom_i : public virtual XAtom
        {
                public:
                
                ::Atom     atom_;
                ::Display *server_;
                  string   name_;
                  X_DataReceiver *connection_;

                XAtom_i (::Atom a,
                          ::Display *d,
                          X_DataReceiver *c)
                                : atom_ (a),
                                  server_ (d),
                                  name_ (Latin1("")),
                                  connection_ (c) {}
		
                int id ();
                string name ();                
        };

        class XProperty_i : public virtual XProperty
        {
                XAtom_i*  property;
                XAtom_i*  return_type;
                int      return_format;
                unsigned long int    nof_items_returned;
                unsigned long int    bytes_after_return;
                unsigned char*  buf;
                int      buf_size;

                public:
                
                XProperty_i (XAtom_i *p, XAtom_i *rt, int rf, 
                             unsigned long int nir,
                             unsigned long int bar,
                             unsigned char *b,int bs)
                                : property (p),
                                  return_type (rt),
                                  return_format (rf),
                                  nof_items_returned (nir),
                                  bytes_after_return (bar),
                                  buf (b), buf_size (bs) {}
		
                int       nofElements ();		 
                int       getInteger (int i);
		string    getString ();
		XAtom_ptr getAtom (int i);
		int       getIOR (int i);
        };
        
        class XVisual_i : public virtual XVisual
        {
                public:

                int depth_;
                ::Visual *visual_;

                int           depth ();
                int           typeOfVisual ();
                unsigned long redMask ();
                unsigned long greenMask ();
                unsigned long blueMask ();
                int           bitsPerRGB ();
                int           nofMapEntries ();

        };
        
        class XText_i : public virtual XText
        {
                public:
                
		string         texts_[256];
                ::XFontStruct *fonts_[256];
		int            nitems_;
                
                int            width_;
                int            height_;  
                short          ascent_;	
                short          descent_;
                
		XText_i () : nitems_ (0),width_(0),height_(0),ascent_(0),descent_(0){}

                void addItem (string s, XFont_ptr font);
                int  width ();
                int  height ();
                int  ascent ();
                int  descent ();
                void addInternalItem (string s, ::XFontStruct *font);                
        };

        class XCallback_i : public virtual XCallback,
                            public Callback_i
        {
                public:                

                ::XEvent event;                

		XCallback_i (Component_i *c=0, 
                             SysCaller *b=0);
                                
                int eventType ();
                int x ();
                int y (); 
        }; 
        /*
        struct Colormap_i : public virtual Colormap,
                            public Component_i
        {
                ::Colormap cmap;
		::Display *server_;
		::Screen *screen_;
                
                Colormap_i (Component_i * c, 
                            string n,
                            ::Display *d,
                            ::Screen *s);
                Color getColor (int red, int green, int blue);
                Color getColor (string name);
        };
        */
        struct XScreen_i : public XScreen,
                           public Component_i
        {		
		::Display *server_;
		::Screen *screen_;
		X_DataReceiver *connection_;
		
                XVisual_i *visuals_;
                int nof_visuals_;
		
                XF86VidModeModeInfo **video_modes_;
                int nof_video_modes_;
                int screen_nr_;      
		
                bool dga_mapped_;
                bool dga_in_use_;
                char *dga_addr_;
                int  dga_width_;
                int  dga_bank_size_;
                int  dga_mem_size_;      

                int  cmap_;

                public:
                
		XScreen_i (Component_ptr  c, 
                          string n, 
                          ::Display *d,
                          ::Screen *s,
                          int screen_number,
                          X_DataReceiver *connection);

                int width ();
		int height ();
                int widthmm ();
		int heightmm ();
		Color getBlackPixel ();
		Color getWhitePixel ();
                Color getColorByRGB (int red, int green, int blue);                
                Color getColorByName (string name);
                
		XWindow_ptr getRootWindow ();
		
                int screenNr () { return screen_nr_; }
		
                int nofVisuals ();
                XVisual_ptr visual (int i=-1);
		
                int nofVideoModes ();
		XF86VidModeModeInfo* videoModeInfo (int i);
                void setVideoMode (int i);
                int  getBestVideoMode (int x, int y);
                void setViewPort (int x, int y);
		
                bool hasDGA ();
                void getDGA (char **addr, int *width, 
                                     int *bankSize, int *memSize);
                void useDGA (int flags = 0);
		
        };
    
        struct XFont_i : public virtual XFont,
                         public Component_i
        {
		TranslateTable *tables [16];
		::XFontStruct *fonts [16];
		int nfonts_;
		
                short ascent_;	// Baseline to top edge of raster. 
                short descent_;	// Baseline to bottom edge of raster. 

                public:
                
		XFont_i (Component_ptr  c,
                         string n);
		
                int ascent ();
		int descent ();
		void addToText (string s, XText_ptr text);
		void load (::Display *d, Orientation o);		
        };
    
        struct XGC_i : public virtual XGC,
                       public Component_i
        {
		::Display *server_;
		::GC gc_;
		WindowXFactory_i *factory_;
                
		XGC_i (Component_ptr c, 
                       string n, 
                       ::Display *server,
                       ::Drawable drawable,
                       WindowXFactory_i *factory);                

                void setForeground (int c);
		void setBackground (int c);
		void setLineAttributes (int width,
                                        int line_style,
                                        int end_style,
                                        int join_style);
		void setArcMode (int arc_mode);
        };
    
        struct XDrawable_i : public virtual XDrawable,
                             public Component_i
        { 
		::Drawable drawable_;
		::Screen *screen_;
		::Display *server_;
		X_DataReceiver *connection_;
		Factory *factory_;
                int running_thread_;

		XDrawable_i (Component_ptr c,
                            string n, 
                            ::Drawable drawable,
                            ::Screen *screen,
                            ::Display *server,
                            X_DataReceiver *connection,
                            Factory *factory)
                                : Component_i (c,n), ClassType (Latin1("Drawable")),
                                  drawable_ (drawable),
                                  screen_ (screen),
                                  server_ (server),
                                  connection_ (connection),
                                  factory_ (factory) { }
                
                void drawPoint (XGC_ptr gc, int x, int y);
		void drawLine (XGC_ptr gc, int x1, int y1,
                               int x2, int y2);
                void drawRectangle(XGC_ptr gc,int x1,int y1,
                                   int x2, int y2);
		void fillRectangle(XGC_ptr gc,int x1,int y1,
                                   int x2, int y2);
		void drawText (XGC_ptr gc, 
                               XText_ptr text,
                               int x, int y,
                               Orientation o);
		void drawDrawable (XGC_ptr gc, 
                                   XDrawable_ptr image, 
                                   int x, int y, 
                                   int w=0, int h=0, 
                                   int sx=0, int sy=0);
		void drawImage (XGC_ptr gc, 
                                XImage_ptr image, 
                                int x, int y, 
                                int w=0, int h=0, 
                                int sx=0, int sy=0);
        };
    
        struct XPixmap_i : public virtual XPixmap,
                           public XDrawable_i
        {
		XPixmap_i (Component_ptr c,
                           string n,
                           int width, int height,
                          ::Window window,
                           ::Screen *screen,
                           ::Display *server,
                           X_DataReceiver *connection);
                
		int width ();
		int height ();		
        };
    
        struct XWindow_i : public virtual XWindow,
                           public XDrawable_i
        {
		::XWindowAttributes attributes_;
		int event_mask_;
		
		XCallback_i *expose_,
			*motion_notify_,
			*button_press_,
			*key_press_,                
			*enter_notify_, 
			*leave_notify_,
                        *create_notify_;
	
                Flowback_ptr drop_;

		XWindow_i (Component_ptr c,
                        string n,
                        XWindow_ptr parent,
                        int x, int y, 
                        int width, int height,
                        ::Window window,
                        ::Screen *screen,
                        ::Display *server,
                        X_DataReceiver *connection);

                list<Window_ptr> &children ();
                void addWindowChild (Window_ptr w);
                void dropWindowChild (Window_ptr w);
                void dropAllChildren ();
                void moveToParent (Window_ptr w);
                Window_ptr parent ();
                
		int width ();
		int height ();
		int x();
		int y();
                void move (int x, int y);
                void resizepos (int w, int h, int x=-900000, int y=-900000);
                void mapWindow ();
                void mapAll ();
                void unmapWindow ();
                void focusWindow ();
                void unfocusWindow ();
		void clearArea (int x, int y,
                                        int width, int height,
                                        bool exposures);
                void clearWindow ();
                void setWindowBackgroundPixmap (XPixmap_ptr p);
                void shapeCombineMask (XPixmap_ptr m);
		
		void onExpose      (XCallback *cb);
		void onButtonPress (XCallback *cb);
                void onKeyPress (XCallback *cb);
		void onEnterNotify (XCallback *cb);
		void onLeaveNotify (XCallback *cb);
		void onMotionNotify (XCallback *cb);
		void onCreateNotify (XCallback *cb);	
		
                list<XAtom_ptr> listOfProperties ();
                XProperty_ptr property (XAtom_ptr atom);
                void         deleteProperty (XAtom_ptr atom);
                void         changeProperty (::Atom property,
                                                     ::Atom type,
                                                     int format,
                                                     int mode,
                                                     char *data,
                                                     int nelements);

                void grabPointer (int event_mask);
                void unGrabPointer ();
		
                void setDropFlowback (Flowback *fb);	
        };
 
        struct XImage_i : public virtual Window_X_Interfaces::XImage,
                          public Component_i
        {
		::XImage      *image_;
		char          *buffer_;
		int            size_;
		XScreen_i     *screen_;
		::Display     *server_;
                
		XImage_i (Component_ptr c,
                          string n,
                          XScreen_ptr screen,
                          int width, int height);		

		int width ();
		int height ();
		void drawImage (XGC_ptr gc, 
                                XDrawable_ptr drawable, 
                                int x, int y, 
                                int w=0, int h=0, 
                                int sx=0, int sy=0);
		char *buffer ();
		int size ();		
        };
	
        struct WindowXFactory_i : public virtual WindowXFactory,
                                  public Factory_i
        {
                WindowXFactory_i ();                
                
		void useFactory (string url);
                void sync ();
                
                int       nofScreens ();
                XScreen_ptr getScreen (int n=-1);
		
                XWindow_ptr getWindow (int wid);
                
                XFont_ptr   newFont (Component_ptr composite,
                                     string n,
                                     Orientation o);
                
                XAtom_ptr    getAtom (string name);

                XText_ptr   newText (Component_ptr composite);
                
                XGC_ptr     newGC (Component_ptr composite,
                                    XScreen_ptr screen);

                Window_ptr newWindow (Component_ptr composite,
                                      string n,
                                      Window_ptr parent);                
                
                XWindow_ptr   newXWindow (Component_ptr composite,
                                         string n,
                                         XWindow_ptr parent,
                                         int x=0, int y=0,
                                         int width=0, int height=0);
                
                XImage_ptr    newImage (Component_ptr composite,
                                        XScreen_ptr screen,
                                        int width, int height);
		
                XScreen_ptr   queryPointerSingle (::Window wid,
                                                  ::Window &r,
                                                  ::Window &c,
                                                  int &x, int &y,
                                                  int &x_root, int &y_root,
                                                  unsigned int &mask);
		
                XScreen_ptr  queryPointer (::Window wid,
                                           ::Window &r,
                                           ::Window &c,
                                           int &x, int &y,
                                           int &x_root, int &y_root,
                                           unsigned int &mask);

                unichar keycodeToUnichar (int keycode);
                
		static X_DataReceiver *connection_;
                StringMap keysource_;
                map<int,int> *keymap_;
        };

        extern WindowXFactory_i *window_x_factory;        
};

#endif













