// Copyright (C) 1999 Fredrik hrstrm
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _Window_X_Interfaces_hh
#define _Window_X_Interfaces_hh

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/extensions/shape.h>
#include <X11/extensions/xf86vmode.h>
#include <X11/extensions/xf86dga.h>

#include "Window_Interfaces.hh"

namespace Window_X_Interfaces
{
        using namespace Window_Interfaces;
        
        struct XAtom;
        typedef XAtom *XAtom_ptr;        
        struct XProperty;
        typedef XProperty *XProperty_ptr;        
        struct XVisual;
        typedef XVisual *XVisual_ptr;        
        struct XText;
        typedef XText *XText_ptr;
        struct XCallback;
        typedef XCallback *XCallback_ptr;
        struct Colormap;
        typedef Colormap *Colormap_ptr;        
        struct XScreen;
        typedef XScreen *XScreen_ptr;        
        struct XFont;
        typedef XFont *XFont_ptr;                
        struct XGC;
        typedef XGC *XGC_ptr;                
        struct XDrawable;
        typedef XDrawable *XDrawable_ptr;                
        struct XPixmap;
        typedef XPixmap *XPixmap_ptr;                
        struct XWindow;
        typedef XWindow *XWindow_ptr;                
        struct XImage;
        typedef XImage *XImage_ptr;                
        struct WindowXFactory;
        typedef WindowXFactory *WindowXFactory_ptr;
        
        struct XAtom
        {
                virtual int id () = 0;
                virtual string name () = 0;
        };
        
        struct XProperty
        {
                virtual int      nofElements () = 0;
                virtual int      getInteger (int i) = 0;
		virtual string   getString () = 0;
		virtual XAtom_ptr getAtom (int i) = 0;
		virtual int      getIOR (int i) = 0;
        };
        
        struct XVisual 
        {
                virtual int           depth () = 0;
                virtual int           typeOfVisual () = 0;
                virtual unsigned long redMask () = 0;
                virtual unsigned long greenMask () = 0;
                virtual unsigned long blueMask () = 0;
                virtual int           bitsPerRGB () = 0;
                virtual int           nofMapEntries () = 0;
        };
        
        struct XText
        {	
                virtual void addItem (string s, XFont_ptr font) = 0;
                virtual int width () = 0;                
                virtual int height () = 0;                
                virtual int ascent() = 0;                
                virtual int descent () = 0;                
        };
        
        struct XCallback : public virtual Callback
        {                
                virtual int eventType () = 0;
                virtual int x () = 0;
                virtual int y () = 0;
        };

        /*
        struct Colormap : public virtual Component
        {
                virtual Color getColor (int red, int green, int blue) = 0;
                virtual Color getColor (string name) = 0;
        };
        */
        
        struct XScreen : public virtual Component
        {
                virtual int width () = 0;
                virtual int height () = 0;
                virtual int widthmm () = 0;
                virtual int heightmm () = 0;
                virtual Color getBlackPixel () = 0;
                virtual Color getWhitePixel () = 0;
                virtual Color getColorByRGB (int red, int green, int blue) = 0;
                virtual Color getColorByName (string name) = 0;                
                
                virtual XWindow_ptr getRootWindow () = 0;

                virtual int screenNr () = 0;
                
                virtual int nofVisuals () = 0;
                virtual XVisual_ptr visual (int i=-1) = 0;
                
                virtual int nofVideoModes () = 0;
                virtual XF86VidModeModeInfo* videoModeInfo (int i) = 0;
                virtual void setVideoMode (int i) = 0;
                virtual int  getBestVideoMode (int x, int y) = 0;
                virtual void setViewPort (int x, int y) = 0;
                
                virtual bool hasDGA () = 0;
                virtual void getDGA (char **addr, int *width, 
                                     int *bankSize, int *memSize) = 0;
                virtual void useDGA (int flags = 0) = 0;
        };
        
        struct XFont : public virtual Component
        {
                virtual int ascent () = 0;
                virtual int descent () = 0;
                virtual void addToText (string s, XText_ptr text) = 0;
                virtual void load (::Display *d, Orientation o) = 0;
        };
        
        struct XGC : public virtual Component
        {
                virtual void setForeground (int pixel) = 0;
                virtual void setBackground (int pixel) = 0;
                virtual void setLineAttributes (int width,
                                                int line_style,
                                                int end_style,
                                                int join_style) = 0;
                virtual void setArcMode (int arc_mode) = 0;        
        };
        
        struct XDrawable : public virtual Window_Interfaces::Drawable
        {
                virtual void drawPoint (XGC_ptr gc, int x, int y) = 0;
                virtual void drawLine (XGC_ptr gc, int x1, int y1,
                                       int x2, int y2) = 0;
                virtual void drawRectangle(XGC_ptr gc,int x1,int y1,
                                           int x2, int y2) = 0;
                virtual void fillRectangle(XGC_ptr gc,int x1,int y1,
                                           int x2, int y2) = 0;
                virtual void drawText (XGC_ptr gc, 
                                       XText_ptr text,
                                       int x, int y,
                                       Orientation o) = 0;
                virtual void drawDrawable (XGC_ptr gc, 
                                           XDrawable_ptr drawable, 
                                           int x, int y, 
                                           int w=0, int h=0, 
                                           int sx=0, int sy=0) = 0;
                virtual void drawImage (XGC_ptr gc, 
                                        XImage_ptr image, 
                                        int x, int y, 
                                        int w=0, int h=0, 
                                        int sx=0, int sy=0) = 0;
                virtual int width ()=0;
                virtual int height ()=0;
        };
        
        struct XPixmap : public virtual Window_Interfaces::Pixmap,
                         public virtual XDrawable
        {
                virtual int width () = 0;
                virtual int height () = 0;
        };
        
        struct XWindow : public virtual Window_Interfaces::Window,
                         public virtual XDrawable
        {
                virtual list<Window_ptr> &children () = 0;
                
                virtual int width () = 0;
                virtual int height () = 0;
                virtual int x() = 0;
                virtual int y() = 0;
                virtual void move (int x, int y) = 0;
                virtual void resizepos (int w, int h,
                                        int x=-900000, int y=-900000) = 0;
                virtual void mapWindow () = 0;
                virtual void mapAll () = 0;
                virtual void unmapWindow () = 0;
                virtual void clearArea (int x, int y,
                                        int width, int height,
                                        bool exposures) = 0;
                virtual void clearWindow () = 0;
                virtual void setWindowBackgroundPixmap (XPixmap_ptr p) = 0;
                virtual void shapeCombineMask (XPixmap_ptr m) = 0;
                
                virtual void onExpose      (XCallback_ptr cb) = 0;
                virtual void onButtonPress (XCallback_ptr cb) = 0;
                virtual void onKeyPress (XCallback_ptr cb) = 0;                
                virtual void onEnterNotify (XCallback_ptr cb) = 0;
                virtual void onLeaveNotify (XCallback_ptr cb) = 0;
                virtual void onMotionNotify (XCallback_ptr cb) = 0;
                virtual void onCreateNotify (XCallback_ptr cb) = 0;    
                
                virtual list<XAtom_ptr> listOfProperties () = 0;
                virtual XProperty_ptr  property (XAtom_ptr atom) = 0;
                virtual void         deleteProperty (XAtom_ptr atom) = 0;
                virtual void         changeProperty (::Atom property,
                                                     ::Atom type,
                                                     int format,
                                                     int mode,
                                                     char *data,
                                                     int nelements) = 0;
                
                virtual void grabPointer (int event_mask) = 0;
                virtual void unGrabPointer () = 0;
                
                virtual void setDropFlowback (Flowback *fb) = 0;
        };
        
        struct XImage : public virtual Component
        {
                virtual int width () = 0;
                virtual int height () = 0;
                virtual void drawImage (XGC_ptr gc, 
                                        XDrawable_ptr drawable, 
                                        int x, int y, 
                                        int w=0, int h=0, 
                                        int sx=0, int sy=0) = 0;
                virtual char *buffer () = 0;
                virtual int size () = 0;
        };
        
        struct WindowXFactory : public virtual Factory
        {
                virtual void sync () = 0;
                
                virtual int         nofScreens () = 0;
                virtual XScreen_ptr getScreen (int n=-1) = 0;
                
                virtual XWindow_ptr getWindow (int wid) = 0;
                virtual XFont_ptr   newFont (Component_ptr composite,
                                             string n, 
                                             Orientation o) = 0;
                
                virtual XAtom_ptr  getAtom (string name) = 0;

                virtual XText_ptr  newText (Component_ptr composite) = 0;
                
                virtual XGC_ptr    newGC (Component_ptr composite,
                                          XScreen_ptr screen) = 0;

                virtual XWindow_ptr newXWindow (Component_ptr composite,
                                                string n,
                                                XWindow_ptr parent,
                                                int x, int y,
                                                int width, int height) = 0;
                
                virtual XImage_ptr  newImage (Component_ptr composite,
                                            XScreen_ptr screen,
                                            int width,
                                            int height) = 0;
                
                virtual XScreen_ptr queryPointerSingle (::Window wid,
                                                        ::Window &r,
                                                        ::Window &c,
                                                        int &x, int &y,
                                                        int &x_root, int &y_root,
                                                        unsigned int &mask) = 0;
                
                virtual XScreen_ptr queryPointer (::Window wid,
                                                  ::Window &r,
                                                  ::Window &c,
                                                  int &x, int &y,
                                                  int &x_root, int &y_root,
                                                  unsigned int &mask) = 0;

                virtual unichar keycodeToUnichar (int keycode) = 0;
        };

        typedef WindowXFactory *WindowXFactory_ptr;
};

#endif









