// Copyright (C) 1999 Fredrik hrstrm
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#include"buffer.hh"

buffer::buffer () : data_ (new BufferData)
{
}

buffer::buffer (int i) : data_ (new BufferData (i))
{
}

buffer::buffer (long i) : data_ (new BufferData (sizeof(long)))
{
    memcpy (data_->text_, &i, sizeof(long));
}

buffer::buffer (float f) : data_ (new BufferData (sizeof(float)))
{
    memcpy (data_->text_, &f, sizeof(float));
}

buffer::buffer (double d) : data_ (new BufferData (sizeof(double)))
{
    memcpy (data_->text_, &d, sizeof(double));
}

buffer::buffer (char *t) 
    : data_ (new BufferData (strlen (t)))
{
    memcpy (data_->text_,t,strlen(t));
}

buffer::buffer (string &s) 
    : data_ (new BufferData (s.size()))
{
    memcpy (data_->text_,s,sizeof(unichar)*s.size());
}

buffer::buffer (char *t, int length) 
    : data_ (new BufferData (length))
{
  memcpy (data_->text_,t,length);
}

int buffer::size ()
{ 
  return data_->size_;
}

char *buffer::getChar () { return data_->text_; }

buffer::operator string () 
{ 
    char t[size()+1];
    memcpy (t,data_->text_,size());
    t[size()] = 0;
    return Latin1(t);
}


char &buffer::operator [] (int i) { return data_->text_[i]; }


void buffer::store (char *buffer, int length) 
{
    if (length<size ()) memcpy (buffer,data_->text_,length);
    else
    {
	memcpy (buffer, data_->text_, size());
	for (int i = size (); i<length; i++) buffer [i] = 0;
    }
}


buffer buffer::extract (int start, int stop)
{
    char b [size()];

    if (stop<start || stop>size()) stop = size ();
    if (start<0) start = 0;
    int n = stop - start;
    memcpy (b, data_->text_+start, n);

    return buffer (b,n);
}

buffer buffer::scan (string &s)
{
    int a, o;

    for (a=0; data_->text_[a] == ' ' && a<size(); ++a);
    for (o=a; data_->text_[o] != ' ' && o<size(); ++o);
    s = extract (a,o);
    return extract (o);
}

buffer buffer::scan (int &i)
{
    int a, o;

    for (a=0; data_->text_[a] == ' ' && a<size(); ++a);
    for (o=a+1; data_->text_[o] != ' ' && o<size(); ++o);
    buffer b = extract (a,o);
    i = atoi (b.getChar());
    return extract (o);
}

/*
void buffer::show (ostream &s)
{
    for (int i=0; i<data_->size_; ++i)
    {
	if (data_->text_[i]<32 ||
	    data_->text_[i]>126)
	{
	    s << "@";
	}
	else
	{
	    s << data_->text_[i];
	}
    }
}
*/
buffer &buffer::operator += (buffer s)
{
    int si = size()+s.size();
    char *b = new char [si];
    memcpy (b,data_->text_,size());
    memcpy (b+size(),s.data_->text_,s.size());
    delete [] data_->text_;
    data_->text_ = b;    
    data_->size_ = si;
    return *this;
}

void buffer::setSize (int i)
{
    delete [] data_->text_;
    data_->text_ = new char [i];
    data_->size_ = i;
}

buffer operator + (buffer s, buffer t)
{
  char b [s.size()+t.size()];
  memcpy (b,s.getChar(),s.size());
  memcpy (b+s.size(),t.getChar(),t.size());
  return buffer (b, s.size()+t.size());
}

bool operator == (buffer s, buffer t)
{
  return !memcmp (s.getChar(),t.getChar(),s.size());
}

bool operator != (buffer s, buffer t)
{
  return memcmp (s.getChar(),t.getChar(),s.size());
}

string buffer::getWord (int n)
{
    int i,j;
    char *c = data_->text_;

    for (i=0; c[i]==' ' && i<data_->size_; ++i);
    
    for (; n>0 && i<data_->size_; --n)
    {
	for (; c[i]!=' ' && i<data_->size_; ++i);
	for (; c[i]==' ' && i<data_->size_; ++i);
    }
    for (j=i; c[j]!=' ' && j<data_->size_; ++j);

    if (i==j) return Latin1("");
    return string (extract (i,j));
}

int buffer::nofWords ()
{
    char *c = data_->text_;
    int n = c[0]==' '?0:1;

    for (int i=1; i<data_->size_; ++i)
    {
	if (c[i-1]==' ' && c[i]!=' ') ++n;
    }
    return n;    
}


