// Copyright (C) 1999 Fredrik hrstrm
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _buffer_hh
#define _buffer_hh

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "ref.hh"
#include "string.hh"

class buffer
{
        public:

        buffer ();
        buffer (int i);
        buffer (long l);
        buffer (float f);
        buffer (double d);
        buffer (char *t);
        buffer (string &s);
        buffer (char *t, int length);
        
        int size ();
        operator string ();
        char &operator [] (int i);
        void   store (char *t, int length); 
        buffer extract (int start, int stop=0);
        buffer &operator += (buffer s);
        void setSize (int i);
        
        char *getChar ();
        string getWord (int n);
        int    nofWords ();
        
        buffer scan (string &s);
        buffer scan (int &i);
        //	     void show (ostream &s);
        
        friend buffer   operator + (buffer s, buffer t);
        friend bool     operator == (buffer s, buffer t);
        friend bool     operator != (buffer s, buffer t);
        /*          
                    friend ostream &operator << (ostream &os, buffer t);
                    friend istream &operator >> (istream &is, buffer &t); */
        
        struct BufferData;
        
        class iter
        {
                public:  iter () : start_ (0), pos_ (0), end_ (0) { }
                void useBuffer (buffer &b) 
                { data_ = b.data_; pos_ = start_ = data_->text_; 
                end_ = data_->text_+data_->size_; }
                void go (int i) { pos_+=i; }
                void store (char *s)
                {
                        for (int i=0; s[i]; ++i)
			        *(pos_++) = s[i]; 
                }
                void store (char *s, int n)
                {
                        for (int i=0; i<n; ++i)
			        *(pos_++) = s[i]; 
                }
                void store (string &s)
                {		
                        for (int i=0; i<s.size(); ++i)
			        *(pos_++) = s[i]; 
                }
                void read (string &s)
                {
                        s = Latin1("");
                        while (*pos_==' ') pos_++;
                        while (*pos_!=' ') s = s+*(pos_++);
                }
                
                private: lptr<BufferData> data_;
                char *start_, *pos_, *end_;
        };
        
        private: struct BufferData : public ref
        {
                BufferData () : text_(0), size_(0) { }
                BufferData (int s):text_ (new char[s]), size_(s) { }
                ~BufferData () { delete text_; }
                char *text_;
                int   size_;
        };
        
        buffer (BufferData *data) : data_ (data) { }
        
        lptr<BufferData> data_;
        friend iter;
};

#endif











