// Copyright (C) 1997 Fredrik Ohrstrom /////////////////////////////////////
// Copyright (C) 1999 Fredrik Ohrstrom /////////////////////////////////////
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA
//
// File     : ref.hh
// Contents : ref is parent to all objects who wants pointer counting.
//            lptr is a smart template pointer which points to local ref 
//            objects.
// Classes  : ref lptr<>
//

#ifndef _ref_hh
#define _ref_hh

//   Class: ref
// Purpose: Base of all objects who wants reference counting and hashing.

class ref
{
   public: ref () : ref_ (0) { }
           virtual ~ref () { }

           void reference ()   { ref_++; }
           void deReference () { if (--ref_ == 0) delete this; }

           int ref_;
};

//   Class: lptr<>
// Purpose: Smart pointer to local ref objects.

template<class T>
class lptr
{
  public:  lptr () : ref_ (0) { }
           lptr (T *ref) : ref_ (ref) { if (ref_) ref_->reference (); }
           lptr (const lptr &p):ref_ (p.ref_) { if (ref_) ref_->reference (); }
           ~lptr () { if (ref_) ref_->deReference (); }

           lptr &operator = (const lptr &p) {
                                           if (ref_) ref_->deReference ();
					   ref_ = p.ref_;
					   if (ref_) ref_->reference ();
					   return *this;
	                                 }

           lptr &operator = (T *r) {
                                    if (ref_) ref_->deReference (); 
				    ref_ = r;
				    if (ref_) ref_->reference ();
				    return *this;
	                          } 

           T *operator -> () const { return ref_; }
           operator T* () const { return ref_; }
           operator bool () const { return ref_!=0; }

  private: T *ref_;
};

#endif




