// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#include<list>
#include"string.hh"
#include<stdio.h>
#include<stdlib.h>

void printString (string &s)
{
        printf ("s.data_=%p\n",(string::StringData*)s.data_);
        
        if ((string::StringData*)s.data_==0)
        {
                printf ("ARGGGGG! String does not point to data object!!!!!!\n");
                return;
        }
        if (s.data_->text_==0)
        {
                printf ("Stringdata is NULL\n");
                return;
        }

        //        printf ("%s", *s << endl;
}

void
string::print (FILE *f, char *pre, char *post) const
{
        char *t = new char[unistrlen(data_->text_)+1];
        storeAsLatin1 (t);
        if (pre!=0)
                fprintf (f, "%s%s%s", pre, t, post);
        else
                fprintf (f, "%s", t);
        delete t;
}

void
string::printv (FILE *f) 
{
        int l = unistrlen(data_->text_);
        
        for (int i=0; i<l; ++i)
        {
                fprintf (f, "%d (%c)", data_->text_[i], (char)(data_->text_[i]));
        }
}

bool operator < (string s, string t)
{
        unichar *a=s.data_->text_, *b=t.data_->text_;
        int i=0;
        // Scan past all matching characters.
        for (; a[i] && b[i] && a[i]==b[i]; ++i);

        // The first diffing character was less then return true.
        if (a[i] && b[i])
        {
                if (a[i]<b[i]) return true;
                else return false;
        }

        // Else one of the strings was shorter. If it is the
        // first one return true.
        if (a[i]==0 && b[i]!=0) return true;
        return false;
}
        
TranslateTable::TranslateTable (string filename, int ul, int uh)
    : table (new unichar [uh-ul+1]), unilow (ul), unihigh (uh), no_mapping(false)
{
        FILE *f;
        int n;
        unsigned int jis,uni,foo=0;
        char file [256];
	if (filename==Latin1("*")) 
	{
		no_mapping = true;
		return;
	}
        UniToUtf (filename, file);
        f = fopen (file,"r");
        if (!f) { printf ("FELLLLL!!!\n"); }
        while (!feof(f))
        {
                n = fscanf (f,"%x %x", &jis, &uni);
                if (n==2 && uni>=ul && uni<=uh)	
                {
                        table [uni-ul] = jis;
                        foo ++;
                }
        }
        fclose (f);
}
 
TranslateTable::~TranslateTable ()
{
    delete table;
}

string 
TranslateTable::translate (string s)
{
    string t = s.copy ();
	if (!no_mapping) t.translate (unilow, unihigh, table);
    return t;
}

int UtfToUni (const char *uin, unichar *out)
{
    const unsigned char *in = (const unsigned char*)uin;
    int i=0,o=0;
    for (; in[i];) 
    {
	if (in[i]<=127) { out[o++] = in[i++]; continue; }
	if ((in[i]&0xc0)==0xc0) { i++; continue; }
	if ((in[i]&0x20)==0x20) 
	{
	    out[o++] = ((in[i]&0x0f)<<12)|((in[i+1]&0x3f)<<6)|(in[i+2]&0x3f);
	    i+=3;
	    continue;
	}
	out[o++] = ((in[i]&0x1f)<<6)|(in[i+1]&0x3f);
	i+=2;
    }
    out[o]=0;
    return o;
}

int UtfToUnin (const char *uin, unichar *out, int length)
{
    const unsigned char *in = (const unsigned char*)uin;
    int i=0,o=0;
    for (; i<length;)
    {
	if (in[i]<=127) { out[o++] = in[i++]; continue; }
	if ((in[i]&0xc0) == 0xc0) { i++; continue; }
	if ((in[i]&0x20) == 0x20) 
	{
	    out[o++] = ((in[i]&0x0f)<<12)|((in[i+1]&0x3f)<<6)|(in[i+2]&0x3f);
	    i+=3;
	    continue;
	}
	out[o++] = ((in[i]&0x1f)<<6)|(in[i+1]&0x3f);
	i+=2;
    }
    out[o]=0;
    return o;
}

int utfstrlen (const char *s)
{
    int l=0,i;
    for (i=0; s[i]; ++i) if ((((unsigned char*)s)[i]&0xc0)!=0xc0) ++l;
    return l;
}

int unistrlen (unichar *s)
{
    int l=0;
    for (; s[l]; ++l);
    return l;
}
 
int unistrlenasutf (unichar *s)
{
    int l=0;
    for (int i=0; s[i]; ++i) 
    {
	if (s[i]<=0x7f) ++l;
	else
	if (s[i]<=0x7ff) l+=2;
	else
	l+=3;
    }    
    return l;
}

int UniToUtf (unichar *in, char *uout) 
{
    unsigned char *out = (unsigned char*)uout;
    int i=0,o=0;
    for (; in[i]; ++i)
    {
	if (in[i]<=0x7f) { out[o++]=(unsigned char)in[i]; continue; }
	if (in[i]<=0x7ff) 
	{ 
	    out[o]   = 0x80 | ((in[i]>>6)&0x1f);
	    out[o+1] = 0xc0 | (in[i]&0x3f);
	    o+=2;
	    continue;
	}
	out [o]   = 0xa0 | (in[i]>>12);
	out [o+1] = 0xc0 | ((in[i]>>6)&0x3f);
	out [o+2] = 0xc0 | (in[i]&0x3f);
	o+=3;
    }
    out [o] = 0;
    return o;
}

int UniToUtfn (unichar *in, char *uout, int length) 
{
    unsigned char *out = (unsigned char*)uout;
    int i=0,o=0;
    for (; i<length; ++i)
    {
	if (in[i]<=0x7f) { out[o++]=(unsigned char)in[i]; continue; }
	if (in[i]<=0x7ff) 
	{ 
	    out[o]   = 0x80 | ((in[i]>>6)&0x1f);
	    out[o+1] = 0xc0 | (in[i]&0x3f);
	    o+=2;
	    continue;
	}
	out [o]   = 0xa0 | (in[i]>>12);
	out [o+1] = 0xc0 | ((in[i]>>6)&0x3f);
	out [o+2] = 0xc0 | (in[i]&0x3f);
	o+=3;
    }
    out [o] = 0;
    return o;
}

int UniToLatin1n (unichar *in, char *uout, int length) 
{
    unsigned char *out = (unsigned char*)uout;
    int i=0,o=0;
    for (; i<length; ++i)
    {
	if (in[i]<=0xff) { out[o++]=(unsigned char)in[i]; continue; }
        out[o++] = 42;
    }
    out [o] = 0;
    return o;
}

string
substitute (string in,
            string pre,
            string post,
            map<string,string> &keys)
{
        int prepos = 0, postpos = 0, blockstart = 0;
        int length = in.size (), prelen = pre.size(), postlen = post.size();
        string result = Latin1("");
        
        while (blockstart<length)
        {
                prepos = in.findSubstring (blockstart, pre);
                // Did we find a presymbol...
                if (prepos<0) break;
                // Is it possible to extract key...
                if (prepos+prelen+postlen+1>length) break;

                postpos = in.findSubstring (prepos+prelen, post);
                // Did we find a postsymbol...
                if (postpos<0) break;
                string key = in.substring (prepos+prelen,postpos);
                map<string,string>::iterator i = keys.find (key);
                string v;
                if (i==keys.end()) v = pre+key+post;
                else v = (*i).second;
                result = result + in.substring (blockstart, prepos) + v;

                blockstart = postpos+postlen;
        }
        if (blockstart<length)
                result = result + in.substring (blockstart,in.size());
                
        return result;        
}


string
substituteChildren (string in,
                    map<string,string> &values,
                    map<string,int> &ordering,
                    list<string> &keylist)
{
        string pre = Latin1("%{");        
        string post = Latin1("}%");
        string childrentag = Latin1("CHILDREN");        
        map<string,bool> usedkeys;
        int prepos = 0, postpos = 0, blockstart = 0;
        int length = in.size (), prelen = pre.size(), postlen = post.size();
        string result = Latin1("");
        
        while (blockstart<length)
        {
                prepos = in.findSubstring (blockstart, pre);
                // Did we find a presymbol...
                if (prepos<0) break;
                // Is it possible to extract key...
                if (prepos+prelen+postlen+1>length) break;

                postpos = in.findSubstring (prepos+prelen, post);
                // Did we find a postsymbol...
                if (postpos<0) break;
                string key = in.substring (prepos+prelen,postpos);
                int oldcomma = 0, comma = key.findSubstring (0, Latin1(",")), bestorder=9999999;
                string bestkey = key, tmpkey;
                
                while (comma>0)
                {
                        tmpkey = key.substring (oldcomma, comma);
                        usedkeys[tmpkey] = true;                        
                        int t = ordering[tmpkey];
                        if (t<bestorder)
                        {
                                //                                tmpkey.print (stderr, "Found bestkey ", "");
                                //fprintf (stderr, " order=%d\n",t);
                                bestorder=t;
                                bestkey = tmpkey;
                        }
                        oldcomma=comma+1;
                        comma = key.findSubstring (oldcomma, Latin1(","));
                        if (comma<0 && oldcomma<key.size())
                                comma = key.size();
                }
                key = bestkey;
                string &v = values[key];

                if (v.size()==0) v = pre+key+post;
                
                result = result + in.substring (blockstart, prepos) + v;

                blockstart = postpos+postlen;
        }
        if (blockstart<length)
                result = result + in.substring (blockstart,in.size());

        
        string childs, topchild;
        int besttop=999999;
        list<string>::iterator i;
        for (i=keylist.begin(); i!=keylist.end(); ++i)
        {
                if (usedkeys[*i]==false)
                {
                        childs=childs+values[*i];
                        int t = ordering[*i];
                        if (t<besttop)
                        {
                                besttop = t;
                                topchild = values[*i];
                        }
                }
        }
        map<string,string> childkey;
        childkey[childrentag] = childs;
        result = substitute (result, Latin1("%["), Latin1("]%"), childkey);
        childkey[childrentag] = topchild;
        result = substitute (result, Latin1("%{"), Latin1("}%"), childkey);
        
        return result;        
}












