// Copyright (C) 1997 Fredrik Ohrstrom /////////////////////////////////////
// Copyright (C) 1999 Fredrik Ohrstrom /////////////////////////////////////
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA
//
// File:     string.H
// Contents: A non distributed unicode string class.
// Classes:  string

#ifndef _string_hh
#define _string_hh

#include <map>
#include <list>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
//#include <iostream.h>

#include "ref.hh"

typedef unsigned short unichar;

#define U_HORSE unichar(0x99ac)
#define U_ALEF  unichar(0x05D0)
#define U_BET   unichar(0x05D1)
#define U_TAV   unichar(0x05EA)
#define U_CB    unichar(0x0411)
#define U_CZJ   unichar(0x0416)
#define U_CJA   unichar(0x042f)

int UtfToUni (const char *uin, unichar *out);
int UtfToUnin (const char *uin, unichar *out, int length);
int utfstrlen (const char *s);
int unistrlen (unichar *s);
int unistrlenasutf (unichar *s);
int UniToUtf (unichar *in, char *uout);
int UniToUtfn (unichar *in, char *uout, int length);
int UniToLatin1n (unichar *in, char *uout, int length);

class string
{
        public:  

        string ();
        // string (const char *t);
        // string (char *t, int n);        
        // string (char *t);        
        string copy ();

        int     size () const; 
        int     storeAsUtf (char *t, int length=-1) const;
        int     storeAsLatin1 (char *t, int length=-1) const;
        int     asInt () const;
        void    asString (int i);
        void    asString (float f);
        void    asString (double d);
        void    translate (int unilow,
                           int unihigh, 
                           unichar *table);
        string    copyData (unichar *t, int l);
        void      useUniData (const unichar *t, long unsigned int l);
        void      use8bitData (const char *t, long unsigned int l);

        string  substring (int from, int to=0) const;
        int     findSubstring (int from, string pattern) const;
        string  replaceSubstring (string pattern, string data, int from=0);
        string  replaceSubstrings (string pattern, string data);
        
        string  trim ();
        string  trimends ();        
        
        long hashValue () const;

        operator unichar * () const;
        unichar &operator [] (int n);
        friend string   operator + (string s, string t);
        friend string   operator + (string s, char *t);
        friend string   operator + (char *s, string t);
        friend string   operator + (string s, unichar t);
        friend string   operator + (unichar s, string t);
        friend bool     operator == (string s, string t);
        friend bool     operator != (string s, string t);
        friend bool     operator < (string s, string t);        
        /*        friend ostream &operator << (ostream &os, string t);
                  friend istream &operator >> (istream &is, string &t);            */

        void print (FILE *f=stderr, char *pre=0, char *post=0) const;
        void printv (FILE *f=stderr);        
        
        private:

        struct StringData : public ref
        {
                StringData () : text_ (new unichar[1]) { *text_ = 0; }
                StringData (unichar *text) : text_ (text) { }
                ~StringData () { delete text_; }
                unichar *text_;
        };

        string (StringData *data) : data_ (data) { }

        lptr<StringData> data_;

        friend void printString (string &s);
        friend string Latin1 (const char *t);
        friend string Utf (const char *t);
};

void printString (string *s);
        

struct TranslateTable
{
        TranslateTable (string filename, int ul, int uh);
        ~TranslateTable ();

        string translate (string s);
        bool   handle (unichar c) { return (c>=unilow && c<=unihigh); }

        unichar *table;
        unichar unilow, unihigh;
        bool    no_mapping;
        TranslateTable (TranslateTable &t) { };
};

inline
string Latin1 (const char *t)
{
        long siz = strlen(t)+1;
        string s;
        s.use8bitData (t,siz);
        return s;
}


inline
string Utf (const char *t)
{
        long siz = utfstrlen(t)+1;
        unichar *us = new unichar [siz];
        UtfToUni (t, us);        
        string s;
        s.useUniData (us,siz);
        return s;        
}       


inline
string::string () : data_ (new StringData)
{
}

/*
inline
string::string (const char *t) : data_ (new StringData (new unichar[strlen(t)+1]))
{
        Latin1ToUni (t,data_->text_);
}

inline
string::string (char *t, int n) : data_ (new StringData (new unichar[n+1]))
{
        Latin1ToUnin (t,data_->text_, n);
}
*/
/*
inline
string::string (char *t) : data_ (new StringData (new unichar[utfstrlen(t)+1]))
{
        UtfToUni (t,data_->text_);
        delete t;
        fprintf (stderr, "Deleted non-const string\n");
}
*/
inline
int string::size () const
{ 
        return unistrlen (data_->text_);
}

inline
int string::storeAsUtf (char *buffer, int length) const
{
        if (length<=0 || length>size ()) length=size();
        return UniToUtfn (data_->text_,buffer,length);
}

inline
int string::storeAsLatin1 (char *buffer, int length) const
{
        if (length<=0 || length>size ()) length=size();
        return UniToLatin1n (data_->text_,buffer,length);
}

inline
int string::asInt () const
{
        char buffer[32];
        UniToUtf (data_->text_,buffer);
        return (atoi (buffer));
}

inline
string string::copy ()
{
        unichar *buffer = new unichar [size()+1];
        memcpy (buffer, data_->text_, (size()+1)*sizeof(unichar));
        return string (new string::StringData (buffer));
}

inline
string string::copyData (unichar *t, int l)
{
        unichar *buffer = new unichar [l+1];
        memcpy (buffer, t, l*sizeof(unichar));
        return string (new string::StringData (buffer));
}

inline
void string::useUniData (const unichar *t, long unsigned int l)
{
        unichar *buffer = new unichar [l+1];
        memcpy (buffer, t, l*sizeof(unichar));
        data_ = new string::StringData (buffer);
}

inline
void string::use8bitData (const char *t, long unsigned int l)
{
        unichar *buffer = new unichar [l];
        const unsigned char *tt = (const unsigned char*)t;
        for (long unsigned int i=0; i<l; ++i) buffer [i] = unichar(tt[i]);
        data_ = new string::StringData (buffer);
}

inline
string string::substring (int from, int to) const
{
        if (to==0 && from>0) to = size ();
        int l = to-from;
        if (l<1)
        {
                unichar *buffer = new unichar [1];
                buffer [0] = 0;
                return string (new string::StringData(buffer));
        }
        
        unichar *buffer = new unichar [l+1];
        memcpy (buffer, data_->text_+from, l*sizeof (unichar));
        buffer[l] = 0;

        return string (new string::StringData (buffer));
}

inline
int string::findSubstring (int from, string pattern) const
{
        unichar *t  = data_->text_;
        int      tl = size ();

        unichar *p  = pattern.data_->text_;
        int      pl = pattern.size ();
        int i,j;
        
        for (i=from; i<tl-pl+1; ++i)
        {
                if (t[i] == p[0])
                {
                        for (j=1; j<pl; ++j)
                        {
                                if (t[i+j]!=p[j]) break;
                        }
                        if (j==pl) return i;
                }
        }
        return -1;
}
 
inline
string string::replaceSubstring (string pattern,
				 string data, 
				 int from)
{
        int p = findSubstring (from, pattern);
        if (p==-1) return *this;

        return substring (0, p) + data + substring (p+pattern.size());
}

inline
string string::replaceSubstrings (string pattern,
				  string data)
{
        string tmp = *this;
        int p;

        for (;;)
        {
                p = tmp.findSubstring (0, pattern);
                if (p==-1) return tmp;
                tmp = tmp.substring (0, p) + data + tmp.substring (p+pattern.size());
        }
}

inline
void string::asString (int i) 
{
        data_ = new StringData (new unichar[32]);
        char buffer[32];
        sprintf (buffer,"%d",i);
        UtfToUni (buffer,data_->text_);
}

inline
void string::asString (float f) 
{
        data_ = new StringData (new unichar[32]);
        char buffer[32];
        sprintf (buffer,"%f",f);
        UtfToUni (buffer,data_->text_);
}

inline
void string::asString (double d) 
{
        data_ = new StringData (new unichar[64]);
        char buffer[64];
        sprintf (buffer,"%f",d);
        UtfToUni (buffer,data_->text_);
}

inline
void string::translate (int unilow,
			int unihigh,
			unichar *table)
{
        int l = size ();
        unichar *t = data_->text_;
        for (int i=0; i<l; ++i)
        {
                if (t[i]>=unilow && t[i]<=unihigh)
                {
                        t[i] = table[t[i]-unilow];
                }
        }
}

inline
string string::trim ()
{
        int l = size ();
        unichar *from = data_->text_;

        int start=0,stop=l;        
        while (start<l && (from[start]==' ' || from[start]=='\t')) ++start;        
        while (stop>start && (from[stop-1]==' ' || from[stop-1]=='\t')) --stop;

        unichar *to = new unichar[stop-start+1];        
        for (int i=start; i<stop; ++i) to[i-start]=from[i]; 
        to[stop-start] = 0;

        return string (new StringData (to));        
}

inline
string string::trimends ()
{
        int l = size ();
        unichar *from = data_->text_;

        int start=0, stop=l;        
        while (start<l && (from[start]==' ' || from[start]=='\t' || from[start]=='\n' || from[start]=='\r'))
                ++start;        
        while (stop>start && (from[stop-1]==' ' || from[stop-1]=='\t' || from[stop-1]=='\n' || from[stop-1]=='\r'))
                --stop;

        unichar *to = new unichar[stop-start+1];
        for (int i=start; i<stop; ++i) to[i-start]=from[i]; 
        to[stop-start] = 0;

        return string (new StringData (to));
}

inline
long string::hashValue () const
{
        unichar *c = data_->text_;
        long sum=0,last=0;        
        for (; *c != 0; ++c)
        {
                sum += last^*c;
                last ^= *c << 1;                
        }
        return sum;
}

inline
string::operator unichar * () const
{
        return data_->text_;
}

inline
unichar &string::operator [] (int n)
{
        return data_->text_[n];
}

inline
string operator + (string s, string t)
{
        int ss = s.size (),
	ts = t.size ();
        unichar *text = new unichar [ss+ts+1];
        memcpy (text,s.data_->text_,ss*sizeof(unichar));
        memcpy (text+ss,t.data_->text_,ts*sizeof(unichar));
        text[ss+ts] = 0;
        return string (new string::StringData (text));
}

inline
string operator + (char *s, string t)
{
        int ss = utfstrlen (s),
	ts = t.size ();
        unichar *text = new unichar [ss+ts+1];
        UtfToUni (s,text);
        memcpy (text+ss,t.data_->text_,ts*sizeof(unichar));
        text[ss+ts] = 0;
        return string (new string::StringData (text));
}

inline
string operator + (string s, char *t)
{
        int ts = utfstrlen (t),
	ss = s.size ();
        unichar *text = new unichar [ss+ts+1];
        memcpy (text,s.data_->text_,ss*sizeof(unichar));
        UtfToUni (t,text+ss);
        text[ss+ts] = 0;
        return string (new string::StringData (text));
}

inline
string operator + (string s, unichar t)
{
        int ss = s.size ();
        unichar *text = new unichar [ss+2];
        memcpy (text,s.data_->text_,ss*sizeof(unichar));
        text[ss] = t;
        text[ss+1] = 0;
        return string (new string::StringData (text));
}

inline
string operator + (unichar s, string t)
{
        int ts = t.size ();
        unichar *text = new unichar [ts+2];
        text[0] = s;
        memcpy (text+1,t.data_->text_,ts*sizeof(unichar));
        text[ts+1] = 0;
        return string (new string::StringData (text));
}

inline
bool operator == (string s, string t)
{
        unichar *a=s.data_->text_, *b=t.data_->text_;
        int i=0;
        for (; a[i] && b[i]; ++i)
        {
                if (a[i]!=b[i]) return false;
        }
        if (a[i]!=b[i]) return false;
        return true;
}

inline
bool operator != (string s, string t)
{
        return !(s==t);
}


bool operator < (string s, string t);

string substitute (string n,
                   string pre,
                   string post,
                   map<string,string> &keys);

string substituteChildren (string n,
                           map<string,string> &keys,
                           map<string,int> &ordering,
                           list<string> &childs);

/*
inline
ostream &operator << (ostream &os, string t)
{
        char buffer [unistrlenasutf(t.data_->text_)+1];
        UniToUtf (t.data_->text_, buffer);
        return os << buffer;
}

inline
istream &operator >> (istream &is, string &t)
{
        char buffer [1024];
        is >> buffer;
        t = string (buffer);
        return is;
}
*/



#endif






