// Copyright (C) 1999 Fredrik Ohrstrom
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License as published by the Free Software Foundation; either
//    version 2 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
//    02111-1307, USA

#ifndef _tcp_hh
#define _tcp_hh

#include<sys/types.h>
#include<sys/socket.h>
#include<netinet/in.h>
#include<string.h>
#include<sys/uio.h>
#include<unistd.h>
#include<signal.h>
#include<sys/ioctl.h>

#include<list>

#include "Component_Implementations.hh"

extern void renderResponse (char *in, char *out);
extern void parseRequest (char *in, map<string,string> &values);

namespace Tcp
{
        using namespace Component_Implementations;

        class Client;
        class TCPServer;
        class Handler;

        class Handler
        {
                public:
                
                virtual void handle (Client *c) = 0;
        };

        class Client : public Component_i
        {
                public:

                Client (TCPServer *server);
                void start (int fd);
                void stop ();

                int readdata (char *buffer, int n)
                {
                        return read (fd_, buffer, n);
                }

                int writedata (char *buffer, int n)
                {
                        return write (fd_, buffer, n);
                }

                void serveClient (Callback_i *cb);
                
                Semaphore nap_;                
                Thread    echo_thread_;
                int       fd_;
                TCPServer   *server_;
        };

        class TCPServer : public Component_i
        {
                public:
                
                TCPServer (int port);
                void dispatchClient (int fd);
                void returnClient (Client *c);
                void useHandler (Handler *h)
                {
                        handler_ = h;
                }
                Handler *handler () 
                {
                        return handler_;
                }
                
                void acceptConnections (Callback_i *cb);

                int nof_clients;
                Semaphore waiting_for_clients;
                Thread accept_thread;
                int sockfd, newsockfd, clilen;
                struct sockaddr_in cli_addr, serv_addr;
                Handler *handler_;
                
                Mutex freelist_protect;
                list<Client*> freelist;
        };

        string TranslateHtmlSensitiveChars (string s);
        string TranslateHtmlSensitiveCharsBack (string s);        
};

#endif

