// Copyright (C) 1999 Fredrik hrstrm
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//    USA.

#include<unistd.h>
#include<map>

#include"Window_Http_Implementations.hh"
#include"Interaction_Http_Implementations.hh"
#include"TCP_Implementations.hh"
 
using namespace TCP_Implementations;

using namespace Window_Http_Implementations;
using namespace Interaction_Http_Implementations;

InteractionHttpFactory_ptr ui = interaction_http_factory;
WindowHttpFactory_ptr gfx = window_http_factory;





// This is the actual class which contains the application logic
// The rest of this file is going to be placed in separate files.

class UIWebTest : public HttpPart_i
{
        public:

        AtomicGroup_ptr uiwebtest_group;
        AtomicMessage_ptr header, welcome;
        
        UIWebTest (Component_ptr composite,
                   Group_ptr g)
                        : HttpPart_i (composite, Latin1("UIWebTest"), g),
                          ClassType (Latin1("UIWebTestApplication"))
        {
                uiwebtest_group = ui->newAtomicGroup (this, Latin1("UIWebTest_group"), parent ());
                header = ui->newAtomicMessage (this, Latin1("header"), uiwebtest_group);
                welcome = ui->newAtomicMessage (this, Latin1("welcome"), uiwebtest_group);                
                welcome->parent ()->focus (welcome);
                
                uiwebtest_group->activate ();
        }

};





///////////////////////////////////////////////////////////////////////////////////


class HttpPartEditor : public HttpPart_i
{
        public:

        Group_ptr editgroup;
        TextChoice_ptr component_path_view;
        AtomicFlowControl_ptr goto_component, resource_appearance, resource_content;

        TextChoice_ptr resource_path_view;
        AtomicFlowControl_ptr goto_resource;
        TextMessage_ptr possible_resources;

        TextEdit_ptr component_code;
        AtomicFlowControl_ptr save_changes, ignore_changes;

        TextModel_ptr model;

        string component_path;
        string resource_path;
        HttpPart_ptr top;
        Part_ptr part;
        
        HttpPartEditor (Component_ptr c,
                        string n,
                        Group_ptr g,
                        HttpPart_ptr t)
                        : HttpPart_i (c,n,g), ClassType (Latin1("HttpPartEditor")), top (t)
        {
                model = new TextModel_i;
                editgroup = g;
                component_path_view = ui->newTextChoice (this, Latin1("component_path_view"), editgroup);
                goto_component = ui->newAtomicFlowControl (this, Latin1("goto_component"), editgroup);
                goto_component->onAction (call (this, &HttpPartEditor::gotoPart));
                resource_appearance = ui->newAtomicFlowControl (this, Latin1("resource_appearance"), editgroup);
                resource_appearance->onAction (call (this, &HttpPartEditor::resourceAppearance));
                resource_content = ui->newAtomicFlowControl (this, Latin1("resource_content"), editgroup);
                resource_content->onAction (call (this, &HttpPartEditor::resourceContent));
                
                resource_path_view = ui->newTextChoice (this, Latin1("resource_path_view"), editgroup);
                goto_resource = ui->newAtomicFlowControl (this, Latin1("goto_resource"), editgroup);
                goto_resource->onAction (call (this, &HttpPartEditor::gotoResource));

                possible_resources = ui->newTextMessage (this, Latin1("possible_resources"), editgroup);
                
                component_code = ui->newTextEdit (this, Latin1("component_code"), editgroup, model);
                setPath (true, path()+"/save_changes(HttpAtomicFlowControl)/content(String)");

                save_changes = ui->newAtomicFlowControl (this, Latin1("save_changes"), editgroup);
                save_changes->onAction (call (this, &HttpPartEditor::saveChanges));
                ignore_changes = ui->newAtomicFlowControl (this, Latin1("ignore_changes"), editgroup);
                ignore_changes->onAction (call (this, &HttpPartEditor::ignoreChanges));

                editgroup->activate ();
        }
        
        void ignoreChanges (Callback_i *cb)
        {
                editgroup->parent()->unfocus (editgroup);
        }

        void saveChanges (Callback_i *cb)
        {
                resource_path = resource_path_view->getText ();
                string re = model->asString (Latin1("\n"));
                Component_Implementations::resources->setResource (resource_path, re);
                editgroup->parent()->unfocus (editgroup);
                top->reloadAppearance ();
                HttpPart_ptr pp = dynamic_cast<HttpPart_ptr> (part);
                if (pp!=NULL) pp->reloadAppearance ();
        }

        void gotoPart (Callback_i *cb)
        {
                setPath (true,component_path_view->getText ());                
        }

        void gotoResource (Callback_i *cb)
        {
                setPath (false, resource_path_view->getText ());
        }

        void resourceAppearance (Callback_i *cb)
        {
                setPath (true, part->path()+Latin1("/appearance(String)"));
        }

        void resourceContent (Callback_i *cb)
        {
                setPath (true, part->path()+Latin1("/content(String)"));
        }        

        void setPath (bool direct, string val)
        {
                string data, possibles;
                if (direct==true)
                {
                        component_path = val;
                        resource_path = Component_Implementations::resources->whichResource (component_path);
                        data = Component_Implementations::resources->getResource (component_path);
                        list<string> pr = Component_Implementations::resources->getResources (component_path);
                        list<string>::iterator i;
                        for (i=pr.begin(); i!=pr.end(); ++i)
                        {
                                possibles = possibles+(*i)+"<br>";
                        }
                        if (possibles.size()==0)
                        {
                                possibles = Latin1 (" ");
                                resource_path = Latin1 (" ");
                                data = Latin1 (" ");
                        }
                }
                else
                {
                        resource_path = val;                        
                        component_path = Latin1 (" ");
                        possibles = Latin1 (" ");
                        data = Component_Implementations::resources->getExactResource (resource_path);
                        if (data.size()==0)
                        {
                                data = Latin1 (" ");
                        }
                }
                
                // Update interface /////////////////////
                component_path_view->setText (component_path);
                resource_path_view->setText (resource_path);
                possible_resources->setText (possibles);
                model->fromString (data);
                component_code->updateModel ();
        }
                
        void editPart (Part_ptr p, string postfix)
        {
                setPath (true, p->path()+postfix);
                string s = p->path()+postfix;
                s.print (stderr, "Should edit >", "<\n");
                component_path.print (stderr, "Does edit >", "<\n");
                
                editgroup->parent()->focus (editgroup);
                part = p;
        }
};

class HttpSystem : public Component_i
{
        public:

        Group_ptr system_group, info_group, application_group, editor_group;
        AtomicMessage_ptr system_tag;
        AtomicFlowControl_ptr goto_info, goto_editor, goto_site, save_ui;
        HttpPartEditor *html_editor;
        
        HttpSystem (Component_ptr c,
                    Group_ptr g)
                        : Component_i (c,Latin1("http_system")), ClassType (Latin1("HttpSystem"))
        {
                // System Group
                system_group = ui->newAtomicGroup (this, Latin1("system_group"), g);
                goto_info = ui->newAtomicFlowControl (this, Latin1("goto_info"), system_group);
                goto_info->onAction (call (this, &HttpSystem::gotoInfo));
                goto_editor = ui->newAtomicFlowControl (this, Latin1("goto_editor"), system_group);
                goto_editor->onAction (call (this, &HttpSystem::gotoEditor));
                goto_site = ui->newAtomicFlowControl (this, Latin1("goto_site"), system_group);
                goto_site->onAction (call (this, &HttpSystem::gotoSite));
                save_ui = ui->newAtomicFlowControl (this, Latin1("save_ui"), system_group);
                save_ui->onAction (call (this, &HttpSystem::saveUI));                
                
                // Info subgroup
                info_group = ui->newAtomicGroup (this, Latin1("info_group"), system_group);
                system_tag = ui->newAtomicMessage (this, Latin1("system_tag"), info_group);
                info_group->activate ();
                
                // Editor subgroup
                editor_group = ui->newAtomicGroup (this, Latin1("editor_group"), system_group);
                html_editor = new HttpPartEditor (this, Latin1("html_editor"), editor_group,
                                                  dynamic_cast<HttpPart_ptr>(g));
                g->useEditor (html_editor);
                
                // Application subgroup
                application_group = ui->newAtomicGroup (this, Latin1("application_group"), system_group);
                application_group->activate ();
                
                gotoInfo (NULL);
                system_group->activate ();
        }
        
        void gotoInfo (Callback_i *cb)
        {
                system_group->focus (info_group);
        }

        void gotoEditor (Callback_i *cb)
        {
                system_group->focus (editor_group);                
        }

        void saveUI (Callback_i *cb)
        {
                Component_Implementations::resources->saveToFile ("uiwebtest.ny");
        }
        
        void gotoSite (Callback_i *cb)
        {
                system_group->focus (application_group);                
        }
        
        Group_ptr applicationGroup ()
        {
                return application_group;
        }
};



class WebClient : public Component_i
{
        public:

        WebClient (Component_ptr composite,
                   string name)
                        : Component_i (composite, name), ClassType (Latin1("WebClient"))
        {}
};

class Webserver : public Component_i
{
        HttpGraphicsServer_ptr display;
        map<string,WebClient*> clients;
        int client_no;
        
        public:

        Webserver ()
                        : Component_i (NULL, Latin1("startup")), ClassType (Latin1("Startup"))
        {
                client_no = 1;
                display = gfx->newHttpGraphicsServer (NULL, Latin1("display"), 4000);
                display->onConnectRequest (call (this, &Webserver::handleconnect));
                display->onDisconnectRequest (call (this, &Webserver::handledisconnect));
        }

        void handleconnect (HttpConnectRequest_i *cb)
        {
                fprintf (stderr, "Received connect request!\n");
                HttpGraphicsServer_ptr d = cb->getHttpGraphicsServer ();
                string id;
                id.asString (client_no++);
                id = Latin1("s")+id+Latin1("s");
                HttpScreen_ptr screen = gfx->newHttpScreen (d, Latin1("screen"), id);
                cb->setHttpScreen (screen);
                
                WebClient *c = new WebClient (this, id);
                Group_ptr group = ui->newHttpRootGroup (c, Latin1("webroot"), screen->getRootWindow ());
                string app = cb->getApplication ();
                if (app==Latin1("system"))
                {
                        fprintf (stderr, "Created system group!\n");
                        group->useEditor (group);
                        HttpSystem *s = new HttpSystem (c, group);
                        UIWebTest *w = new UIWebTest (c, s->applicationGroup());
                }
                else
                {
                        fprintf (stderr, "Created uiwebtest object!\n");
                        UIWebTest *w = new UIWebTest (c, group);
                        app.print (stderr, "APP=","\n");
                        string ap = app.substring (0,10);
                        ap.print (stderr, "AP=","\n");
                }

                group->activate ();
                clients[id] = c;
        }

        void handledisconnect (HttpConnectRequest_i *cb)
        {
                fprintf (stderr, "Received disconnect request!\n");
                HttpScreen_ptr screen = cb->getHttpScreen ();
                screen->id().print (stderr, "ID=","\n");

                /*
                map<string,WebClient*>::iterator i;
                i = clients.find (screen->id());
                if (i!=clients.end())
                {
                        WebClient *client = (*i).second;
                        clients.erase (i);
                        client->cleanup()->_dispose ();
                        screen->cleanup()->_dispose ();
                }
                */
        }
};

int main (int argc, char **argv)
{
        useResourceFile ("uiwebtest.res");
        
        Webserver s;

        for (;;) sleep (120);
}





        

        
        
        
        
