// Program: XTest
// Purpose: Test the Window_X_Interfaces and Implementations.
//
// Copyright (C) 1999 Fredrik hrstrm
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//    USA.

#include"Component_Implementations.hh"
#include"Window_X_Implementations.hh"

using namespace Component_Implementations;
using namespace Window_X_Implementations;

WindowXFactory_ptr gfx = window_x_factory;

struct XTest : public Component_Implementations::Component_i
{
        XScreen_ptr screen;
        XWindow_ptr root;
        XWindow_ptr window;
        XWindow_ptr button;
        Orientation foor;
        XFont_ptr   font;
        string      textdata;
        XText_ptr   text;
        XGC_ptr     gc, gcwhite;

        XTest ()
                        : Component_i (0, Latin1("XTest")), ClassType (Latin1("Applet"))
        {
        using namespace Component_Implementations;
        using namespace Window_X_Implementations;        

                
                screen = gfx->getScreen ();
                gc = gfx->newGC (this, screen);
                gcwhite = gfx->newGC (this, screen);
                gcwhite->setForeground (0xffffffff);
                gcwhite->setBackground (0);
                foor.scale (1,1);
                font = gfx->newFont (this, Latin1("forall"), foor);
                
                root = screen->getRootWindow ();
                window = gfx->newXWindow (this, Latin1("window"),
                                                root,
                                                0, 0,
                                                512, 512);
                
                window->onExpose (call (this, &XTest::renderWindow));
                window->onButtonPress (call (this, &XTest::pressed));

                button = gfx->newXWindow (this, Latin1("button"),
                                                window,
                                                100, 100, 300, 22);
                
                button->onExpose (call (this, &XTest::renderButton));
                button->onButtonPress (call (this, &XTest::bpressed));
                button->onKeyPress (call (this, &XTest::keypressed));                
                
                button->mapAll ();
                
                window->mapAll ();
                
                gfx->sync();
                
                /*                fprintf (stderr, "---------------------\n");                
                printTree ();
                fprintf (stderr, "---------------------\n"); */
        }

        void renderWindow (XCallback_i *cb)
        {
                window->drawRectangle (gc, 0, 0, window->width()-1,
                                       window->height()-1);
                window->drawRectangle (gc, 1, 1, window->width()-3,
                                       window->height()-3);
                window->drawRectangle (gc, 2, 2, window->width()-5,
                                       window->height()-5);
                gfx->sync ();                
        }

        void renderButton (XCallback_i *cb)
        {                
                button->drawRectangle (gc, 0, 0, button->width()-1,
                                       button->height()-1);
		button->fillRectangle (gcwhite,
                                       1, 1, button->width()-2,
                                       button->height()-2);

                text = gfx->newText (this);
                text->addItem (textdata, font);
                
                button->drawText (gc, text, 6,2+text->height()-1,foor);
                        
                gfx->sync ();                
        }

        void recurse (int x1, int y1, int w, int h)
        {
                if (w<16 || h<16)
                {
                        window->drawRectangle (gc, x1+2, y1+2, w-4, h-4);
                        return;                        
                }                
                                
                recurse (x1, y1, w/2, h/2);
                recurse (x1+w/2, y1, w/2, h/2);
                recurse (x1, y1+h/2, w/2, h/2);
                recurse (x1+w/2, y1+h/2, w/2, h/2);                
        }        
        
        void pressed (XCallback_i *cb)
        {
                cerr << "Pressed at (" << cb->x() << "x" << cb->y() << ")" << endl;
                window->fillRectangle (gc, cb->x()-3, cb->y()-3, 6, 6);
                gfx->sync();                
        }

        void keypressed (XCallback_i *cb)
        {
                unichar ascii  = gfx->keycodeToUnichar (cb->event.xkey.keycode);
                cerr << "Keypressed with state=" << cb->event.xkey.state
                     << " keycode=" << cb->event.xkey.keycode
                     << " unicode=" <<  ascii <<  endl;
                if (ascii==8)
                        textdata = textdata.substring (0, textdata.size()-1);
                else                
                        textdata = textdata+ascii;
                renderButton (cb);
        }
        
        void bpressed (XCallback_i *cb)
        {
                cerr << "TJO!" << endl;
        }        
};

XScreen_ptr pp;


int main (int argc, char **argv)
{
        useResourceFile ("xtest.res");
        XTest *t = new XTest;
        //        t->printTree (); 

        sleep (1000);
}









